﻿/*  
     This file is part of Emulators Organizer
    A program that organize emulators and roms.
   
    Copyright (C) 2009 - 2012  Ali Hadid, programmed by Ala Hadid

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Windows.Forms;
using System.Drawing;

namespace AHD.Emulators_Organizer.Base
{
    public class RomComparer : IComparer<Rom>
    {
        bool AtoZ = true;
        EmulatorsOrganizer.SortMode _sortMode = EmulatorsOrganizer.SortMode.Name;
        public RomComparer(bool AtoZ, EmulatorsOrganizer.SortMode sortMode)
        {
            _sortMode = sortMode;
            this.AtoZ = AtoZ;
        }
        public int Compare(Rom x, Rom y)
        {
            switch (_sortMode)
            {
                case EmulatorsOrganizer.SortMode.Name:
                    if (AtoZ)
                        return (StringComparer.Create(System.Threading.Thread.CurrentThread.CurrentCulture, false)).Compare(x.Name, y.Name);
                    else
                        return (-1 * (StringComparer.Create(System.Threading.Thread.CurrentThread.CurrentCulture, false)).Compare(x.Name, y.Name));
                case EmulatorsOrganizer.SortMode.FileType:
                    if (AtoZ)
                        return (StringComparer.Create(System.Threading.Thread.CurrentThread.CurrentCulture, false)).Compare(Path.GetExtension(x.Path), Path.GetExtension(y.Path));
                    else
                        return (-1 * (StringComparer.Create(System.Threading.Thread.CurrentThread.CurrentCulture, false)).Compare(Path.GetExtension(x.Path), Path.GetExtension(y.Path)));
                case EmulatorsOrganizer.SortMode.Path:
                    if (AtoZ)
                        return (StringComparer.Create(System.Threading.Thread.CurrentThread.CurrentCulture, false)).Compare(x.Path, y.Path);
                    else
                        return (-1 * (StringComparer.Create(System.Threading.Thread.CurrentThread.CurrentCulture, false)).Compare(x.Path, y.Path));
                case EmulatorsOrganizer.SortMode.PlayedTime:
                    if (AtoZ)
                        return (StringComparer.Create(System.Threading.Thread.CurrentThread.CurrentCulture, false)).Compare(x.PlayedTimes, y.PlayedTimes);
                    else
                        return (-1 * (StringComparer.Create(System.Threading.Thread.CurrentThread.CurrentCulture, false)).Compare(x.PlayedTimes, y.PlayedTimes));
                case EmulatorsOrganizer.SortMode.Rating:
                    if (AtoZ)
                        return (StringComparer.Create(System.Threading.Thread.CurrentThread.CurrentCulture, false)).Compare(x.Rating, y.Rating);
                    else
                        return (-1 * (StringComparer.Create(System.Threading.Thread.CurrentThread.CurrentCulture, false)).Compare(x.Rating, y.Rating));
                case EmulatorsOrganizer.SortMode.Size:
                    if (AtoZ)
                        return (StringComparer.Create(System.Threading.Thread.CurrentThread.CurrentCulture, false)).Compare(x.RomSize, y.RomSize);
                    else
                        return (-1 * (StringComparer.Create(System.Threading.Thread.CurrentThread.CurrentCulture, false)).Compare(x.RomSize, y.RomSize));
            }
            return 0;
        }
    }
    public class PlaylistItemComparer : IComparer<PlayListItem>
    {
        bool AtoZ = true;
        EmulatorsOrganizer.SortMode _sortMode = EmulatorsOrganizer.SortMode.Name;
        public PlaylistItemComparer(bool AtoZ, EmulatorsOrganizer.SortMode sortMode)
        {
            _sortMode = sortMode;
            this.AtoZ = AtoZ;
        }
        public int Compare(PlayListItem x, PlayListItem y)
        {
            switch (_sortMode)
            {
                case EmulatorsOrganizer.SortMode.Name:
                    if (AtoZ)
                        return (StringComparer.Create(System.Threading.Thread.CurrentThread.CurrentCulture, false)).Compare(x.Rom.Name, y.Rom.Name);
                    else
                        return (-1 * (StringComparer.Create(System.Threading.Thread.CurrentThread.CurrentCulture, false)).Compare(x.Rom.Name, y.Rom.Name));
                case EmulatorsOrganizer.SortMode.FileType:
                    if (AtoZ)
                        return (StringComparer.Create(System.Threading.Thread.CurrentThread.CurrentCulture, false)).Compare(Path.GetExtension(x.Rom.Path), Path.GetExtension(y.Rom.Path));
                    else
                        return (-1 * (StringComparer.Create(System.Threading.Thread.CurrentThread.CurrentCulture, false)).Compare(Path.GetExtension(x.Rom.Path), Path.GetExtension(y.Rom.Path)));
                case EmulatorsOrganizer.SortMode.Path:
                    if (AtoZ)
                        return (StringComparer.Create(System.Threading.Thread.CurrentThread.CurrentCulture, false)).Compare(x.Rom.Path, y.Rom.Path);
                    else
                        return (-1 * (StringComparer.Create(System.Threading.Thread.CurrentThread.CurrentCulture, false)).Compare(x.Rom.Path, y.Rom.Path));
                case EmulatorsOrganizer.SortMode.PlayedTime:
                    if (AtoZ)
                        return (StringComparer.Create(System.Threading.Thread.CurrentThread.CurrentCulture, false)).Compare(x.Rom.PlayedTimes, y.Rom.PlayedTimes);
                    else
                        return (-1 * (StringComparer.Create(System.Threading.Thread.CurrentThread.CurrentCulture, false)).Compare(x.Rom.PlayedTimes, y.Rom.PlayedTimes));
                case EmulatorsOrganizer.SortMode.Rating:
                    if (AtoZ)
                        return (StringComparer.Create(System.Threading.Thread.CurrentThread.CurrentCulture, false)).Compare(x.Rom.Rating, y.Rom.Rating);
                    else
                        return (-1 * (StringComparer.Create(System.Threading.Thread.CurrentThread.CurrentCulture, false)).Compare(x.Rom.Rating, y.Rom.Rating));
                case EmulatorsOrganizer.SortMode.Size:
                    if (AtoZ)
                        return (StringComparer.Create(System.Threading.Thread.CurrentThread.CurrentCulture, false)).Compare(x.Rom.RomSize, y.Rom.RomSize);
                    else
                        return (-1 * (StringComparer.Create(System.Threading.Thread.CurrentThread.CurrentCulture, false)).Compare(x.Rom.RomSize, y.Rom.RomSize));
            }
            return 0;
        }
    }

    public class ConsoleComparer : IComparer<EmuConsole>
    {
        bool AtoZ = true;
        public ConsoleComparer(bool AtoZ)
        {
            this.AtoZ = AtoZ;
        }
        public int Compare(EmuConsole x, EmuConsole y)
        {
            if (AtoZ)
                return (StringComparer.Create(System.Threading.Thread.CurrentThread.CurrentCulture, false)).Compare(x.Name, y.Name);
            else
                return (-1 * (StringComparer.Create(System.Threading.Thread.CurrentThread.CurrentCulture, false)).Compare(x.Name, y.Name));
        }
    }
    public class PlaylistComparer : IComparer<PlayList>
    {
        bool AtoZ = true;
        public PlaylistComparer(bool AtoZ)
        {
            this.AtoZ = AtoZ;
        }
        public int Compare(PlayList x, PlayList y)
        {
            if (AtoZ)
                return (StringComparer.Create(System.Threading.Thread.CurrentThread.CurrentCulture, false)).Compare(x.Name, y.Name);
            else
                return (-1 * (StringComparer.Create(System.Threading.Thread.CurrentThread.CurrentCulture, false)).Compare(x.Name, y.Name));
        }
    }
    public class ListViewItemComparer : IComparer
    {
        private int col;
        private bool AtoZ;
        public ListViewItemComparer(int column,bool AtoZ)
        {
            this.AtoZ = AtoZ;
            col = column;
        }
        public int Compare(object x, object y)
        {
            return AtoZ? String.Compare(((ListViewItem)x).SubItems[col].Text, ((ListViewItem)y).SubItems[col].Text):
             (String.Compare(((ListViewItem)x).SubItems[col].Text, ((ListViewItem)y).SubItems[col].Text) * -1);
        }
    }

}
