﻿/*  
     This file is part of Emulators Organizer
    A program that organize emulators and roms.
   
    Copyright (C) 2009 - 2012  Ali Hadid, programmed by Ala Hadid

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Xml;
using System.Xml.Serialization;
using AHD.Emulators_Organizer.Base;
using EmulatorsOrganizer;

namespace EmulatorsOrganizer.Controls
{
    public partial class CommandlinesEditor : UserControl
    {
        bool _DirectApply = false;
        public CommandlinesEditor()
        {
            InitializeComponent();
        }
        /// <summary>
        /// Get or set if the user can edit commandlines
        /// </summary>
        public bool EditMode
        { 
            get { return toolStrip1.Visible; } 
            set 
            {
                toolStrip1.Visible = value;
                if (!value)
                    treeView1.ContextMenuStrip = null;
                else
                    treeView1.ContextMenuStrip = contextMenuStrip1;
            } 
        }
        public bool DirectApply
        {
            get { return _DirectApply; }
            set { _DirectApply = value; }
        }
        void Apply()
        {
            _CommandLinesGroups.Clear();
            foreach (TreeNode_CommandLinesGroup GR in treeView1.Nodes)
            {
                _CommandLinesGroups.Add(GR.CommandLinesGroup);
            }
        }
        List<CommandLinesGroup> _CommandLinesGroups = new List<CommandLinesGroup>();
        /// <summary>
        /// Get or set the command line groups
        /// </summary>
        public List<CommandLinesGroup> CommandLinesGroups
        {
            get
            {
                Apply();
                return _CommandLinesGroups;
            }
            set
            {
                _CommandLinesGroups = value;
                if (_CommandLinesGroups == null)
                    _CommandLinesGroups = new List<CommandLinesGroup>();
                Refreshcommandlines();
            }
        }
        void Refreshcommandlines()
        {
            treeView1.Nodes.Clear();
            bool found = false;
            foreach (CommandLinesGroup gr in _CommandLinesGroups)
            {
                if (gr.Name == "<Default>")
                { found = true; break; }
            }
            if (!found)
            {
                CommandLinesGroup gro = new CommandLinesGroup();
                CommandLine cm = new CommandLine();
                cm.commandLine = "<RomPath>";
                cm.Name = "Rom Path";
                cm.Enabled = true;

                gro.Name = "<Default>";
                gro.Enabled = true;
                gro.CommandLines.Add(cm);
                _CommandLinesGroups.Add(gro);
            }
            foreach (CommandLinesGroup group in _CommandLinesGroups)
            {
                TreeNode_CommandLinesGroup gr = new TreeNode_CommandLinesGroup();
                gr.CommandLinesGroup = group;
                gr.SelectedImageIndex = 0;
                gr.ImageIndex = 1;
                gr.Checked = group.Enabled;
                gr.RefreshCommandLines(2, 2);
                treeView1.Nodes.Add(gr);
            }
        }

        private void treeView1_AfterCheck(object sender, TreeViewEventArgs e)
        {
            if (e.Node == null)
                return;
            if (e.Node.GetType() == typeof(TreeNode_CommandLinesGroup))
            {
                TreeNode_CommandLinesGroup gr = (TreeNode_CommandLinesGroup)e.Node;
                if (gr.Text == "<Default>" & !e.Node.Checked)
                {
                    MessageBox.Show("You can't disable this group");
                    e.Node.Checked = true;
                    return;
                }
                ((TreeNode_CommandLinesGroup)e.Node).CommandLinesGroup.Enabled = e.Node.Checked;
            }
            else if (e.Node.GetType() == typeof(TreeNode_CommandLine))
            {
                TreeNode_CommandLinesGroup gr = (TreeNode_CommandLinesGroup)e.Node.Parent;
                if (gr.Text == "<Default>" & !e.Node.Checked)
                {
                    MessageBox.Show("You can't disable this commandline");
                    e.Node.Checked = true;
                    return;
                }
                ((TreeNode_CommandLine)e.Node).CommandLine.Enabled = e.Node.Checked;
            }
            else if (e.Node.GetType() == typeof(TreeNode_Parameter))
            {
                TreeNode_CommandLinesGroup gr = (TreeNode_CommandLinesGroup)e.Node.Parent.Parent;
                if (gr.Text == "<Default>" & !e.Node.Checked)
                {
                    MessageBox.Show("You can't disable this parameter");
                    e.Node.Checked = true;
                    return;
                }
                ((TreeNode_Parameter)e.Node).Parameter.Enabled = e.Node.Checked;
            }
        }

        private void addGroupToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Frm_EnterName name = new Frm_EnterName("Enter the name of the new group", "Group");
            if (name.ShowDialog(this) == DialogResult.OK)
            {
                foreach (TreeNode tree in treeView1.Nodes)
                {
                    if (tree.Text.ToLower() == name.NameEntered.ToLower())
                    {
                        MessageBox.Show("This group already exists");
                        return;
                    }
                }
                TreeNode_CommandLinesGroup tr = new TreeNode_CommandLinesGroup();
                CommandLinesGroup gr = new CommandLinesGroup();
                gr.Name = name.NameEntered;
                gr.Enabled = true;

                tr.CommandLinesGroup = gr;
                tr.SelectedImageIndex = 0;
                tr.ImageIndex = 1;
                tr.Checked = true;

                treeView1.Nodes.Add(tr);

                if (_DirectApply)
                    Apply();
            }
        }

        private void treeView1_AfterSelect(object sender, TreeViewEventArgs e)
        {
            if (treeView1.SelectedNode == null)
            {
                editToolStripMenuItem.Enabled = deleteToolStripMenuItem.Enabled = toolStripButton3.Enabled = toolStripButton1.Enabled = false;
                return;
            }
            editToolStripMenuItem.Enabled = deleteToolStripMenuItem.Enabled = toolStripButton3.Enabled = toolStripButton1.Enabled = true;
            addCommandlineToolStripMenuItem1.Enabled = addCommandlineToolStripMenuItem.Enabled = treeView1.SelectedNode.GetType() == typeof(TreeNode_CommandLinesGroup);
            addParameterToolStripMenuItem1.Enabled = addParameterToolStripMenuItem.Enabled = treeView1.SelectedNode.GetType() == typeof(TreeNode_CommandLine);
        }

        private void addCommandlineToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (treeView1.SelectedNode == null)
                return;

            TreeNode_CommandLinesGroup gr = (TreeNode_CommandLinesGroup)treeView1.SelectedNode;
            if (gr.Text == "<Default>")
            {
                MessageBox.Show("You can't add commandline to the default commandlines group");
                return;
            }

            Frm_Commandline name = new Frm_Commandline("", "", true);
            name.Text = "Add commandline";
            if (name.ShowDialog(this) == System.Windows.Forms.DialogResult.OK)
            {
                CommandLine cm = new CommandLine();
                cm.commandLine = name.CodeEntered;
                cm.Name = name.NameEntered;
                cm.Enabled = true;

                gr.CommandLinesGroup.CommandLines.Add(cm);
                gr.RefreshCommandLines(2, 2);
                gr.Expand(); 
                if (_DirectApply)
                    Apply();
            }
        }

        private void addParameterToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (treeView1.SelectedNode == null)
                return;

            TreeNode_CommandLinesGroup gr = (TreeNode_CommandLinesGroup)treeView1.SelectedNode.Parent;
            if (gr.Text == "<Default>")
            {
                MessageBox.Show("You can't add parameter to a commandline in the default commandlines group");
                return;
            }

            Frm_Commandline name = new Frm_Commandline("", "", false);
            name.Text = "Add parameter";
            if (name.ShowDialog(this) == DialogResult.OK)
            {
                TreeNode_CommandLine cm = (TreeNode_CommandLine)treeView1.SelectedNode;
                Parameter par = new Parameter();
                par.parameter = name.CodeEntered;
                par.Enabled = true;

                cm.CommandLine.Parameters.Add(par);
                cm.RefreshParameters(2, 2);
                cm.Expand();
                if (_DirectApply)
                    Apply();
            }
        }
        //delete
        private void toolStripButton1_Click(object sender, EventArgs e)
        {
            if (treeView1.SelectedNode == null)
                return;

            if (treeView1.SelectedNode.GetType() == typeof(TreeNode_CommandLinesGroup))
            {
                if (treeView1.SelectedNode.Text == "<Default>")
                {
                    MessageBox.Show("You can't remove the default commandlines group");
                    return;
                }
                if (MessageBox.Show("Are you sure ?", "Delete commandlines group", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == System.Windows.Forms.DialogResult.Yes)
                {
                    treeView1.Nodes.Remove(treeView1.SelectedNode);
                    if (_DirectApply)
                        Apply();
                }
            }
            else if (treeView1.SelectedNode.GetType() == typeof(TreeNode_CommandLine))
            {
                if (treeView1.SelectedNode.Parent.Text == "<Default>")
                {
                    MessageBox.Show("You can't remove this commandline, it belongs to the default group");
                    return;
                }
                if (MessageBox.Show("Are you sure ?", "Delete commandline", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == System.Windows.Forms.DialogResult.Yes)
                {
                    TreeNode_CommandLinesGroup gr = (TreeNode_CommandLinesGroup)treeView1.SelectedNode.Parent;
                    gr.CommandLinesGroup.CommandLines.Remove(((TreeNode_CommandLine)treeView1.SelectedNode).CommandLine);
                    gr.Nodes.Remove(treeView1.SelectedNode);
                    if (_DirectApply)
                        Apply();
                }
            }
            else if (treeView1.SelectedNode.GetType() == typeof(TreeNode_Parameter))
            {
                if (MessageBox.Show("Are you sure ?", "Delete parameter", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == System.Windows.Forms.DialogResult.Yes)
                {
                    TreeNode_CommandLine gr = (TreeNode_CommandLine)treeView1.SelectedNode.Parent;
                    gr.CommandLine.Parameters.Remove(((TreeNode_Parameter)treeView1.SelectedNode).Parameter);
                    gr.Nodes.Remove(treeView1.SelectedNode);
                    if (_DirectApply)
                        Apply();
                }
            }
        }
        //edit
        private void toolStripButton3_Click(object sender, EventArgs e)
        {
            if (treeView1.SelectedNode == null)
                return;
            if (treeView1.SelectedNode.GetType() == typeof(TreeNode_CommandLinesGroup))
            {
                TreeNode_CommandLinesGroup gr = (TreeNode_CommandLinesGroup)treeView1.SelectedNode;
                if (gr.Text == "<Default>")
                {
                    MessageBox.Show("You can't edit the default commandlines group");
                    return;
                }

                Frm_EnterName name = new Frm_EnterName("Edit commandlines group", ((TreeNode_CommandLinesGroup)treeView1.SelectedNode).CommandLinesGroup.Name);
                if (name.ShowDialog(this) == DialogResult.OK)
                {
                    foreach (TreeNode tree in treeView1.Nodes)
                    {
                        if (tree != treeView1.SelectedNode)
                            if (tree.Text == name.NameEntered)
                            {
                                MessageBox.Show("This group already exists");
                                return;
                            }
                    }
                    treeView1.SelectedNode.Text = name.NameEntered;
                    ((TreeNode_CommandLinesGroup)treeView1.SelectedNode).CommandLinesGroup.Name = name.NameEntered;
                }
            }
            else if (treeView1.SelectedNode.GetType() == typeof(TreeNode_CommandLine))
            {
                TreeNode_CommandLinesGroup gr = (TreeNode_CommandLinesGroup)treeView1.SelectedNode.Parent;
                if (gr.Text == "<Default>")
                {
                    MessageBox.Show("You can't edit a commandline from the default commandlines group");
                    return;
                }
                Frm_Commandline name = new Frm_Commandline(((TreeNode_CommandLine)treeView1.SelectedNode).CommandLine.Name, ((TreeNode_CommandLine)treeView1.SelectedNode).CommandLine.commandLine, true);
                name.Text = "Edit commandline";
                if (name.ShowDialog(this) == System.Windows.Forms.DialogResult.OK)
                {
                    ((TreeNode_CommandLine)treeView1.SelectedNode).CommandLine.Name = name.NameEntered;
                    ((TreeNode_CommandLine)treeView1.SelectedNode).CommandLine.commandLine = name.CodeEntered;
                    ((TreeNode_CommandLine)treeView1.SelectedNode).RefreshName();
                }
            }
            else if (treeView1.SelectedNode.GetType() == typeof(TreeNode_Parameter))
            {
                TreeNode_CommandLinesGroup gr = (TreeNode_CommandLinesGroup)treeView1.SelectedNode.Parent.Parent;
                if (gr.Text == "<Default>")
                {
                    MessageBox.Show("You can't edit a parameter from the default commandlines group");
                    return;
                }
                ((TreeNode_Parameter)treeView1.SelectedNode).Parameter.Enabled = treeView1.SelectedNode.Checked;

                Frm_Commandline name = new Frm_Commandline("", ((TreeNode_Parameter)treeView1.SelectedNode).Parameter.parameter, false);
                name.Text = "Edit parameter";
                if (name.ShowDialog(this) == DialogResult.OK)
                {
                    ((TreeNode_Parameter)treeView1.SelectedNode).Parameter.parameter = name.CodeEntered;
                    ((TreeNode_Parameter)treeView1.SelectedNode).RefreshName();

                }
            }
            if (_DirectApply)
                Apply();
        }

        private void toolStripButton2_Click(object sender, EventArgs e)
        {
            if (MessageBox.Show("Are you sure ?\n\nThis cannot be undone", "Clear commandlines", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == System.Windows.Forms.DialogResult.Yes)
            {
                treeView1.Nodes.Clear();
                _CommandLinesGroups.Clear();
                CommandLinesGroup gro = new CommandLinesGroup();
                CommandLine cm = new CommandLine();
                cm.commandLine = "<RomPath>";
                cm.Name = "Rom Path";
                cm.Enabled = true;

                gro.Name = "<Default>";
                gro.Enabled = true;
                gro.CommandLines.Add(cm);
                _CommandLinesGroups.Add(gro);

                //load the groups
                foreach (CommandLinesGroup gr in _CommandLinesGroups)
                {
                    TreeNode_CommandLinesGroup tr = new TreeNode_CommandLinesGroup();
                    tr.CommandLinesGroup = gr;
                    tr.SelectedImageIndex = 0;
                    tr.ImageIndex = 1;
                    tr.Checked = gr.Enabled;
                    tr.RefreshCommandLines(2, 2);

                    treeView1.Nodes.Add(tr);
                }

                if (_DirectApply)
                    Apply();
            }
        }
        //move up
        private void toolStripButton4_Click(object sender, EventArgs e)
        {
            if (treeView1.SelectedNode == null)
                return;
            if (treeView1.SelectedNode.GetType() == typeof(TreeNode_CommandLinesGroup))
            {
                TreeNode_CommandLinesGroup GR = (TreeNode_CommandLinesGroup)treeView1.SelectedNode;
                int index = GR.Index;

                treeView1.Nodes.Remove(GR);

                if (index > 0)
                    index--;

                treeView1.Nodes.Insert(index, GR);
                treeView1.SelectedNode = GR;
            }
            else if (treeView1.SelectedNode.GetType() == typeof(TreeNode_CommandLine))
            {
                TreeNode_CommandLinesGroup GR = (TreeNode_CommandLinesGroup)treeView1.SelectedNode.Parent;
                TreeNode_CommandLine CM = (TreeNode_CommandLine)treeView1.SelectedNode;
                int index = CM.Index;

                GR.Nodes.Remove(CM);
                GR.CommandLinesGroup.CommandLines.RemoveAt(index);

                if (index > 0)
                    index--;

                GR.Nodes.Insert(index, CM);
                GR.CommandLinesGroup.CommandLines.Insert(index, CM.CommandLine);

                treeView1.SelectedNode = CM;
            }
            else if (treeView1.SelectedNode.GetType() == typeof(TreeNode_Parameter))
            {
                TreeNode_CommandLine CM = (TreeNode_CommandLine)treeView1.SelectedNode.Parent;
                TreeNode_Parameter PA = (TreeNode_Parameter)treeView1.SelectedNode;
                int index = PA.Index;

                CM.Nodes.Remove(PA);
                CM.CommandLine.Parameters.RemoveAt(index);

                if (index > 0)
                    index--;

                CM.Nodes.Insert(index, PA);
                CM.CommandLine.Parameters.Insert(index, PA.Parameter);

                treeView1.SelectedNode = PA;
            }
            if (_DirectApply)
                Apply();
        }
        //move down
        private void toolStripButton5_Click(object sender, EventArgs e)
        {
            if (treeView1.SelectedNode == null)
                return;
            if (treeView1.SelectedNode.GetType() == typeof(TreeNode_CommandLinesGroup))
            {
                TreeNode_CommandLinesGroup GR = (TreeNode_CommandLinesGroup)treeView1.SelectedNode;
                int index = GR.Index;

                treeView1.Nodes.Remove(GR);

                if (index < treeView1.Nodes.Count)
                    index++;

                treeView1.Nodes.Insert(index, GR);
                treeView1.SelectedNode = GR;
            }
            else if (treeView1.SelectedNode.GetType() == typeof(TreeNode_CommandLine))
            {
                TreeNode_CommandLinesGroup GR = (TreeNode_CommandLinesGroup)treeView1.SelectedNode.Parent;
                TreeNode_CommandLine CM = (TreeNode_CommandLine)treeView1.SelectedNode;
                int index = CM.Index;

                GR.Nodes.Remove(CM);
                GR.CommandLinesGroup.CommandLines.RemoveAt(index);

                if (index < GR.Nodes.Count)
                    index++;

                GR.Nodes.Insert(index, CM);
                GR.CommandLinesGroup.CommandLines.Insert(index, CM.CommandLine);

                treeView1.SelectedNode = CM;
            }
            else if (treeView1.SelectedNode.GetType() == typeof(TreeNode_Parameter))
            {
                TreeNode_CommandLine CM = (TreeNode_CommandLine)treeView1.SelectedNode.Parent;
                TreeNode_Parameter PA = (TreeNode_Parameter)treeView1.SelectedNode;
                int index = PA.Index;

                CM.Nodes.Remove(PA);
                CM.CommandLine.Parameters.RemoveAt(index);

                if (index < CM.Nodes.Count)
                    index++;

                CM.Nodes.Insert(index, PA);
                CM.CommandLine.Parameters.Insert(index, PA.Parameter);

                treeView1.SelectedNode = PA;
            }
            if (_DirectApply)
                Apply();
        }

        private void toolStripButton6_Click(object sender, EventArgs e)
        {
            SaveFileDialog sav = new SaveFileDialog();
            sav.Filter = "XML (*xml)|*.xml";
            if (sav.ShowDialog(this) == System.Windows.Forms.DialogResult.OK)
            {
                CommandLinesXML xmll = new CommandLinesXML();
                foreach (TreeNode_CommandLinesGroup GR in treeView1.Nodes)
                {
                    xmll.CommandLinesGroups.Add(GR.CommandLinesGroup);
                }

                XmlSerializer SER = new XmlSerializer(typeof(CommandLinesXML));
                Stream STR = new FileStream(sav.FileName, FileMode.Create);
                SER.Serialize(STR, xmll);
                STR.Close();
            }
        }

        private void toolStripButton7_Click(object sender, EventArgs e)
        {
            OpenFileDialog op = new OpenFileDialog();
            op.Filter = "XML (*xml)|*.xml";
            if (op.ShowDialog(this) == System.Windows.Forms.DialogResult.OK)
            {
                XmlSerializer SER = new XmlSerializer(typeof(CommandLinesXML));
                Stream STR = new FileStream(op.FileName, FileMode.Open, FileAccess.Read);
                CommandLinesXML LOADED = (CommandLinesXML)SER.Deserialize(STR);
                STR.Close();

                if (LOADED != null)
                {
                    if (MessageBox.Show("Are you sure you want to load this file and discard current changes ?", "Load xml", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == System.Windows.Forms.DialogResult.Yes)
                    {
                        treeView1.Nodes.Clear();
                        //make sure
                        if (LOADED.CommandLinesGroups == null)
                            LOADED.CommandLinesGroups = new List<CommandLinesGroup>();
                        bool found = false;
                        foreach (CommandLinesGroup gr in LOADED.CommandLinesGroups)
                        {
                            if (gr.Name == "<Default>")
                            { found = true; break; }
                        }
                        if (!found)
                        {
                            CommandLinesGroup gro = new CommandLinesGroup();
                            CommandLine cm = new CommandLine();
                            cm.commandLine = "<RomPath>";
                            cm.Name = "Rom Path";
                            cm.Enabled = true;

                            gro.Name = "<Default>";
                            gro.Enabled = true;
                            gro.CommandLines.Add(cm);
                            LOADED.CommandLinesGroups.Add(gro);
                        }
                        _CommandLinesGroups = LOADED.CommandLinesGroups;
                        //load the groups
                        Refreshcommandlines();
                        if (_DirectApply)
                            Apply();
                    }
                }
            }
        }

        private void treeView1_KeyUp(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Delete)
                toolStripButton1_Click(this, null);
            if (e.KeyCode == Keys.Return)
                toolStripButton3_Click(this, null);
        }
    }
}
