﻿/*  
     This file is part of Emulators Organizer
    A program that organize emulators and roms.
   
    Copyright (C) 2009 - 2012  Ali Hadid, programmed by Ala Hadid

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Diagnostics;
using AHD.Emulators_Organizer.Base;
using AHD.Utilities;

namespace EmulatorsOrganizer.Controls
{
    public partial class SoundPlayer : UserControl
    {
        Rom _rom;
        public SoundPlayer()
        {
            InitializeComponent();
            axWindowsMediaPlayer1.CurrentItemChange += new AxWMPLib._WMPOCXEvents_CurrentItemChangeEventHandler(axWindowsMediaPlayer1_CurrentItemChange);
            axWindowsMediaPlayer1.PlayStateChange += new AxWMPLib._WMPOCXEvents_PlayStateChangeEventHandler(axWindowsMediaPlayer1_PlayStateChange);
        }
        public bool AutoStart
        { get { return checkBox1.Checked; } set { checkBox1.Checked = value; axWindowsMediaPlayer1.settings.autoStart = value; } }
        public void Stop()
        {
            axWindowsMediaPlayer1.Ctlcontrols.stop();
        }
        void axWindowsMediaPlayer1_PlayStateChange(object sender, AxWMPLib._WMPOCXEvents_PlayStateChangeEvent e)
        {
            if (axWindowsMediaPlayer1.playState == WMPLib.WMPPlayState.wmppsPlaying)
            {
                for (int i = 0; i < listView1.Items.Count; i++)
                {
                    if (listView1.Items[i].ImageIndex != 0)
                    {
                        listView1.Items[i].ImageIndex = 1; break;
                    }
                }
            }
            else
            {
                for (int i = 0; i < listView1.Items.Count; i++)
                {
                    if (listView1.Items[i].ImageIndex != 0)
                    {
                        listView1.Items[i].ImageIndex = 2; break;
                    }
                }
            }
        }

        void axWindowsMediaPlayer1_CurrentItemChange(object sender, AxWMPLib._WMPOCXEvents_CurrentItemChangeEvent e)
        {
            for (int i = 0; i < listView1.Items.Count; i++)
                listView1.Items[i].ImageIndex = 0;
            int j = 0;
            foreach (string sound in _rom.SoundRecords)
            {
                if (((WMPLib.IWMPMedia)e.pdispMedia).sourceURL == Utilities.GetFullPath(sound))
                {
                  //  if (axWindowsMediaPlayer1.playState == WMPLib.WMPPlayState.wmppsPlaying)
                   //     listView1.Items[j].ImageIndex = 1;
                   // else
                        listView1.Items[j].ImageIndex = 2;

                    break;
                }
                j++;
            }
        }
        void RefreshSongs()
        {
            listView1.Items.Clear();
            axWindowsMediaPlayer1.Ctlcontrols.stop();
            axWindowsMediaPlayer1.currentPlaylist.clear();
            axWindowsMediaPlayer1.URL = null;
            if (_rom != null)
            {
                if (_rom.SoundRecords == null)
                    _rom.SoundRecords = new List<string>();
                foreach (string file in _rom.SoundRecords)
                {
                    WMPLib.IWMPMedia m1 = axWindowsMediaPlayer1.newMedia(Utilities.GetFullPath(file));
                    listView1.Items.Add(Path.GetFileNameWithoutExtension(file), 0);
                    listView1.Items[listView1.Items.Count - 1].SubItems.Add(m1.durationString);
                    listView1.Items[listView1.Items.Count - 1].SubItems.Add(Path.GetExtension(file).Substring(1));
                    axWindowsMediaPlayer1.currentPlaylist.appendItem(m1);
                }
            }
        }
        public Rom Rom
        {
            get { return _rom; }
            set
            {
                _rom = value;
                RefreshSongs();
                if (axWindowsMediaPlayer1.settings.autoStart)
                    axWindowsMediaPlayer1.Ctlcontrols.play();
            }
        }

        public event EventHandler SaveRequired;
        private void toolStripButton1_Click(object sender, EventArgs e)
        {
            OpenFileDialog op = new OpenFileDialog();
            op.Title = "Browse for sound files";
            op.Filter = "Supported sound files (*.mp3;*.ogg;*.wav;*.midi;*.flc;*.ape;*.wma;*.vox;*.tta;*.raw;*.mpc;*.m4p;*.iklax;*.gsm;*.dct;*.awb;*.Au;*.atrac;*.amr;*.alac;*.aac;*.aiff;*.act)|*.mp3;*.ogg;*.wav;*.midi;*.flc;*.ape;*.wma;*.vox;*.tta;*.raw;*.mpc;*.m4p;*.iklax;*.gsm;*.dct;*.awb;*.Au;*.atrac;*.amr;*.alac;*.aac;*.aiff;*.act|All Files|*.*";
            op.Multiselect = true;
            if (op.ShowDialog(this) == DialogResult.OK)
            {
                foreach (string file in op.FileNames)
                {
                    foreach (string old in _rom.SoundRecords)
                    {
                        if (file == Utilities.GetFullPath(old))
                            continue;
                    }
                    _rom.SoundRecords.Add(file);
                }
                RefreshSongs();
                if (SaveRequired != null)
                    SaveRequired(this, new EventArgs());
            }
        }

        private void toolStripButton2_Click(object sender, EventArgs e)
        {
            if (listView1.SelectedItems.Count > 0)
            {
                if (MessageBox.Show("Are you sure ?", "Delete sound item(s)", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
                {
                    for (int i = 0; i < listView1.SelectedItems.Count; i++)
                    {
                        _rom.SoundRecords.RemoveAt(listView1.SelectedItems[i].Index);
                        listView1.Items.RemoveAt(listView1.SelectedItems[i].Index);
                        i = -1;
                    }
                    RefreshSongs();
                    if (SaveRequired != null)
                        SaveRequired(this, new EventArgs());
                }
            }
        }

        private void toolStripButton3_Click(object sender, EventArgs e)
        {
            if (listView1.SelectedItems.Count > 0)
            {
                if (MessageBox.Show("Are you sure ?\n\nThis can't be undone !!", "Delete sound item(s)", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
                {
                    axWindowsMediaPlayer1.Ctlcontrols.stop();
                    axWindowsMediaPlayer1.currentPlaylist.clear();
                    axWindowsMediaPlayer1.URL = null;
                    for (int i = 0; i < listView1.SelectedItems.Count; i++)
                    {
                        try
                        {
                            File.Delete(Utilities.GetFullPath(_rom.SoundRecords[listView1.SelectedItems[i].Index]));
                            _rom.SoundRecords.RemoveAt(listView1.SelectedItems[i].Index);
                            listView1.Items.RemoveAt(listView1.SelectedItems[i].Index);
                            i = -1;
                        }
                        catch 
                        { }
                    }
                    RefreshSongs();
                    if (SaveRequired != null)
                        SaveRequired(this, new EventArgs());
                }
            }
        }

        private void toolStripButton4_Click(object sender, EventArgs e)
        {
            if (listView1.SelectedItems.Count == 1)
            {
                axWindowsMediaPlayer1.Ctlcontrols.stop();
                try { Process.Start(Utilities.GetFullPath(_rom.SoundRecords[listView1.SelectedItems[0].Index])); }
                catch { }
            }
        }

        private void toolStripButton5_Click(object sender, EventArgs e)
        {
            if (listView1.SelectedItems.Count == 1)
            {
                try
                {
                    Process.Start("explorer.exe", @"/select, " + Utilities.GetFullPath(_rom.SoundRecords[listView1.SelectedItems[0].Index]));
                }
                catch { }
            }
        }

        private void listView1_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            if (listView1.SelectedItems.Count == 1)
            {
                axWindowsMediaPlayer1.Ctlcontrols.playItem(axWindowsMediaPlayer1.currentPlaylist.get_Item(listView1.SelectedItems[0].Index));
            }
        }

        private void toolStripButton6_Click(object sender, EventArgs e)
        {
            if (listView1.SelectedItems.Count == 1)
            {
                string soundFile = _rom.SoundRecords[listView1.SelectedItems[0].Index];
                int index = listView1.SelectedItems[0].Index;

                _rom.SoundRecords.Remove(soundFile);

                if (index > 0)
                    index--;

                _rom.SoundRecords.Insert(index, soundFile);
                RefreshSongs();
                listView1.Items[index].Selected = true;
                if (SaveRequired != null)
                    SaveRequired(this, new EventArgs());
            }
        }

        private void toolStripButton7_Click(object sender, EventArgs e)
        {
            if (listView1.SelectedItems.Count == 1)
            {
                string soundFile = _rom.SoundRecords[listView1.SelectedItems[0].Index];
                int index = listView1.SelectedItems[0].Index;

                _rom.SoundRecords.Remove(soundFile);

                if (index < listView1.Items.Count-1)
                    index++;

                _rom.SoundRecords.Insert(index, soundFile);
                RefreshSongs();
                listView1.Items[index].Selected = true;
                if (SaveRequired != null)
                    SaveRequired(this, new EventArgs());
            }
        }

        private void checkBox1_CheckedChanged(object sender, EventArgs e)
        {
            axWindowsMediaPlayer1.settings.autoStart = checkBox1.Checked; 
        }
    }
}
