﻿/*  
     This file is part of Emulators Organizer
    A program that organize emulators and roms.
   
    Copyright (C) 2009 - 2012  Ali Hadid, programmed by Ala Hadid

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.IO;
using System.Windows.Forms;
using System.Diagnostics;
using AHD.Emulators_Organizer.Base;
using AHD.Utilities;
namespace EmulatorsOrganizer.Controls
{
    public partial class VideoPlayer : UserControl
    {
        public VideoPlayer()
        {
            InitializeComponent();
            axWindowsMediaPlayer1.CurrentItemChange += new AxWMPLib._WMPOCXEvents_CurrentItemChangeEventHandler(axWindowsMediaPlayer1_CurrentItemChange);
            axWindowsMediaPlayer1.PlayStateChange += new AxWMPLib._WMPOCXEvents_PlayStateChangeEventHandler(axWindowsMediaPlayer1_PlayStateChange);
      
        }

        public event EventHandler SaveRequired;
        Rom rom;

        public bool AutoStart
        { get { return checkBox1.Checked; } set { checkBox1.Checked = value; axWindowsMediaPlayer1.settings.autoStart = value; } }
        public void Stop()
        {
            axWindowsMediaPlayer1.Ctlcontrols.stop();
        }
        public Rom Rom
        {
            get { return rom; }
            set 
            {
                rom = value; 
                RefreshList();
                if (axWindowsMediaPlayer1.settings.autoStart)
                    axWindowsMediaPlayer1.Ctlcontrols.play();
            }
        }
        public void RefreshList()
        {
            listView1.Items.Clear();
            axWindowsMediaPlayer1.Ctlcontrols.stop();
            axWindowsMediaPlayer1.currentPlaylist.clear();
            axWindowsMediaPlayer1.URL = null;
            if (rom != null)
            {
                if (rom.Videos == null)
                    rom.Videos = new List<string>();
                foreach (string file in rom.Videos)
                {
                    WMPLib.IWMPMedia m1 = axWindowsMediaPlayer1.newMedia(Utilities.GetFullPath(file));
                    listView1.Items.Add(Path.GetFileNameWithoutExtension(file), 0);
                    listView1.Items[listView1.Items.Count - 1].SubItems.Add(m1.durationString);
                    listView1.Items[listView1.Items.Count - 1].SubItems.Add(Path.GetExtension(file).Substring(1));
                    axWindowsMediaPlayer1.currentPlaylist.appendItem(m1);
                }
            }
        }
        void axWindowsMediaPlayer1_PlayStateChange(object sender, AxWMPLib._WMPOCXEvents_PlayStateChangeEvent e)
        {
            if (axWindowsMediaPlayer1.playState == WMPLib.WMPPlayState.wmppsPlaying)
            {
                for (int i = 0; i < listView1.Items.Count; i++)
                {
                    if (listView1.Items[i].ImageIndex != 0)
                    {
                        listView1.Items[i].ImageIndex = 1; break;
                    }
                }
                listView1.Visible = false;
            }
            else
            {
                for (int i = 0; i < listView1.Items.Count; i++)
                {
                    if (listView1.Items[i].ImageIndex != 0)
                    {
                        listView1.Items[i].ImageIndex = 2; break;
                    }
                }
                listView1.Visible = true;
            }
        }
        void axWindowsMediaPlayer1_CurrentItemChange(object sender, AxWMPLib._WMPOCXEvents_CurrentItemChangeEvent e)
        {
            for (int i = 0; i < listView1.Items.Count; i++)
                listView1.Items[i].ImageIndex = 0;
            int j = 0;
            foreach (string sound in rom.Videos)
            {
                if (((WMPLib.IWMPMedia)e.pdispMedia).sourceURL == Utilities.GetFullPath(sound))
                {
                    //  if (axWindowsMediaPlayer1.playState == WMPLib.WMPPlayState.wmppsPlaying)
                    //     listView1.Items[j].ImageIndex = 1;
                    // else
                    listView1.Items[j].ImageIndex = 2;

                    break;
                }
                j++;
            }
        }
        private void toolStripButton1_Click(object sender, EventArgs e)
        {
            OpenFileDialog op = new OpenFileDialog();
            op.Title = "Browse for sound files";
            op.Filter = "Common Media Formats|*.avi;*.AVI;*.mpeg;*.MPEG;*.mpg;*.MPG;*.mpeg;*.MPEG;*.mpg;*.MPG;*.mpv;*.mp4;*.MP4;*.3gp;*.3GP;*.mov;*.MOV;*.wmv;*.WMV;*.flv;*.FLV;|"
                + "All Files|*"; 
            op.Multiselect = true;
            if (op.ShowDialog(this) == DialogResult.OK)
            {
                foreach (string file in op.FileNames)
                {
                    foreach (string old in rom.Videos)
                    {
                        if (file == Utilities.GetFullPath(old))
                            continue;
                    }
                    rom.Videos.Add(file);
                }
                RefreshList();
                if (SaveRequired != null)
                    SaveRequired(this, new EventArgs());
            }
        }

        private void toolStripButton2_Click(object sender, EventArgs e)
        {
            if (listView1.SelectedItems.Count > 0)
            {
                if (MessageBox.Show("Are you sure ?", "Delete video item(s)", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
                {
                    for (int i = 0; i < listView1.SelectedItems.Count; i++)
                    {
                        rom.Videos.RemoveAt(listView1.SelectedItems[i].Index);
                        listView1.Items.RemoveAt(listView1.SelectedItems[i].Index);
                        i = -1;
                    }
                    RefreshList();
                    if (SaveRequired != null)
                        SaveRequired(this, new EventArgs());
                }
            }
        }

        private void toolStripButton3_Click(object sender, EventArgs e)
        {
            if (listView1.SelectedItems.Count > 0)
            {
                if (MessageBox.Show("Are you sure ?\n\nThis can't be undone !!", "Delete video item(s)", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
                {
                    axWindowsMediaPlayer1.Ctlcontrols.stop();
                    axWindowsMediaPlayer1.currentPlaylist.clear();
                    axWindowsMediaPlayer1.URL = null;
                    for (int i = 0; i < listView1.SelectedItems.Count; i++)
                    {
                        try
                        {
                            File.Delete(Utilities.GetFullPath(rom.Videos[listView1.SelectedItems[i].Index]));
                            rom.Videos.RemoveAt(listView1.SelectedItems[i].Index);
                            listView1.Items.RemoveAt(listView1.SelectedItems[i].Index);
                            i = -1;
                        }
                        catch
                        { }
                    }
                    RefreshList();
                    if (SaveRequired != null)
                        SaveRequired(this, new EventArgs());
                }
            }
        }

        private void toolStripButton4_Click(object sender, EventArgs e)
        {
            try
            {
                if (listView1.SelectedItems.Count == 1)
                {
                    try
                    {
                        Process.Start("explorer.exe", @"/select, " + Utilities.GetFullPath(rom.Videos[listView1.SelectedItems[0].Index]));
                    }
                    catch { }
                }
            }
            catch { }
        }
        private void toolStripButton5_Click(object sender, EventArgs e)
        {
            if (listView1.SelectedItems.Count == 1)
            {
                axWindowsMediaPlayer1.Ctlcontrols.stop();
                try { Process.Start(Utilities.GetFullPath(rom.Videos[listView1.SelectedItems[0].Index])); }
                catch { }
            }
        }

        private void checkBox1_CheckedChanged(object sender, EventArgs e)
        {
            axWindowsMediaPlayer1.settings.autoStart = checkBox1.Checked;
        }

        private void listView1_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            if (listView1.SelectedItems.Count == 1)
            {
                axWindowsMediaPlayer1.Ctlcontrols.playItem(axWindowsMediaPlayer1.currentPlaylist.get_Item(listView1.SelectedItems[0].Index));
            }
        }
    }
}
