﻿/*  
     This file is part of Emulators Organizer
    A program that organize emulators and roms.
   
    Copyright (C) 2009 - 2012  Ali Hadid, programmed by Ala Hadid

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows.Forms;
using System.IO;
using System.Runtime.Serialization.Formatters.Binary;
using System.Runtime.Serialization;
using AHD.Emulators_Organizer.Base;
using System.Reflection;
//Version 3 started at: Monday, June 13, 2011

namespace EmulatorsOrganizer
{
    static class Program
    {
        static Properties.Settings _Usersettings = new Properties.Settings();
        static Settings _settings = new Settings();
        static Project _currentProject = new Project();
        static string _CurrentProjectPath = "";
        static string _StartUpPath = "";
        static Frm_main _mainForm;
        /// <summary>
        /// The main entry point for the application.
        /// </summary>
        [STAThread]
        static void Main(string[] args)
        {
            Application.EnableVisualStyles();
            Application.SetCompatibleTextRenderingDefault(false);
            _Usersettings.Reload();
            _StartUpPath = Application.StartupPath;
            LoadSettings();
            Application.Run(_mainForm = new Frm_main(args));
        }
        /// <summary>
        /// Get the user settings which will be saved in application data.
        /// </summary>
        public static Properties.Settings UserSettings
        { get { return _Usersettings; } }
        /// <summary>
        /// Get or set the project
        /// </summary>
        public static Project Project
        { get { return _currentProject; } set { _currentProject = value; } }
        public static string ProjectPath
        { get { return _CurrentProjectPath; } set { _CurrentProjectPath = value; } }
        public static string StartUpDir
        { get { return _StartUpPath; } }
        /// <summary>
        /// Get the main form
        /// </summary>
        public static Frm_main frmMain
        { get { return _mainForm; } }
        public static bool SaveProject(string FilePath)
        {
            try
            {
                FileStream fs = new FileStream(FilePath, FileMode.Create);
                BinaryFormatter formatter = new BinaryFormatter();
                formatter.Serialize(fs, _currentProject);
                fs.Close();
                _CurrentProjectPath = FilePath;
                return true;
            }
            catch { _CurrentProjectPath = ""; MessageBox.Show("Can't save the project there !!"); }
            return false;
        }
        public static bool LoadProject(string FilePath)
        {
            try
            {
                if (File.Exists(FilePath))
                {
                    FileStream fs = new FileStream(FilePath, FileMode.Open, FileAccess.Read);
                    BinaryFormatter formatter = new BinaryFormatter();
                    _currentProject = (Project)formatter.Deserialize(fs);
                    fs.Close();
                    _CurrentProjectPath = FilePath;
                    return true;
                }
            }
            catch { _CurrentProjectPath = ""; MessageBox.Show("Can't load that project !!"); }
            return false;
        }
        /// <summary>
        /// Get Emulators Organizer current version
        /// </summary>
        public static string Version
        { get { return Assembly.GetExecutingAssembly().GetName().Version.ToString(); } }
        public static Settings Setting
        { get { return _settings; } set { _settings = value; } }

        public static void SaveSettings()
        {
            try
            {
                FileStream fs = new FileStream(StartUpDir + "//Settings.st", FileMode.Create, FileAccess.Write);
                BinaryFormatter formatter = new BinaryFormatter();
                formatter.Serialize(fs, _settings);
                fs.Close();
            }
            catch { }
        }
        public static void LoadSettings()
        {
            try
            {
                if (File.Exists(StartUpDir + "//Settings.st"))
                {
                    FileStream fs = new FileStream(StartUpDir + "//Settings.st", FileMode.Open, FileAccess.Read);
                    BinaryFormatter formatter = new BinaryFormatter();
                    _settings = (Settings)formatter.Deserialize(fs);
                    fs.Close();
                }
            }
            catch { }
        }
    }
}
