﻿/*  
     This file is part of Emulators Organizer
    A program that organize emulators and roms.
   
    Copyright (C) 2009 - 2012  Ali Hadid, programmed by Ala Hadid

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.IO;
using System.Windows.Forms;
using AHD.Utilities;
using AHD.Emulators_Organizer.Base;

namespace EmulatorsOrganizer.Frms
{
    public partial class Frm_DetectVideos : Form
    {
        string _lastSelected = "";
        EmuConsole console;
        public Frm_DetectVideos(EmuConsole console)
        {
            this.console = console;
            InitializeComponent();

            //refresh folders
            if (console.VideoDirs == null)
                console.VideoDirs = new List<string>();
            foreach (string dir in console.VideoDirs)
            {
                if (Directory.Exists(Utilities.GetFullPath(dir)))
                    listBox1.Items.Add(dir);
            }

            if (listBox1.Items.Count > 0)
                _lastSelected = Utilities.GetFullPath(listBox1.Items[listBox1.Items.Count - 1].ToString());

            //refresh extensions
            if (console.VideoExtensions == null)
                console.VideoExtensions = new List<string>(new string[] { ".avi", ".mpeg", ".mpg", ".mpeg", ".mp4", ".3gp", ".mov", ".wmv", ".flv" });
            if (console.VideoExtensions.Count == 0)
                console.VideoExtensions = new List<string>(new string[] { ".avi", ".mpeg", ".mpg", ".mpeg", ".mp4", ".3gp", ".mov", ".wmv", ".flv" });

            foreach (string ex in console.VideoExtensions)
                comboBox1.Items.Add(ex);

            if (comboBox1.Items.Count > 0)
                comboBox1.SelectedIndex = 0;
        }

        private void linkLabel1_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            FolderBrowserDialog fol = new FolderBrowserDialog();
            fol.SelectedPath = _lastSelected;
            if (fol.ShowDialog(this) == System.Windows.Forms.DialogResult.OK)
            {
                _lastSelected = fol.SelectedPath;
                foreach (string path in listBox1.Items)
                {
                    if (path.ToLower() == Utilities.GetDotPath(fol.SelectedPath).ToLower())
                        return;
                }
                listBox1.Items.Add(Utilities.GetDotPath(fol.SelectedPath));
            }
        }

        private void linkLabel2_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            if (listBox1.SelectedIndex >= 0)
                listBox1.Items.RemoveAt(listBox1.SelectedIndex);
        }

        private void linkLabel4_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            Frm_EnterName name = new Frm_EnterName("Enter extension", "");
            if (name.ShowDialog(this) == System.Windows.Forms.DialogResult.OK)
            {
                int i = 0;
                foreach (string ex in comboBox1.Items)
                {
                    if (ex.ToLower() == "." + name.NameEntered.ToLower())
                    {
                        comboBox1.SelectedIndex = i;
                        return;
                    }
                    i++;
                }
                comboBox1.Items.Add("." + name.NameEntered.ToLower());
                comboBox1.SelectedIndex = comboBox1.Items.Count - 1;
            }
        }

        private void linkLabel3_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            if (comboBox1.Items.Count > 0)
            {
                comboBox1.Items.RemoveAt(comboBox1.SelectedIndex);
            }
            if (comboBox1.Items.Count > 0)
                comboBox1.SelectedIndex = 0;
        }

        private void button1_Click(object sender, EventArgs e)
        {
            //1 get dirs
            if (listBox1.Items.Count == 0)
            {
                MessageBox.Show("There's no folder !!");
                return;
            }
            //1 get dirs
            if (comboBox1.Items.Count == 0)
            {
                MessageBox.Show("There's no extension to search for !!");
                return;
            }
            console.VideoDirs.Clear();
            console.VideoExtensions.Clear();
            foreach (string folder in listBox1.Items)
            {
                console.VideoDirs.Add(folder);
            }
            foreach (string ex in comboBox1.Items)
            {
                console.VideoExtensions.Add(ex);
            }

            progressBar1.Visible = true;
            List<string> dirs = new List<string>();
            foreach (string folder in listBox1.Items)
            {
                dirs.AddRange(Directory.GetFiles(Utilities.GetFullPath(folder), "*", SearchOption.AllDirectories));
            }

            progressBar1.Maximum = console.Roms.Count;
            string[] Exs = console.VideoExtensions.ToArray();

            foreach (Rom ro in console.Roms)
            {
                ro.Videos = new List<string>();
                for (int i = 0; i < dirs.Count; i++)
                {
                    bool Found = false;
                    string ImageName = Path.GetFileNameWithoutExtension(dirs[i]).ToLower();
                    string RomRealName = Path.GetFileNameWithoutExtension(ro.Path).ToLower();
                    string ImageExtension = Path.GetExtension(dirs[i]).ToLower();
                    if (ImageName.Length >= RomRealName.Length)
                    {
                        foreach (string ext in Exs)
                        {
                            if (ImageExtension.ToLower() == ext)
                            {
                                if (ImageName.Substring(0, RomRealName.Length).ToLower() == RomRealName.ToLower())
                                {
                                    ro.Videos.Add(Utilities.GetDotPath(dirs[i]));
                                    dirs.RemoveAt(i);
                                    i = -1;
                                    Found = true;
                                    break;
                                }
                            }
                        }
                    }
                    if (Found)
                        break;
                }
                try { progressBar1.Value++; }
                catch { }
            }
            this.DialogResult = System.Windows.Forms.DialogResult.OK;
            Close();
        }
        private void button2_Click(object sender, EventArgs e)
        {
            this.DialogResult = System.Windows.Forms.DialogResult.Cancel;
            Close();
        }
    }
}
