﻿/*  
     This file is part of Emulators Organizer
    A program that organize emulators and roms.
   
    Copyright (C) 2009 - 2012  Ali Hadid, programmed by Ala Hadid

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using AHD.Utilities;
using AHD.Emulators_Organizer.Base;

namespace EmulatorsOrganizer.Frms
{
    public partial class Frm_ExportToDatabase : Form
    {
        EmuConsole console;
        bool aToZ = true;
        public Frm_ExportToDatabase(EmuConsole console)
        {
            InitializeComponent();
            this.console = console;
            //refresh roms
            foreach (Rom rom in console.Roms)
            {
                listView1.Items.Add(rom.Name);
                listView1.Items[listView1.Items.Count - 1].Checked = true;
                listView1.Items[listView1.Items.Count - 1].SubItems.Add(Path.GetExtension(rom.Path));

                listView1.Items[listView1.Items.Count - 1].SubItems.Add(Utilities.GetFileSize(rom.Path) +
                    " (" + Utilities.GetSizeAsBytes(rom.Path) + " B)");

                listView1.Items[listView1.Items.Count - 1].SubItems.Add(Utilities.GetFullPath(rom.Path));
                Application.DoEvents();
            }
        }

        private void button8_Click(object sender, EventArgs e)
        {
            this.Close();
        }
        //select all
        private void button1_Click(object sender, EventArgs e)
        {
            foreach (ListViewItem it in listView1.Items)
                it.Checked = true;
        }
        //select none
        private void button2_Click(object sender, EventArgs e)
        {
            foreach (ListViewItem it in listView1.Items)
                it.Checked = false;
        }
        //sort by ..
        private void button3_Click(object sender, EventArgs e)
        {
            listView1.ListViewItemSorter = new ListViewItemComparer(0, aToZ);
            aToZ = !aToZ;
        }
        private void button4_Click(object sender, EventArgs e)
        {
            listView1.ListViewItemSorter = new ListViewItemComparer(1, aToZ);
            aToZ = !aToZ;
        }
        private void listView1_ColumnClick(object sender, ColumnClickEventArgs e)
        {
            listView1.ListViewItemSorter = new ListViewItemComparer(e.Column, aToZ);
            aToZ = !aToZ;
        }
        //move up
        private void button5_Click(object sender, EventArgs e)
        {
            if (listView1.SelectedItems.Count != 1)
                return;
            //get selected item
            ListViewItem selected = listView1.SelectedItems[0];
            int index = selected.Index;
            //remove it !!
            listView1.Items.Remove(listView1.SelectedItems[0]);
            //move up
            index--;
            if (index < 0)
                index = 0;
            //insert
            listView1.Items.Insert(index, selected);
        }
        private void button6_Click(object sender, EventArgs e)
        {
            if (listView1.SelectedItems.Count != 1)
                return;
            //get selected item
            ListViewItem selected = listView1.SelectedItems[0];
            int index = selected.Index;
            //remove it !!
            listView1.Items.Remove(listView1.SelectedItems[0]);
            //move down
            index++;
            if (index >= listView1.Items.Count)
                index = listView1.Items.Count;
            //insert
            listView1.Items.Insert(index, selected);
        }
        private void listView1_ItemDrag(object sender, ItemDragEventArgs e)
        {
            if (listView1.SelectedItems.Count == 0)
                return;
            DoDragDrop(listView1.SelectedItems, DragDropEffects.Move);
        }
        private void listView1_DragOver(object sender, DragEventArgs e)
        {
            if (!e.Data.GetDataPresent(DataFormats.FileDrop))
            {
                e.Effect = DragDropEffects.Move;
            }
            else
                e.Effect = DragDropEffects.None;
        }
        private void listView1_DragDrop(object sender, DragEventArgs e)
        {
            Point targetPoint = listView1.PointToClient(new Point(e.X, e.Y));
            ListViewItem targetItem = listView1.GetItemAt(targetPoint.X, targetPoint.Y);
            int index = targetItem.Index;

            for (int i = 0; i < listView1.SelectedItems.Count; i++)
            {
                ListViewItem dragedItem = listView1.SelectedItems[i];
                listView1.Items.Remove(listView1.SelectedItems[i]);
                dragedItem.Selected = false;
                i = -1;

                listView1.Items.Insert(index, dragedItem);
                index++;
            }
        }
        //Export
        private void button7_Click(object sender, EventArgs e)
        {
            SaveFileDialog sav = new SaveFileDialog();
            sav.Title = "Export to database";
            sav.Filter = DatabaseManager.GetFilter(true);
            if (sav.ShowDialog(this) == System.Windows.Forms.DialogResult.OK)
            {
                label_progress.Visible = progressBar1.Visible = true;
                button7.Enabled = button8.Enabled = groupBox2.Enabled = groupBox1.Enabled = false;
                //get database file
                IDatabaseFile db = DatabaseManager.GetFromExtension(Path.GetExtension(sav.FileName));
                db.Progress += new EventHandler<ProgressArg>(db_Progress);

                //create the database list
                List<DatabaseRom> items = new List<DatabaseRom>();
                foreach (ListViewItem item in listView1.Items)
                {
                    if (item.Checked)
                    {
                        DatabaseRom rom = new DatabaseRom();
                        rom.Name = item.Text;
                        rom.Description = item.Text;
                        rom.Path = item.SubItems[3].Text;
                        items.Add(rom);
                    }
                }
                //save
                db.Files = items.ToArray();
                db.SaveFile(sav.FileName);
                label_progress.Visible = progressBar1.Visible = false;
                button7.Enabled = button8.Enabled = groupBox2.Enabled = groupBox1.Enabled = true;
            }
        }
        void db_Progress(object sender, ProgressArg e)
        {
            label_progress.Text = e.Status + " (" + e.PrecentageCompleted + " %)";
            progressBar1.Value = e.PrecentageCompleted;
            Application.DoEvents();
        }
    }
}
