﻿/*  
     This file is part of Emulators Organizer
    A program that organize emulators and roms.
   
    Copyright (C) 2009 - 2012  Ali Hadid, programmed by Ala Hadid

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Net;
using System.IO;
using System.Diagnostics;
using AHD.Utilities;
namespace EmulatorsOrganizer.Frms
{
    public partial class Frm_ImageSearcher : Form
    {
        public Frm_ImageSearcher(string SearchFor, string folder)
        {
            InitializeComponent();
            GoogleImageSearcher.ResultsCount = 20;
            textBox1.Text = SearchFor;
            textBox2.Text = folder;
        }

        List<string> SelectedLinks = new List<string>();
        public event EventHandler OptionRequest;
        delegate void AddListViewItem(string item);
        delegate void AddImageToList(Image item);
        delegate void ClearListView();

        void AddItem(string item)
        {
            if (!this.InvokeRequired)
                AddItem1(item);
            else
                this.Invoke(new AddListViewItem(AddItem1), new object[] { item });
        }
        void AddItem1(string item)
        {
            listView1.Items.Add(item);
        }
        void AddImage(Image item)
        {
            if (!this.InvokeRequired)
                AddImage1(item);
            else
                this.Invoke(new AddImageToList(AddImage1), new object[] { item });
        }
        void AddImage1(Image item)
        {
            imageList1.Images.Add(item);
        }
        void ClearTheList()
        {
            if (!this.InvokeRequired)
                ClearTheList1();
            else
                this.Invoke(new ClearListView(ClearTheList1));
        }
        void ClearTheList1()
        {
            listView1.Items.Clear();
        }
        void ClearTheImageList()
        {
            if (!this.InvokeRequired)
                ClearTheImageList1();
            else
                this.Invoke(new ClearListView(ClearTheImageList1));
        }
        void ClearTheImageList1()
        {
            imageList1.Images.Clear();
        }
        public List<string> SelectedImageLinks
        {
            get { return SelectedLinks; }
        }
        public string FolderSelected
        {
            get { return textBox2.Text; }
        }

        void SearchImages()
        {
            if (textBox1.Text.Length == 0)
            {
                MessageBox.Show("Please enter the rom name to search for");
                return;
            }
            try
            {
                if (!checkBox1.Checked)
                {
                    GoogleImageSearcher.GetImages(textBox1.Text, "");
                }
                else
                {
                    if (numericUpDown_h.Value > 0 & numericUpDown_width.Value > 0)
                    {
                        GoogleImageSearcher.GetImages(textBox1.Text, numericUpDown_width.Value + "x" + numericUpDown_h.Value);
                    }
                    else
                    {
                        MessageBox.Show("The image size MUST not include any 0 !!");
                        return;
                    }
                }
                ClearTheList();
                ClearTheImageList();
                for (int i = 0; i < GoogleImageSearcher.ImageThunmbnails.Length; i++)
                {
                    AddImage(getImage(GoogleImageSearcher.ImageThunmbnails[i]));
                    AddItem(GoogleImageSearcher.ImageLinks[i]);
                }
            }
            catch
            {
                MessageBox.Show("Unable to connect");
                return;
            }
        }

        private void listView1_DrawItem(object sender, DrawListViewItemEventArgs e)
        {
            int h = imageList1.ImageSize.Height;
            int w = imageList1.ImageSize.Width;
            if (e.Item.Selected)
            {
                if (listView1.Focused)
                    e.Graphics.FillRectangle(new SolidBrush(SystemColors.Highlight),
                      e.Bounds.X, e.Bounds.Y, e.Bounds.Width, e.Bounds.Height);
                else
                    e.Graphics.FillRectangle(new SolidBrush(SystemColors.ButtonFace),
                e.Bounds.X, e.Bounds.Y, e.Bounds.Width, e.Bounds.Height);
            }
            //try
            {
                e.Graphics.DrawImage(imageList1.Images[e.ItemIndex], e.Bounds.X + ((e.Bounds.Width / 2) - (w / 2)), e.Bounds.Y + 2, w, h);
                Size ss = TextRenderer.MeasureText(GoogleImageSearcher.ImageSizes[e.ItemIndex], new Font("Tahoma", 8, FontStyle.Regular));
                e.Graphics.FillRectangle(new SolidBrush(Color.Black), new Rectangle(e.Bounds.X + 1, e.Bounds.Y + 1, ss.Width + 3, ss.Height));
                e.Graphics.DrawString(GoogleImageSearcher.ImageSizes[e.ItemIndex], new Font("Tahoma", 8, FontStyle.Regular),
                    new SolidBrush(Color.White), new PointF(e.Bounds.X + 1, e.Bounds.Y + 1));
            }
            // catch { }
            e.DrawText(TextFormatFlags.Bottom | TextFormatFlags.HorizontalCenter | TextFormatFlags.EndEllipsis);

        }

        private void button1_Click(object sender, EventArgs e)
        {
            GoogleImageSearcher.ResultsCount = (int)numericUpDown1.Value;
            SearchImages();
        }

        private Image getImage(string url)
        {
            Image im = null;
            try
            {
                HttpWebRequest request = (HttpWebRequest)WebRequest.Create(url);
                request.Method = "GET";
                request.Timeout = 15000;
                request.ProtocolVersion = HttpVersion.Version11;

                using (HttpWebResponse response = (HttpWebResponse)request.GetResponse())
                {
                    using (Stream responseStream = response.GetResponseStream())
                    {
                        im = Image.FromStream(responseStream);
                    }
                }
            }
            catch
            {
            }
            return im;
        }

        private void listView1_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            try
            {
                Process.Start(listView1.SelectedItems[0].Text);
            }
            catch { }
        }

        private void button2_Click(object sender, EventArgs e)
        {
            if (listView1.SelectedItems.Count == 0)
            {
                MessageBox.Show("Select image you want to download first !!");
                return;
            }
            if (!Directory.Exists(textBox2.Text))
            {
                MessageBox.Show("Please browse for a download folder first, the folder isn't exists.");
                textBox2.SelectAll();
                return;
            }

            SelectedLinks.Clear();
            foreach (ListViewItem item in listView1.SelectedItems)
            {
                SelectedLinks.Add(item.Text);
            }

            this.DialogResult = System.Windows.Forms.DialogResult.OK;
            Close();
        }

        private void button3_Click(object sender, EventArgs e)
        {
            this.DialogResult = System.Windows.Forms.DialogResult.Cancel;
            Close();
        }

        private void linkLabel1_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            try
            {
                Process.Start("http://images.google.com/");
            }
            catch { }
        }

        private void button5_Click(object sender, EventArgs e)
        {
            FolderBrowserDialog fol = new FolderBrowserDialog();
            fol.Description = "Select folder where to save files";
            fol.SelectedPath = textBox2.Text;
            if (fol.ShowDialog(this) == System.Windows.Forms.DialogResult.OK)
            {
                textBox2.Text = fol.SelectedPath;
            }
        }

        private void button6_Click(object sender, EventArgs e)
        {
            if (OptionRequest != null)
                OptionRequest(sender, e);
        }
    }
}
