﻿/*  
     This file is part of Emulators Organizer
    A program that organize emulators and roms.
   
    Copyright (C) 2009 - 2012  Ali Hadid, programmed by Ala Hadid

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using BER.CDCat.Export;
using System.IO;
using IsoCreator;

namespace SharpISO
{
    public partial class Frm_FolderToIso : Form
    {
        Thread m_thread = null;
        IsoCreator.IsoCreator m_creator = null;

        public Frm_FolderToIso()
        {
            InitializeComponent();
            m_creator = new IsoCreator.IsoCreator();
            m_creator.Progress += new ProgressDelegate(m_creator_Progress);
            m_creator.Finish += new FinishDelegate(m_creator_Finish);
            m_creator.Abort += new AbortDelegate(m_creator_Abort);
        }
        private delegate void SetLabelDelegate(string text);
        private delegate void SetNumericValueDelegate(int value);
        private void SetLabelStatus(string text)
        {
            this.labelStatus.Text = text;
            this.labelStatus.Refresh();
        }

        private void SetProgressValue(int value)
        {
            this.progressBar.Value = value;
        }

        private void SetProgressMaximum(int maximum)
        {
            this.progressBar.Maximum = maximum;
        }

        void m_creator_Abort(object sender, AbortEventArgs e)
        {
            MessageBox.Show(e.Message, "Abort", MessageBoxButtons.OK, MessageBoxIcon.Stop);
            MessageBox.Show("The ISO creating process has been stopped.", "Abort", MessageBoxButtons.OK, MessageBoxIcon.Stop);
            buttonStartAbort.Enabled = true;
            buttonStartAbort.Text = "Start";
            progressBar.Value = 0;
            progressBar.Maximum = 0;
            labelStatus.Text = "Process not started";
        }

        void m_creator_Finish(object sender, FinishEventArgs e)
        {
            MessageBox.Show(e.Message, "Finish", MessageBoxButtons.OK, MessageBoxIcon.Information);
            buttonStartAbort.Enabled = true;
            buttonStartAbort.Text = "Start";
            progressBar.Value = 0;
            labelStatus.Text = "Process not started";
        }

        void m_creator_Progress(object sender, ProgressEventArgs e)
        {
            if (e.Action != null)
            {
                if (!this.InvokeRequired)
                {
                    this.SetLabelStatus(e.Action);
                }
                else
                {
                    this.Invoke(new SetLabelDelegate(SetLabelStatus), e.Action);
                }
            }

            if (e.Maximum != -1)
            {
                if (!this.InvokeRequired)
                {
                    this.SetProgressMaximum(e.Maximum);
                }
                else
                {
                    this.Invoke(new SetNumericValueDelegate(SetProgressMaximum), e.Maximum);
                }
            }

            if (!this.InvokeRequired)
            {
                progressBar.Value = (e.Current <= progressBar.Maximum) ? e.Current : progressBar.Maximum;
            }
            else
            {
                int value = (e.Current <= progressBar.Maximum) ? e.Current : progressBar.Maximum;
                this.Invoke(new SetNumericValueDelegate(SetProgressValue), value);
            }
        }

        private void Frm_FolderToIso_FormClosed(object sender, FormClosedEventArgs e)
        {
            Application.Exit();
        }

        private void buttonStartAbort_Click(object sender, EventArgs e)
        {
            if (m_thread == null || !m_thread.IsAlive)
            {
                if (textBox_VolumeName.Text.Trim() != "")
                {
                    m_thread = new Thread(new ParameterizedThreadStart(m_creator.Folder2Iso));
                    m_thread.Start(new IsoCreator.IsoCreator.IsoCreatorFolderArgs(textBox_FolderPath.Text, textBox_IsoPath.Text, textBox_VolumeName.Text));

                    buttonStartAbort.Text = "Abort";
                }
                else
                {
                    MessageBox.Show("Please insert a name for the volume", "No volume name", MessageBoxButtons.OK, MessageBoxIcon.Hand);
                }
            }
            else
            {
                if (MessageBox.Show("Are you sure you want to abort the process?", "Abort", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
                {
                    m_thread.Abort();
                }
            }
        }

        private void button1_Click(object sender, EventArgs e)
        {
            FolderBrowserDialog dialog = new FolderBrowserDialog();
            if (dialog.ShowDialog(this) == DialogResult.OK)
            {
                textBox_FolderPath.Text = dialog.SelectedPath;
            }
        }
        private void button2_Click(object sender, EventArgs e)
        {
            SaveFileDialog dialog = new SaveFileDialog();
            dialog.Filter = "CD Images (*.iso)|*.iso";
            if (dialog.ShowDialog(this) == DialogResult.OK)
            {
                textBox_IsoPath.Text = dialog.FileName;
            }
        }
    }
}
