﻿/*  
     This file is part of Emulators Organizer
    A program that organize emulators and roms.
   
    Copyright (C) 2009 - 2012  Ali Hadid, programmed by Ala Hadid

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Diagnostics;
using System.Threading;
using AHD.Emulators_Organizer.Base;
using EmulatorsOrganizer.Frms;
using AHD.Utilities;

namespace EmulatorsOrganizer
{
    public partial class Frm_main : Form
    {
        int RatingX = 0;
        bool _save = false;
        bool IsRating = false;
        bool IsLoadingRoms = false;
        ActiveElement _ActiveElement = ActiveElement.ConsoleOrCategory;
        List<PlayListItem> _CopiedPlaylistItems = new List<PlayListItem>();
        SevenZip.SevenZipExtractor EXTRACTOR;
        int countDown = 5;
        SortMode _SortMode = SortMode.None;
        Process EmuProccess;
        bool _SortAlpha = true;
        bool EnsureVissibleRom = false;
        Frm_Search findForm;
        List<ListViewItem> DragedItems = new List<ListViewItem>();
        ListViewItem_PlayListItem DestinationItem;
        bool CanDrop;
        bool DragOut = false;
        bool FilesDrag = false;
        bool ItemSearched = false;
        bool consoleATOZ = false;
        bool playlistATOZ = false;

        public Frm_main(string[] args)
        {
            InitializeComponent();
            //load user settings like location, size....
            LoadUserSettings();
            //Load the Initialize settings like remember last opened project
            LoadInitSettings(args);
        }
        bool Save
        {
            get { return _save; }
            set
            {
                _save = value;
                if (Program.UserSettings.AutoSave & value)
                {
                    if (File.Exists(Program.ProjectPath))
                    {
                        if (Program.SaveProject(Program.ProjectPath))
                        {
                            _save = false;
                        }
                    }
                    else
                    {
                        SaveFileDialog Sav = new SaveFileDialog();
                        Sav.Title = "Save profile (auto save, this should appear once)";
                        Sav.Filter = "Emulators Organizer Profile file (*.eop)|*.eop";
                        if (Sav.ShowDialog(this) == DialogResult.OK)
                        {
                            if (Program.SaveProject(Sav.FileName))
                            {
                                _save = false;
                            }
                        }
                    }
                }
                //saveProfileToolStripMenuItem.Enabled = value;
                if (_save)
                {
                    if (Program.Project.Name.Length > 0)
                        this.Text = Program.Project.Name + "* - " + "Emulators Organizer";
                    else
                        this.Text = "Emulators Organizer" + "*";
                }
                else
                {
                    if (Program.Project.Name.Length > 0)
                        this.Text = Program.Project.Name + " - " + "Emulators Organizer";
                    else
                        this.Text = "Emulators Organizer";
                }
            }
        }
        void LoadUserSettings()
        {
            comboBox_thumpType.SelectedIndex = 0;
            this.Size = Program.UserSettings.win_size;
            this.Location = Program.UserSettings.win_location;
            splitContainer1.SplitterDistance = Program.UserSettings.splitContainer1;
            splitContainer2.SplitterDistance = Program.UserSettings.splitContainer2;
            splitContainer3.SplitterDistance = Program.UserSettings.splitContainer3;
            splitContainer4.SplitterDistance = Program.UserSettings.splitContainer4;
            splitContainer5.SplitterDistance = Program.UserSettings.splitContainer5;
            //recent profiles
            if (Program.Setting.RecentProfiles == null)
                Program.Setting.RecentProfiles = new List<string>();
            RefreshRecents();

            trackBar1.Value = Program.UserSettings.consoleImagSize;
            trackBar2.Value = Program.UserSettings.consoleImagSize;
            trackBar3.Value = Program.UserSettings.consoleImagSize;
            trackBar1_Scroll(this, null);

            toolStripButton_commandlinesShowHide.Checked = Program.UserSettings.ShowCommandlinesPanel;
            toolStripButton5_Click(this, null);

            listView_roms.View = Program.UserSettings.ListView;
            switch (Program.UserSettings.ListView)
            {
                case View.Details: detailsToolStripMenuItem_Click(this, null); break;
                case View.LargeIcon: largIconsToolStripMenuItem_Click(this, null); break;
            }
            ViewChanged();
            romStatus1.ShowSnapshot = Program.UserSettings.UseSnapInStatus;
            cntr_ImagesBrowser_covers.StreatchByDefault = Program.UserSettings.forceStretchCovers;
            cntr_ImagesBrowser_snaps.StreatchByDefault = Program.UserSettings.forceStretchSnapshots;
            cntr_ImagesBrowser_covers2.StreatchByDefault = Program.UserSettings.forceStretchCovers;
            cntr_ImagesBrowser_snaps2.StreatchByDefault = Program.UserSettings.forceStretchSnapshots;

            trackBar4.Value = Program.UserSettings.ThunmbnailSize;
            trackBar4_Scroll(this, null);
            imageList1_romsLarge.ImageSize = new Size(trackBar4.Value, trackBar4.Value);
            //columns
            columnHeader_name.Width = Program.UserSettings.Column_name_width;
            columnHeader_path.Width = Program.UserSettings.Column_path_width;
            columnHeader_size.Width = Program.UserSettings.Column_Size_width;
            columnHeader1_fileType.Width = Program.UserSettings.Column_filetype_width;
            columnHeader1_playtime.Width = Program.UserSettings.Column_playedtime_width;
            columnHeader1_rating.Width = Program.UserSettings.Column_rating_width;
            columnHeader1_hasCover.Width = Program.UserSettings.Column_hasCover_width;
            columnHeader1_hasHistory.Width = Program.UserSettings.Column_hasHistory_width;
            columnHeader1_hasInfo.Width = Program.UserSettings.Column_hasInfo_width;
            columnHeader1_hasSnap.Width = Program.UserSettings.Column_hasSnap_width;
            columnHeader1_hasManual.Width = Program.UserSettings.Column_hasManual_width;
            columnHeader_year.Width = Program.UserSettings.Column_year_width;
            columnHeader_video.Width = Program.UserSettings.Column_video_width;
            columnHeader_sounds.Width = Program.UserSettings.Column_sounds_width;

            RefreshColumns();
            columnHeader_path.DisplayIndex = Program.UserSettings.Column_path_Index;
            columnHeader_size.DisplayIndex = Program.UserSettings.Column_Size_Index;
            columnHeader1_fileType.DisplayIndex = Program.UserSettings.Column_filetype_Index;
            columnHeader1_playtime.DisplayIndex = Program.UserSettings.Column_playedtime_Index;
            columnHeader1_rating.DisplayIndex = Program.UserSettings.Column_rating_Index;
            columnHeader1_hasCover.DisplayIndex = Program.UserSettings.Column_hasCover_index;
            columnHeader1_hasHistory.DisplayIndex = Program.UserSettings.Column_hasHistory_index;
            columnHeader1_hasInfo.DisplayIndex = Program.UserSettings.Column_hasInfo_index;
            columnHeader1_hasSnap.DisplayIndex = Program.UserSettings.Column_hasSnap_index;
            columnHeader1_hasManual.DisplayIndex = Program.UserSettings.Column_hasManual_index;
            columnHeader_year.DisplayIndex = Program.UserSettings.Column_year_index;
            columnHeader_video.DisplayIndex = Program.UserSettings.Column_video_index;
            columnHeader_sounds.DisplayIndex = Program.UserSettings.Column_sounds_index;

            fileTypeToolStripMenuItem.Checked = Program.UserSettings.Column_filetype_visible;
            playedTimeToolStripMenuItem.Checked = Program.UserSettings.Column_playedtime_visible;
            pathToolStripMenuItem.Checked = Program.UserSettings.Column_path_visible;
            ratingToolStripMenuItem.Checked = Program.UserSettings.Column_rating_visible;
            sizeToolStripMenuItem.Checked = Program.UserSettings.Column_Size_Visible;
            coverToolStripMenuItem.Checked = Program.UserSettings.Column_hasCover_visible;
            historyToolStripMenuItem.Checked = Program.UserSettings.Column_hasHistory_visible;
            infoToolStripMenuItem.Checked = Program.UserSettings.Column_hasInfo_visible;
            snapshotToolStripMenuItem.Checked = Program.UserSettings.Column_hasSnap_visible;
            manualToolStripMenuItem.Checked = Program.UserSettings.Column_hasManual_visible;
            yearToolStripMenuItem.Checked = Program.UserSettings.Column_year_visible;
            videoToolStripMenuItem.Checked = Program.UserSettings.Column_video_visible;
            soundsToolStripMenuItem.Checked = Program.UserSettings.Column_sounds_visible;

            videoPlayer1.AutoStart = Program.UserSettings.VideoAutoStart;
            soundPlayer1.AutoStart = Program.UserSettings.SoundAutoStart;

            //tabs
            tabControl_down_right.SelectedIndex = Program.UserSettings.tabindex_downRight;
            tabControl_up_right.SelectedIndex = Program.UserSettings.tabindex_upRight;
        }
        void LoadInitSettings(string[] args)
        {
            if (Program.Setting.RememberLastProject)
            {
                //open project
                if (Program.Setting.RecentProfiles != null)
                    if (Program.Setting.RecentProfiles.Count > 0)
                        OpenProfile(Program.Setting.RecentProfiles[0]);

                //remember selection
                if (args != null)
                {
                    if (args.Length > 0)
                    {
                        OpenProfile(Path.GetFullPath(args[0]));
                        return;
                    }
                }
                if (Program.Setting.RememberSelectedRom)
                {
                    if (Program.UserSettings.LastSelectedConsoleOrPlaylist)
                    {
                        if (treeView_consoles.Nodes.Count > Program.UserSettings.LastSelectedConsole)
                        {
                            treeView_consoles.SelectedNode = treeView_consoles.Nodes[Program.UserSettings.LastSelectedConsole];
                            Clear();
                            _ActiveElement = ActiveElement.ConsoleOrCategory;
                            _SortAlpha = false;
                            _SortMode = SortMode.None;
                            RefreshEmulatorFromConsole();
                            RefreshRomsFromConsole();
                            //select rom
                            if (listView_roms.Items.Count > Program.UserSettings.LastSelectedRom)
                            {
                                listView_roms.Items[Program.UserSettings.LastSelectedRom].Selected = true;
                                listView_roms.Items[Program.UserSettings.LastSelectedRom].EnsureVisible();
                                listView_roms.Select(); EnsureVissibleRom = true;
                            }
                            //select emulator
                            if (listView_emulators.Items.Count > Program.UserSettings.LastSelectedEmulator)
                            {
                                listView_emulators.Items[Program.UserSettings.LastSelectedEmulator].Selected = true;
                                listView_emulators.Items[Program.UserSettings.LastSelectedEmulator].EnsureVisible();
                            }
                        }
                    }
                    else
                    {
                        tabControl3.SelectedIndex = 1;
                        if (treeView_playlists.Nodes.Count > Program.UserSettings.LastSelectedConsole)
                        {
                            treeView_playlists.SelectedNode = treeView_playlists.Nodes[Program.UserSettings.LastSelectedConsole];
                            Clear();
                            _ActiveElement = ActiveElement.Playlist;
                            _SortAlpha = false;
                            _SortMode = SortMode.None;
                            RefreshRomsFromPlaylist();
                            //select rom
                            if (listView_roms.Items.Count > Program.UserSettings.LastSelectedRom)
                            {
                                listView_roms.Items[Program.UserSettings.LastSelectedRom].Selected = true;
                                listView_roms.Items[Program.UserSettings.LastSelectedRom].EnsureVisible(); EnsureVissibleRom = true;
                            }
                            //select emulator
                            if (listView_emulators.Items.Count > Program.UserSettings.LastSelectedEmulator)
                            {
                                listView_emulators.Items[Program.UserSettings.LastSelectedEmulator].Selected = true;
                                listView_emulators.Items[Program.UserSettings.LastSelectedEmulator].EnsureVisible();
                            }
                        }
                    }
                }
            }
        }
        void SaveUserSettings()
        {
            Program.UserSettings.win_size = this.Size;
            Program.UserSettings.win_location = this.Location;
            Program.UserSettings.splitContainer1 = splitContainer1.SplitterDistance;
            Program.UserSettings.splitContainer2 = splitContainer2.SplitterDistance;
            Program.UserSettings.splitContainer3 = splitContainer3.SplitterDistance;
            Program.UserSettings.splitContainer4 = splitContainer4.SplitterDistance;
            Program.UserSettings.splitContainer5 = splitContainer5.SplitterDistance;
            Program.UserSettings.consoleImagSize = trackBar1.Value;
            Program.UserSettings.ShowCommandlinesPanel = toolStripButton_commandlinesShowHide.Checked;
            Program.UserSettings.ListView = listView_roms.View;
            Program.UserSettings.ThunmbnailSize = trackBar4.Value;
            //columns
            Program.UserSettings.Column_name_width = columnHeader_name.Width;
            Program.UserSettings.Column_path_width = columnHeader_path.Width;
            Program.UserSettings.Column_Size_width = columnHeader_size.Width;
            Program.UserSettings.Column_filetype_width = columnHeader1_fileType.Width;
            Program.UserSettings.Column_playedtime_width = columnHeader1_playtime.Width;
            Program.UserSettings.Column_rating_width = columnHeader1_rating.Width;
            Program.UserSettings.Column_hasCover_width = columnHeader1_hasCover.Width;
            Program.UserSettings.Column_hasHistory_width = columnHeader1_hasHistory.Width;
            Program.UserSettings.Column_hasInfo_width = columnHeader1_hasInfo.Width;
            Program.UserSettings.Column_hasSnap_width = columnHeader1_hasSnap.Width;
            Program.UserSettings.Column_hasManual_width = columnHeader1_hasManual.Width;
            Program.UserSettings.Column_year_width = columnHeader_year.Width;
            Program.UserSettings.Column_video_width = columnHeader_video.Width;
            Program.UserSettings.Column_sounds_width = columnHeader_sounds.Width;

            Program.UserSettings.Column_path_Index = columnHeader_path.DisplayIndex;
            Program.UserSettings.Column_Size_Index = columnHeader_size.DisplayIndex;
            Program.UserSettings.Column_filetype_Index = columnHeader1_fileType.DisplayIndex;
            Program.UserSettings.Column_playedtime_Index = columnHeader1_playtime.DisplayIndex;
            Program.UserSettings.Column_rating_Index = columnHeader1_rating.DisplayIndex;
            Program.UserSettings.Column_hasCover_index = columnHeader1_hasCover.DisplayIndex;
            Program.UserSettings.Column_hasHistory_index = columnHeader1_hasHistory.DisplayIndex;
            Program.UserSettings.Column_hasInfo_index = columnHeader1_hasInfo.DisplayIndex;
            Program.UserSettings.Column_hasSnap_index = columnHeader1_hasSnap.DisplayIndex;
            Program.UserSettings.Column_hasManual_index = columnHeader1_hasManual.DisplayIndex;
            Program.UserSettings.Column_year_index = columnHeader_year.DisplayIndex;
            Program.UserSettings.Column_video_index = columnHeader_video.DisplayIndex;
            Program.UserSettings.Column_sounds_index = columnHeader_sounds.DisplayIndex;
            if (tabControl3.SelectedIndex == 0)
            {
                Program.UserSettings.LastSelectedConsoleOrPlaylist = true;
                if (treeView_consoles.SelectedNode != null)
                {
                    if (treeView_consoles.SelectedNode.GetType() == typeof(TreeNode_Console))
                        Program.UserSettings.LastSelectedConsole = treeView_consoles.SelectedNode.Index;
                    else
                        Program.UserSettings.LastSelectedConsole = treeView_consoles.SelectedNode.Parent.Index;
                }
            }
            else
            {
                Program.UserSettings.LastSelectedConsoleOrPlaylist = false;
                if (treeView_playlists.SelectedNode != null)
                {
                    Program.UserSettings.LastSelectedConsole = treeView_playlists.SelectedNode.Index;
                }
            }
            if (listView_emulators.SelectedItems.Count > 0)
                Program.UserSettings.LastSelectedEmulator = listView_emulators.SelectedItems[0].Index;
            if (listView_roms.SelectedItems.Count > 0)
                Program.UserSettings.LastSelectedRom = listView_roms.SelectedItems[0].Index;

            Program.UserSettings.VideoAutoStart = videoPlayer1.AutoStart;
            Program.UserSettings.SoundAutoStart = soundPlayer1.AutoStart;

            //tabs
            Program.UserSettings.tabindex_downRight = tabControl_down_right.SelectedIndex;
            Program.UserSettings.tabindex_upRight = tabControl_up_right.SelectedIndex;
            Program.UserSettings.Save();
        }
        void ClearTemp()
        {
            if (Program.UserSettings.ClearTemp)
            {
                if (Directory.Exists(Program.UserSettings.TempFolder))
                {
                    for (int i = 0; i < Directory.GetFiles(Program.UserSettings.TempFolder).Length; i++)
                    {
                        try
                        {
                            if (Program.UserSettings.ClearTempFolderExceptExtensions != null)
                            {
                                if (Program.UserSettings.ClearTempFolderExceptExtensions.Count > 0)
                                {
                                    bool deleteit = true;
                                    foreach (string xt in Program.UserSettings.ClearTempFolderExceptExtensions)
                                    {
                                        if (xt == Path.GetExtension(Directory.GetFiles(Program.UserSettings.TempFolder)[i]))
                                        {
                                            deleteit = false;
                                            break;
                                        }
                                    }
                                    if (deleteit)
                                    {
                                        File.Delete(Directory.GetFiles(Program.UserSettings.TempFolder)[i]);
                                        i = -1;
                                    }
                                }
                                else
                                {
                                    File.Delete(Directory.GetFiles(Program.UserSettings.TempFolder)[i]);
                                    i = -1;
                                }
                            }
                            else
                            {
                                File.Delete(Directory.GetFiles(Program.UserSettings.TempFolder)[i]);
                                i = -1;
                            }
                        }
                        catch { }
                    }
                }
            }
        }
        void RefreshColumns()
        {
            listView_roms.Visible = false;
            listView_roms.Columns.Clear();
            listView_roms.Columns.Add(columnHeader_name);
            for (int i = 0; i < 14; i++)
            {
                if (Program.UserSettings.Column_rating_Index == i)
                {
                    if (Program.UserSettings.Column_rating_visible)
                        listView_roms.Columns.Add(columnHeader1_rating);
                }
                else if (Program.UserSettings.Column_path_Index == i)
                {
                    if (Program.UserSettings.Column_path_visible)
                        listView_roms.Columns.Add(columnHeader_path);
                }
                else if (Program.UserSettings.Column_Size_Index == i)
                {
                    if (Program.UserSettings.Column_Size_Visible)
                        listView_roms.Columns.Add(columnHeader_size);
                }
                else if (Program.UserSettings.Column_playedtime_Index == i)
                {
                    if (Program.UserSettings.Column_playedtime_visible)
                        listView_roms.Columns.Add(columnHeader1_playtime);
                }
                else if (Program.UserSettings.Column_filetype_Index == i)
                {
                    if (Program.UserSettings.Column_filetype_visible)
                        listView_roms.Columns.Add(columnHeader1_fileType);
                }
                else if (Program.UserSettings.Column_hasCover_index == i)
                {
                    if (Program.UserSettings.Column_hasCover_visible)
                        listView_roms.Columns.Add(columnHeader1_hasCover);
                }
                else if (Program.UserSettings.Column_hasHistory_index == i)
                {
                    if (Program.UserSettings.Column_hasHistory_visible)
                        listView_roms.Columns.Add(columnHeader1_hasHistory);
                }
                else if (Program.UserSettings.Column_hasInfo_index == i)
                {
                    if (Program.UserSettings.Column_hasInfo_visible)
                        listView_roms.Columns.Add(columnHeader1_hasInfo);
                }
                else if (Program.UserSettings.Column_hasSnap_index == i)
                {
                    if (Program.UserSettings.Column_hasSnap_visible)
                        listView_roms.Columns.Add(columnHeader1_hasSnap);
                }
                else if (Program.UserSettings.Column_hasManual_index == i)
                {
                    if (Program.UserSettings.Column_hasManual_visible)
                        listView_roms.Columns.Add(columnHeader1_hasManual);
                }
                else if (Program.UserSettings.Column_year_index == i)
                {
                    if (Program.UserSettings.Column_year_visible)
                        listView_roms.Columns.Add(columnHeader_year);
                }
                else if (Program.UserSettings.Column_video_index == i)
                {
                    if (Program.UserSettings.Column_video_visible)
                        listView_roms.Columns.Add(columnHeader_video);
                }
                else if (Program.UserSettings.Column_sounds_index == i)
                {
                    if (Program.UserSettings.Column_sounds_visible)
                        listView_roms.Columns.Add(columnHeader_sounds);
                }
            }
            if (listView_roms.Columns.Count < 14)
            {
                if (Program.UserSettings.Column_rating_visible & !listView_roms.Columns.Contains(columnHeader1_rating))
                    listView_roms.Columns.Add(columnHeader1_rating);
                if (Program.UserSettings.Column_path_visible & !listView_roms.Columns.Contains(columnHeader_path))
                    listView_roms.Columns.Add(columnHeader_path);
                if (Program.UserSettings.Column_Size_Visible & !listView_roms.Columns.Contains(columnHeader_size))
                    listView_roms.Columns.Add(columnHeader_size);
                if (Program.UserSettings.Column_playedtime_visible & !listView_roms.Columns.Contains(columnHeader1_playtime))
                    listView_roms.Columns.Add(columnHeader1_playtime);
                if (Program.UserSettings.Column_filetype_visible & !listView_roms.Columns.Contains(columnHeader1_fileType))
                    listView_roms.Columns.Add(columnHeader1_fileType);
                if (Program.UserSettings.Column_hasCover_visible & !listView_roms.Columns.Contains(columnHeader1_hasCover))
                    listView_roms.Columns.Add(columnHeader1_hasCover);
                if (Program.UserSettings.Column_hasHistory_visible & !listView_roms.Columns.Contains(columnHeader1_hasHistory))
                    listView_roms.Columns.Add(columnHeader1_hasHistory);
                if (Program.UserSettings.Column_hasInfo_visible & !listView_roms.Columns.Contains(columnHeader1_hasInfo))
                    listView_roms.Columns.Add(columnHeader1_hasInfo);
                if (Program.UserSettings.Column_hasSnap_visible & !listView_roms.Columns.Contains(columnHeader1_hasSnap))
                    listView_roms.Columns.Add(columnHeader1_hasSnap);
                if (Program.UserSettings.Column_hasManual_visible & !listView_roms.Columns.Contains(columnHeader1_hasManual))
                    listView_roms.Columns.Add(columnHeader1_hasManual);
                if (Program.UserSettings.Column_year_visible & !listView_roms.Columns.Contains(columnHeader_year))
                    listView_roms.Columns.Add(columnHeader_year);
                if (Program.UserSettings.Column_video_visible & !listView_roms.Columns.Contains(columnHeader_video))
                    listView_roms.Columns.Add(columnHeader_video);
                if (Program.UserSettings.Column_sounds_visible & !listView_roms.Columns.Contains(columnHeader_sounds))
                    listView_roms.Columns.Add(columnHeader_sounds);
            }
            listView_roms.Visible = true;
        }

        void RefreshConsoles()
        {
            treeView_consoles.Nodes.Clear();
            foreach (EmuConsole console in Program.Project.Consoles)
            {
                TreeNode_Console con = new TreeNode_Console();
                con.Console = console;
                con.RefreshCategories();
                treeView_consoles.Nodes.Add(con);
                con.RefreshCategories();
            }
        }
        void RefreshPlaylists()
        {
            treeView_playlists.Nodes.Clear();
            foreach (PlayList playlist in Program.Project.PlayLists)
            {
                TreeNode_PlayList TR = new TreeNode_PlayList();
                TR.PlayList = playlist;
                treeView_playlists.Nodes.Add(TR);
            }
        }
        void RefreshEmulatorFromConsole()
        {
            TreeNode_Console TR = (TreeNode_Console)treeView_consoles.SelectedNode;
            foreach (Emulator emu in TR.Console.Emulators)
            {
                ListViewItem_Emulator IT = new ListViewItem_Emulator();
                IT.Emu = emu;
                listView_emulators.Items.Add(IT);
            }
            if (listView_emulators.Items.Count > 0)
                listView_emulators.Items[0].Selected = true;
            _ActiveElement = ActiveElement.ConsoleOrCategory;
        }
        void RefreshRomsFromConsole()
        {
            listView_roms.Visible = false;
            IsLoadingRoms = true;
            foreach (Rom rom in ((TreeNode_Console)treeView_consoles.SelectedNode).Console.Roms)
            {
                ListViewItem_Rom IT = new ListViewItem_Rom();
                IT.ROM = rom;
                listView_roms.Items.Add(IT);
            }
            IsLoadingRoms = false;
            listView_roms.Visible = true;
            listView_roms.Select();
        }
        void RefreshEmulatorFromCategory()
        {
            TreeNode_Console TR = (TreeNode_Console)treeView_consoles.SelectedNode.Parent;
            foreach (Emulator emu in TR.Console.Emulators)
            {
                ListViewItem_Emulator IT = new ListViewItem_Emulator();
                IT.Emu = emu;
                listView_emulators.Items.Add(IT);
            }
            if (listView_emulators.Items.Count > 0)
                listView_emulators.Items[0].Selected = true;
            _ActiveElement = ActiveElement.ConsoleOrCategory;
        }
        void RefreshRomsFromCategory()
        {
            listView_roms.Visible = false;
            IsLoadingRoms = true;
            TreeNode_Console TR = (TreeNode_Console)treeView_consoles.SelectedNode.Parent;
            TreeNode_Category CC = (TreeNode_Category)treeView_consoles.SelectedNode;
            foreach (Rom ro in TR.Console.Roms)
            {
                if (ro.Categories == null)
                    ro.Categories = new List<string>();
                foreach (string Cat in ro.Categories)
                {
                    if (CC.CategoryIn.Name == Cat)
                    {
                        ListViewItem_Rom R = new ListViewItem_Rom();
                        R.ROM = ro;
                        listView_roms.Items.Add(R);
                    }
                }
            }
            IsLoadingRoms = false; listView_roms.Visible = true;
        }
        void RefreshRomsFromPlaylist()
        {
            listView_roms.Visible = false;
            IsLoadingRoms = true;
            foreach (PlayListItem rom in ((TreeNode_PlayList)treeView_playlists.SelectedNode).PlayList.PlayListItems)
            {
                ListViewItem_PlayListItem IT = new ListViewItem_PlayListItem();
                IT.PlayListItem = rom;
                listView_roms.Items.Add(IT);
            }
            IsLoadingRoms = false;
            listView_roms.Visible = true;
            _ActiveElement = ActiveElement.RomOrPlaylistItem;
        }

        void RunRom()
        {
            //1 get rom path
            string romPath = "";
            string EmulatorPath = "";//Null=no emulator, run rom using system
            string Commandlines = "";//null=no commandlines, use rom path
            if (listView_roms.SelectedItems.Count == 1)
            {
                if (listView_roms.SelectedItems[0].GetType() == typeof(ListViewItem_Rom))
                {
                    if (((ListViewItem_Rom)listView_roms.SelectedItems[0]).ROM.IsURL)
                    {
                        Frm_RomDownload download = new Frm_RomDownload(((ListViewItem_Rom)listView_roms.SelectedItems[0]).ROM.URL,
                            Utilities.GetFullPath(Program.UserSettings.TempFolder) + "\\" + ((ListViewItem_Rom)listView_roms.SelectedItems[0]).ROM.DownloadFileName);
                        download.ShowDialog(this);
                        if (download.OK)
                        {
                            if (treeView_consoles.SelectedNode.GetType() == typeof(TreeNode_Console))
                            {
                                if (((TreeNode_Console)treeView_consoles.SelectedNode).Console.CopyRomToDownloadsFolder)
                                {
                                    try
                                    {
                                        Directory.CreateDirectory(((TreeNode_Console)treeView_consoles.SelectedNode).Console.DownloadsFolder);
                                        File.Copy(Utilities.GetFullPath(Program.UserSettings.TempFolder) + "\\" + ((ListViewItem_Rom)listView_roms.SelectedItems[0]).ROM.DownloadFileName,
                                            ((TreeNode_Console)treeView_consoles.SelectedNode).Console.DownloadsFolder + "\\" + ((ListViewItem_Rom)listView_roms.SelectedItems[0]).ROM.DownloadFileName);
                                        ((ListViewItem_Rom)listView_roms.SelectedItems[0]).ROM.Path = Utilities.GetDotPath(((TreeNode_Console)treeView_consoles.SelectedNode).Console.DownloadsFolder + "\\" + ((ListViewItem_Rom)listView_roms.SelectedItems[0]).ROM.DownloadFileName);
                                        ((ListViewItem_Rom)listView_roms.SelectedItems[0]).ROM.RomSize = Utilities.GetFileSize(((TreeNode_Console)treeView_consoles.SelectedNode).Console.DownloadsFolder + "\\" + ((ListViewItem_Rom)listView_roms.SelectedItems[0]).ROM.DownloadFileName);
                                        ((ListViewItem_Rom)listView_roms.SelectedItems[0]).ROM.IsURL = false;
                                        Save = true;
                                        RunRom();
                                        return;
                                    }
                                    catch
                                    {
                                        MessageBox.Show("Unable to copy rom to the console downloads folder, please specify downloads folder in the console's properties, press F4.");
                                        romPath = Utilities.GetFullPath(Program.UserSettings.TempFolder) + "\\" + ((ListViewItem_Rom)listView_roms.SelectedItems[0]).ROM.DownloadFileName;
                                    }
                                }
                                else
                                {
                                    romPath = Utilities.GetFullPath(Program.UserSettings.TempFolder) + "\\" + ((ListViewItem_Rom)listView_roms.SelectedItems[0]).ROM.DownloadFileName;
                                }
                            }
                            else//category
                            {
                                if (((TreeNode_Console)treeView_consoles.SelectedNode.Parent).Console.CopyRomToDownloadsFolder)
                                {
                                    try
                                    {
                                        Directory.CreateDirectory(((TreeNode_Console)treeView_consoles.SelectedNode.Parent).Console.DownloadsFolder);
                                        File.Copy(Utilities.GetFullPath(Program.UserSettings.TempFolder) + "\\" + ((ListViewItem_Rom)listView_roms.SelectedItems[0]).ROM.DownloadFileName,
                                            ((TreeNode_Console)treeView_consoles.SelectedNode.Parent).Console.DownloadsFolder + "\\" + ((ListViewItem_Rom)listView_roms.SelectedItems[0]).ROM.DownloadFileName);
                                        ((ListViewItem_Rom)listView_roms.SelectedItems[0]).ROM.Path = Utilities.GetDotPath(((TreeNode_Console)treeView_consoles.SelectedNode.Parent).Console.DownloadsFolder + "\\" + ((ListViewItem_Rom)listView_roms.SelectedItems[0]).ROM.DownloadFileName);
                                        ((ListViewItem_Rom)listView_roms.SelectedItems[0]).ROM.RomSize = Utilities.GetFileSize(((TreeNode_Console)treeView_consoles.SelectedNode.Parent).Console.DownloadsFolder + "\\" + ((ListViewItem_Rom)listView_roms.SelectedItems[0]).ROM.DownloadFileName);
                                        ((ListViewItem_Rom)listView_roms.SelectedItems[0]).ROM.IsURL = false;
                                        Save = true;
                                        RunRom();
                                        return;
                                    }
                                    catch
                                    {
                                        MessageBox.Show("Unable to copy rom to the console downloads folder, please specify downloads folder in the console's properties, press F4.");
                                        romPath = Utilities.GetFullPath(Program.UserSettings.TempFolder) + "\\" + ((ListViewItem_Rom)listView_roms.SelectedItems[0]).ROM.DownloadFileName;
                                    }
                                }
                                else
                                {
                                    romPath = Utilities.GetFullPath(Program.UserSettings.TempFolder) + "\\" + ((ListViewItem_Rom)listView_roms.SelectedItems[0]).ROM.DownloadFileName;
                                }
                            }
                        }
                        else
                            return;
                    }
                    else if (File.Exists(Utilities.GetFullPath(((ListViewItem_Rom)listView_roms.SelectedItems[0]).ROM.Path)))
                    {
                        romPath = (Utilities.GetFullPath(((ListViewItem_Rom)listView_roms.SelectedItems[0]).ROM.Path));
                        ((ListViewItem_Rom)listView_roms.SelectedItems[0]).ROM.PlayedTimes++;
                    }
                    else
                    {
                        MessageBox.Show("Rom file isn't exist on the local disk!!");
                        return;
                    }
                }
                else if (listView_roms.SelectedItems[0].GetType() == typeof(ListViewItem_PlayListItem))
                {
                    if (((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Rom.IsURL)
                    {
                        Frm_RomDownload download = new Frm_RomDownload(((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Rom.URL,
                            Utilities.GetFullPath(Program.UserSettings.TempFolder) + "\\" + ((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Rom.DownloadFileName);
                        download.ShowDialog(this);
                        if (download.OK)
                        {
                            if (((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Console.CopyRomToDownloadsFolder)
                            {
                                try
                                {
                                    Directory.CreateDirectory(((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Console.DownloadsFolder);
                                    File.Copy(Utilities.GetFullPath(Program.UserSettings.TempFolder) + "\\" + ((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Rom.DownloadFileName,
                                        ((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Console.DownloadsFolder + "\\" + ((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Rom.DownloadFileName);
                                    ((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Rom.Path = Utilities.GetDotPath(((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Console.DownloadsFolder + "\\" + ((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Rom.DownloadFileName);
                                    ((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Rom.RomSize = Utilities.GetFileSize(((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Console.DownloadsFolder + "\\" + ((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Rom.DownloadFileName);
                                    ((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Rom.IsURL = false;
                                    Save = true;
                                    RunRom();
                                    return;
                                }
                                catch
                                {
                                    MessageBox.Show("Unable to copy rom to the console downloads folder, please specify downloads folder in the console's properties, press F4.");
                                    romPath = Utilities.GetFullPath(Program.UserSettings.TempFolder) + "\\" + ((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Rom.DownloadFileName;
                                }
                            }
                            else
                            {
                                romPath = Utilities.GetFullPath(Program.UserSettings.TempFolder) + "\\" + ((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Rom.DownloadFileName;
                            }
                        }
                        else
                            return;
                    }
                    else if (File.Exists(Utilities.GetFullPath(((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Rom.Path)))
                    {
                        romPath = (Utilities.GetFullPath(((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Rom.Path));
                        ((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Rom.PlayedTimes++;
                    }
                    else
                    {
                        MessageBox.Show("Rom file isn't exist on the local disk!!");
                        return;
                    }
                }
            }
            else
            {
                MessageBox.Show("Please select rom first !!");
                return;
            }

            //2 Get emulator path and commandlines
            if (toolStripButton8_EnableEmulator.Checked)
            {
                if (listView_emulators.SelectedItems.Count == 1)
                {
                    if (File.Exists(Utilities.GetFullPath(((ListViewItem_Emulator)listView_emulators.SelectedItems[0]).Emu.Path)))
                    {
                        EmulatorPath = (Utilities.GetFullPath(((ListViewItem_Emulator)listView_emulators.SelectedItems[0]).Emu.Path));
                    }
                    else
                    {
                        MessageBox.Show("Emulator isn't exist !!");
                        return;
                    }
                    //Extract archive
                    if (Program.UserSettings.ExtractArchiveFirst)
                    {
                        try
                        {
                            Directory.CreateDirectory(Utilities.GetFullPath(Program.UserSettings.TempFolder));
                            EXTRACTOR = new SevenZip.SevenZipExtractor(romPath);
                            if (EXTRACTOR.ArchiveFileData.Count == 1)
                            {
                                if (EXTRACTOR.ArchiveFileData[0].Encrypted)
                                {
                                    Frm_EnterName password = new Frm_EnterName("Please enter archive's password", "");
                                    password.textBox1.UseSystemPasswordChar = true;
                                    if (password.ShowDialog(this) == System.Windows.Forms.DialogResult.OK)
                                    {
                                        try
                                        {
                                            Frm_Extractor extr = new Frm_Extractor(romPath, password.NameEntered, Program.UserSettings.TempFolder, false, null);
                                            extr.ShowDialog(this);
                                            if (extr.Canceld)
                                                return;
                                            romPath = Utilities.GetFullPath(Program.UserSettings.TempFolder + "\\" + EXTRACTOR.ArchiveFileData[0].FileName);
                                            FileInfo inf = new FileInfo(romPath);
                                            if (inf.Length == 0)
                                            {
                                                MessageBox.Show("Can't extract the file, BAD password or file is damaged");
                                                return;
                                            }
                                        }
                                        catch
                                        {
                                            MessageBox.Show("Can't extract the file, BAD password or file is damaged");
                                            return;
                                        }
                                    }
                                    else
                                    {
                                        return;
                                    }
                                }
                                else
                                {
                                    //EXTRACTOR.ExtractArchive(Program.UserSettings.TempFolder);
                                    Frm_Extractor extr = new Frm_Extractor(romPath, null, Program.UserSettings.TempFolder, false, null);
                                    extr.ShowDialog(this);
                                    if (extr.Canceld)
                                        return;
                                    romPath = Utilities.GetFullPath(Program.UserSettings.TempFolder + "\\" + EXTRACTOR.ArchiveFileData[0].FileName);
                                }
                            }
                            else
                            {
                                List<string> filenames = new List<string>();
                                foreach (SevenZip.ArchiveFileInfo file in EXTRACTOR.ArchiveFileData)
                                {
                                    filenames.Add(file.FileName);
                                }
                                Frm_ArchiveFiles ar = new Frm_ArchiveFiles(filenames.ToArray());
                                ar.ShowDialog(this);
                                if (ar.OK)
                                {
                                    if (EXTRACTOR.ArchiveFileData[ar.SelectedRomIndex].Encrypted)
                                    {
                                        Frm_EnterName password = new Frm_EnterName("Please enter archive's password", "");
                                        password.textBox1.UseSystemPasswordChar = true;
                                        if (password.ShowDialog(this) == System.Windows.Forms.DialogResult.OK)
                                        {
                                            try
                                            {
                                                string[] fil = { ar.SelectedRom };
                                                //EXTRACTOR = new SevenZip.SevenZipExtractor(ROMPATH, password.NameEntered);
                                                //EXTRACTOR.ExtractFiles(Program.UserSettings.TempFolder, fil);
                                                Frm_Extractor extr = new Frm_Extractor(romPath, password.NameEntered, Program.UserSettings.TempFolder, true, fil);
                                                extr.ShowDialog(this);
                                                if (extr.Canceld)
                                                    return;
                                                romPath = Utilities.GetFullPath(Program.UserSettings.TempFolder + "\\" + EXTRACTOR.ArchiveFileData[ar.SelectedRomIndex].FileName);
                                                FileInfo inf = new FileInfo(romPath);
                                                if (inf.Length == 0)
                                                {
                                                    MessageBox.Show("Can't extract the file, BAD password or file is damaged");
                                                    return;
                                                }
                                            }
                                            catch
                                            {
                                                MessageBox.Show("Can't extract the file, BAD password or file is damaged");
                                                return;
                                            }
                                        }
                                        else
                                        {
                                            return;
                                        }
                                    }
                                    else
                                    {
                                        string[] fil = { ar.SelectedRom };
                                        //EXTRACTOR.ExtractFiles(Program.UserSettings.TempFolder, fil);
                                        Frm_Extractor extr = new Frm_Extractor(romPath, null, Program.UserSettings.TempFolder, true, fil);
                                        extr.ShowDialog(this);
                                        if (extr.Canceld)
                                            return;
                                        romPath = Utilities.GetFullPath(Program.UserSettings.TempFolder + "\\" + ar.SelectedRom);
                                    }
                                }
                                else
                                { return; }
                            }

                        }
                        catch
                        {

                        }
                    }
                    //commandlines
                    if (toolStripButton7_EnableCommandlines.Checked)
                    {
                        //Normal commandlines
                        if (toolStripButton5_UseEmulatorCommandlines.Checked)
                        {
                            if (((ListViewItem_Emulator)listView_emulators.SelectedItems[0]).Emu.CommandLinesGroups == null)
                                ((ListViewItem_Emulator)listView_emulators.SelectedItems[0]).Emu.CommandLinesGroups = new List<CommandLinesGroup>();
                            bool found = false;
                            foreach (CommandLinesGroup gr in ((ListViewItem_Emulator)listView_emulators.SelectedItems[0]).Emu.CommandLinesGroups)
                            {
                                if (gr.Name == "<Default>")
                                { found = true; break; }
                            }
                            if (!found)
                            {
                                CommandLinesGroup gro = new CommandLinesGroup();
                                CommandLine cm = new CommandLine();
                                cm.commandLine = "<RomPath>";
                                cm.Name = "Rom Path";
                                cm.Enabled = true;

                                gro.Name = "<Default>";
                                gro.Enabled = true;
                                gro.CommandLines.Add(cm);
                                ((ListViewItem_Emulator)listView_emulators.SelectedItems[0]).Emu.CommandLinesGroups.Add(gro);
                            }

                            foreach (CommandLinesGroup GR in ((ListViewItem_Emulator)listView_emulators.SelectedItems[0]).Emu.CommandLinesGroups)
                            {
                                if (GR.Enabled)
                                {
                                    foreach (CommandLine CM in GR.CommandLines)
                                    {
                                        if (CM.Enabled)
                                        {
                                            if (CM.commandLine.Substring(0, 1) == "<")
                                            {
                                                if (CM.commandLine.ToLower() == "<rompath>")
                                                { Commandlines += "\"" + romPath + "\"" + " "; }
                                                else if (CM.commandLine.ToLower().Substring(0, "<statesave>".Length)
                                                     == "<statesave>")
                                                {
                                                    try
                                                    {
                                                        int stateNumber = Convert.ToInt32(CM.commandLine.ToLower().Substring("<statesave>".Length, CM.commandLine.Length - "<statesave>".Length));
                                                        Commandlines += "\"" + ((ListViewItem_Rom)listView_roms.SelectedItems[0]).ROM.StateSaves[stateNumber] + "\"" + " ";
                                                    }
                                                    catch
                                                    { }
                                                }
                                            }
                                            else
                                                Commandlines += "\"" + CM.commandLine + "\"" + " ";
                                            foreach (Parameter PA in CM.Parameters)
                                            {
                                                if (PA.Enabled)
                                                {
                                                    if (PA.parameter.Substring(0, 1) == "<")
                                                    {
                                                        switch (PA.parameter.ToLower())
                                                        {
                                                            case "<rompath>": Commandlines += "\"" + romPath + "\"" + " "; break;
                                                        }
                                                    }
                                                    else
                                                        Commandlines += "\"" + PA.parameter + "\"" + " ";
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                        }
                        else//Rom commandlines
                        {
                            Rom ROM;
                            if (listView_roms.SelectedItems[0].GetType() == typeof(ListViewItem_Rom))
                                ROM = ((ListViewItem_Rom)listView_roms.SelectedItems[0]).ROM;
                            else
                                ROM = ((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Rom;
                            if (ROM.CommandLineGroupsEmulator != null)
                            {
                                bool Found = false;
                                foreach (CommandLineGroupsEmulator GR in ROM.CommandLineGroupsEmulator)
                                {
                                    if (GR.Emulator == ((ListViewItem_Emulator)listView_emulators.SelectedItems[0]).Emu)
                                    {
                                        Found = true;
                                        if (GR.CommandLinesGroups == null)
                                            GR.CommandLinesGroups = new List<CommandLinesGroup>();
                                        bool found = false;
                                        foreach (CommandLinesGroup gr in GR.CommandLinesGroups)
                                        {
                                            if (gr.Name == "<Default>")
                                            { found = true; break; }
                                        }
                                        if (!found)
                                        {
                                            CommandLinesGroup gro = new CommandLinesGroup();
                                            CommandLine cm = new CommandLine();
                                            cm.commandLine = "<RomPath>";
                                            cm.Name = "Rom Path";
                                            cm.Enabled = true;

                                            gro.Name = "<Default>";
                                            gro.Enabled = true;
                                            gro.CommandLines.Add(cm);
                                            GR.CommandLinesGroups.Add(gro);
                                        }

                                        foreach (CommandLinesGroup gr in GR.CommandLinesGroups)
                                        {
                                            if (gr.Enabled)
                                            {
                                                foreach (CommandLine CM in gr.CommandLines)
                                                {
                                                    if (CM.Enabled)
                                                    {
                                                        if (CM.commandLine.Substring(0, 1) == "<")
                                                        {
                                                            if (CM.commandLine.ToLower() == "<rompath>")
                                                            { Commandlines += "\"" + romPath + "\"" + " "; }
                                                            else if (CM.commandLine.ToLower().Substring(0, "<statesave>".Length)
                                                                 == "<statesave>")
                                                            {
                                                                try
                                                                {
                                                                    int stateNumber = Convert.ToInt32(CM.commandLine.ToLower().Substring("<statesave>".Length, CM.commandLine.Length - "<statesave>".Length));
                                                                    Commandlines += "\"" + ((ListViewItem_Rom)listView_roms.SelectedItems[0]).ROM.StateSaves[stateNumber] + "\"" + " ";
                                                                }
                                                                catch
                                                                { }
                                                            }
                                                        }
                                                        else
                                                            Commandlines += "\"" + CM.commandLine + "\"" + " ";
                                                        foreach (Parameter PA in CM.Parameters)
                                                        {
                                                            if (PA.Enabled)
                                                            {
                                                                if (PA.parameter.Substring(0, 1) == "<")
                                                                {
                                                                    switch (PA.parameter.ToLower())
                                                                    {
                                                                        case "<rompath>": Commandlines += "\"" + romPath + "\"" + " "; break;
                                                                    }
                                                                }
                                                                else
                                                                    Commandlines += "\"" + PA.parameter + "\"" + " ";
                                                            }
                                                        }
                                                    }
                                                }
                                            }
                                        }
                                        break;
                                    }
                                }
                                if (!Found)
                                {
                                    Commandlines = null;
                                }
                            }
                        }
                    }
                    else
                    {
                        Commandlines = null;
                    }
                }
                else
                {
                    MessageBox.Show("Please select emulator first !!");
                    return;
                }
            }
            else
            {
                EmulatorPath = null;
            }

            //Run !!
            EmuProccess = new Process();
            if (EmulatorPath == null)//No emulator
            {
                if (Path.GetDirectoryName(romPath) != "")
                    EmuProccess.StartInfo.WorkingDirectory = Path.GetDirectoryName(romPath);
                else
                    EmuProccess.StartInfo.WorkingDirectory = Path.GetPathRoot(romPath);
                EmuProccess.StartInfo.FileName = romPath;
            }
            else
            {
                if (Commandlines != null)
                {
                    EmuProccess.StartInfo.Arguments = Commandlines.Substring(0, Commandlines.Length - 1);
                }
                else
                {
                    EmuProccess.StartInfo.Arguments = "\"" + romPath + "\"";
                }
                EmuProccess.StartInfo.WorkingDirectory = Path.GetDirectoryName(EmulatorPath);
                EmuProccess.StartInfo.FileName = EmulatorPath;
            }
            EmuProccess.Start();
            if (Program.UserSettings.AutoMinimize)
            {
                this.WindowState = FormWindowState.Minimized;
                timer1_autominimize.Start();
            }
        }

        /// <summary>
        /// Clear everything like consoles, roms, emulators ... from the lists
        /// </summary>
        void ClearAll()
        {
            treeView_playlists.Nodes.Clear();
            treeView_consoles.Nodes.Clear();
            Clear();
        }
        /// <summary>
        /// Clear emulators and roms lists
        /// </summary>
        void Clear()
        {
            ItemSearched = false;
            listView_emulators.Items.Clear();
            listView_roms.Items.Clear();
            commandlinesEditor1.CommandLinesGroups = null;
            romStatus1.Rom = null;
            pdFviewer1.PDFPath = "";
            videoPlayer1.Rom = null;
            soundPlayer1.Rom = null;
            infoEditor1_info.RomName = infoEditor1_history.RomName = null;
            treeView1.Nodes.Clear();
        }
        bool AskToSave()
        {
            if (!Save)
            { return true; }
            DialogResult Dia = MessageBox.Show("Do you want to save profile first ?", "Save Profile", MessageBoxButtons.YesNoCancel, MessageBoxIcon.Question);
            if (Dia == DialogResult.Yes)
            {
                if (File.Exists(Program.ProjectPath))
                {
                    if (Program.SaveProject(Program.ProjectPath))
                    {
                        Save = false;
                        return true;
                    }
                }
                SaveFileDialog Sav = new SaveFileDialog();
                Sav.Title = "Save profile";
                Sav.Filter = "Emulators Organizer Profile file (*.eop)|*.eop";
                if (Sav.ShowDialog(this) == DialogResult.OK)
                {
                    if (Program.SaveProject(Sav.FileName))
                    {
                        Save = false;
                        return true;
                    }
                }
                else
                { return false; }
            }
            if (Dia == DialogResult.No)
            {
                return true;
            }
            return false;
        }
        void OpenProfile(string FilePath)
        {
            if (Program.LoadProject(FilePath))
            {
                ClearAll();
                RefreshConsoles();
                RefreshPlaylists();
                AddRecent(FilePath);
                this.Text = Program.Project.Name + " - " + "Emulators Organizer";
            }
        }
        void AddRecent(string Rec)
        {
            if (Path.GetDirectoryName(Rec).Length >= Program.StartUpDir.Length)
            {
                if (Path.GetDirectoryName(Rec).Substring(0, Program.StartUpDir.Length) == Program.StartUpDir)
                {
                    Rec = "." + Rec.Substring(Program.StartUpDir.Length);
                }
            }
            //insert it in the first
            Program.Setting.RecentProfiles.Insert(0, Rec);
            //remove the same one
            for (int i = 1; i < Program.Setting.RecentProfiles.Count; i++)
            {
                if (Program.Setting.RecentProfiles[i] == Rec)
                {
                    Program.Setting.RecentProfiles.RemoveAt(i);
                    break;
                }
            }
            //Limit to 6 elements
            if (Program.Setting.RecentProfiles.Count == 7)
                Program.Setting.RecentProfiles.RemoveAt(6);
            //refresh
            RefreshRecents();
        }
        void RefreshRecents()
        {
            try
            {
                recentProfilesToolStripMenuItem.DropDownItems.Clear();
                foreach (string rec in Program.Setting.RecentProfiles)
                {
                    recentProfilesToolStripMenuItem.DropDownItems.Add(rec);
                }
            }
            catch { }
        }
        void CheckCommandlinesPanel(bool LoadEmu)
        {
            commandlinesEditor1.CommandLinesGroups = null;
            //use rom commandlines enable
            if (listView_roms.SelectedItems.Count == 1)
            {
                if (listView_roms.SelectedItems[0].GetType() == typeof(ListViewItem_Rom))
                {
                    toolStripButton6_UseRomCommandlines.Enabled = ((ListViewItem_Rom)listView_roms.SelectedItems[0]).ROM.HasSpecialCommandLine;
                    if (!LoadEmu)
                    {
                        if (listView_emulators.SelectedItems.Count == 1)
                        {
                            foreach (CommandLineGroupsEmulator GR in ((ListViewItem_Rom)listView_roms.SelectedItems[0]).ROM.CommandLineGroupsEmulator)
                            {
                                if (GR.Emulator == ((ListViewItem_Emulator)listView_emulators.SelectedItems[0]).Emu)
                                {
                                    commandlinesEditor1.CommandLinesGroups = GR.CommandLinesGroups;
                                    break;
                                }
                            }
                        }
                    }
                }
                else
                {
                    toolStripButton6_UseRomCommandlines.Enabled = ((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Rom.HasSpecialCommandLine;
                    if (!LoadEmu)
                    {
                        if (listView_emulators.SelectedItems.Count == 1)
                        {
                            foreach (CommandLineGroupsEmulator GR in ((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Rom.CommandLineGroupsEmulator)
                            {
                                if (GR.Emulator == ((ListViewItem_Emulator)listView_emulators.SelectedItems[0]).Emu)
                                {
                                    commandlinesEditor1.CommandLinesGroups = GR.CommandLinesGroups;
                                    break;
                                }
                            }
                        }
                    }
                }
            }
            //check
            // toolStripButton5_UseEmulatorCommandlines.Checked = true;
            // toolStripButton6_UseRomCommandlines.Checked = false;
            //commandlines fill
            if (LoadEmu)
                if (listView_emulators.SelectedItems.Count == 1)
                    commandlinesEditor1.CommandLinesGroups = ((ListViewItem_Emulator)listView_emulators.SelectedItems[0]).Emu.CommandLinesGroups;
        }
        void ViewChanged()
        {
            panel_romslarge.Visible = listView_roms.View == View.LargeIcon;
        }
        void SortElements()
        {
            if (tabControl3.SelectedIndex == 0)
            {
                if (treeView_consoles.SelectedNode == null)
                    return;
                if (treeView_consoles.SelectedNode.GetType() == typeof(TreeNode_Console))
                {
                    EmuConsole console = ((TreeNode_Console)treeView_consoles.SelectedNode).Console;
                    console.Roms.Sort(new RomComparer(_SortAlpha, _SortMode));
                    Clear();
                    //1 refresh emulators
                    RefreshEmulatorFromConsole();
                    //2 refresh roms
                    RefreshRomsFromConsole();
                    Save = true;
                }
                else
                {
                    EmuConsole console = ((TreeNode_Console)treeView_consoles.SelectedNode.Parent).Console;
                    console.Roms.Sort(new RomComparer(_SortAlpha, _SortMode));
                    Clear();
                    RefreshEmulatorFromCategory();
                    RefreshRomsFromCategory();
                    Save = true;
                }
            }
            else
            {
                if (treeView_playlists.SelectedNode == null)
                    return;
                PlayList pl = ((TreeNode_PlayList)treeView_playlists.SelectedNode).PlayList;
                pl.PlayListItems.Sort(new PlaylistItemComparer(_SortAlpha, _SortMode));
                Clear();
                RefreshRomsFromPlaylist();
                Save = true;
            }
            ApplySortChecks();
        }
        void ApplySortChecks()
        {
            fileTypeToolStripMenuItem2.Checked = fileTypeToolStripMenuItem1.Checked = _SortMode == SortMode.FileType;
            nameToolStripMenuItem1.Checked = nameToolStripMenuItem.Checked = _SortMode == SortMode.Name;
            pathToolStripMenuItem2.Checked = pathToolStripMenuItem1.Checked = _SortMode == SortMode.Path;
            playedTimesToolStripMenuItem1.Checked = playedTimesToolStripMenuItem.Checked = _SortMode == SortMode.PlayedTime;
            ratingToolStripMenuItem2.Checked = ratingToolStripMenuItem1.Checked = _SortMode == SortMode.Rating;
            sizeToolStripMenuItem2.Checked = sizeToolStripMenuItem1.Checked = _SortMode == SortMode.Size;
        }
        private void Frm_main_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (AskToSave())
            {
                if ((this.WindowState == FormWindowState.Maximized) | (this.WindowState == FormWindowState.Minimized))
                    this.WindowState = FormWindowState.Normal;
                SaveUserSettings();
                Program.SaveSettings();
                ClearTemp();
            }
            else
            {
                e.Cancel = true;
            }
        }
        private void openProfileToolStripMenuItem_Click(object sender, EventArgs e)
        {
            OpenFileDialog op = new OpenFileDialog();
            op.Title = "Browse for Emulators Organizer profile";
            op.Filter = "Emulators Organizer Profile (*.eop)|*.eop";
            op.Multiselect = false;
            if (op.ShowDialog(this) == DialogResult.OK)
            {
                if (AskToSave())
                {
                    OpenProfile(op.FileName);
                }
            }
        }
        private void saveProfileToolStripMenuItem_EnabledChanged(object sender, EventArgs e)
        {
            toolStripButton3.Enabled = saveProfileToolStripMenuItem.Enabled;
        }
        //Draw stuff
        private void treeView_consoles_DrawNode(object sender, DrawTreeNodeEventArgs e)
        {
            //clear
            e.Graphics.FillRectangle(new SolidBrush(treeView_consoles.BackColor),
                e.Bounds.X, e.Bounds.Y, e.Bounds.Width, e.Bounds.Height);

            int h = imageList_consoles.ImageSize.Height;
            int w = imageList_consoles.ImageSize.Width;
            Size CharsSize = TextRenderer.MeasureText(e.Node.Text, treeView_consoles.Font);

            if (e.Node.Nodes.Count > 0)
            {
                if (!e.Node.IsExpanded)
                {
                    e.Graphics.DrawImage(Properties.Resources.arrowright, e.Bounds.X + 1, e.Bounds.Y + ((e.Bounds.Height / 2) - 5), 12, 12);
                }
                else
                {
                    e.Graphics.DrawImage(Properties.Resources.arrowdown, e.Bounds.X + 1, e.Bounds.Y + ((e.Bounds.Height / 2) - 5), 12, 12);
                }
            }
            Image im = null;
            if (e.Node.GetType() == typeof(TreeNode_Console))
            {
                if (treeView_consoles.SelectedNode == e.Node)
                {
                    if (treeView_consoles.Focused)
                        e.Graphics.FillRectangle(new SolidBrush(SystemColors.Highlight),
                          e.Bounds.X + 16, e.Bounds.Y + 2, w + CharsSize.Width + 5, h);
                    else
                        e.Graphics.FillRectangle(new SolidBrush(Color.LightGray),
                          e.Bounds.X + 16, e.Bounds.Y + 2, w + CharsSize.Width + 5, h);
                }

                im = ((TreeNode_Console)e.Node).Console.Icon;
                if (im == null)
                    im = Properties.Resources.cpu;

                e.Graphics.DrawImage(im, e.Bounds.X + 16, e.Bounds.Y + 2, w, h);
                e.Graphics.DrawString(e.Node.Text, treeView_consoles.Font, new SolidBrush(Color.Black), new PointF(e.Bounds.X + w + 20, e.Bounds.Y + ((e.Bounds.Height / 2) - 5)));
            }
            else if (e.Node.GetType() == typeof(TreeNode_Category))
            {
                if (treeView_consoles.SelectedNode == e.Node)
                {
                    if (treeView_consoles.Focused)
                        e.Graphics.FillRectangle(new SolidBrush(SystemColors.Highlight),
                          e.Bounds.X + 31, e.Bounds.Y + 2, w + CharsSize.Width + 5, h);
                    else
                        e.Graphics.FillRectangle(new SolidBrush(Color.LightGray),
                          e.Bounds.X + 31, e.Bounds.Y + 2, w + CharsSize.Width + 5, h);
                }

                //im = ((TreeNode_Console)e.Node.Parent).Console.Icon;
                im = ((TreeNode_Category)e.Node).CategoryIn.Icon;
                if (im == null)
                    im = ((TreeNode_Console)e.Node.Parent).Console.Icon;
                if (im == null)
                    im = Properties.Resources.cpu;
                if (e.Bounds.Y > 0)
                {
                    e.Graphics.DrawImage(im, e.Bounds.X + 31, e.Bounds.Y + 2, w, h);
                    e.Graphics.DrawString(e.Node.Text, treeView_consoles.Font,
                        new SolidBrush(Color.Black), new PointF(e.Bounds.X + w + 35, e.Bounds.Y + ((e.Bounds.Height / 2) - 5)));
                }
            }
        }
        private void listView_emulators_DrawItem(object sender, DrawListViewItemEventArgs e)
        {
            //clear
            e.Graphics.FillRectangle(new SolidBrush(treeView_consoles.BackColor),
                e.Bounds.X, e.Bounds.Y, e.Bounds.Width, e.Bounds.Height);
            int h = imageList_consoles.ImageSize.Height;
            int w = imageList_consoles.ImageSize.Width;

            Image im = ((ListViewItem_Emulator)e.Item).Emu.Icon;

            if (im == null)
                im = Properties.Resources.cpu;

            if (e.Item.Selected)
            {
                if (listView_emulators.Focused)
                {
                    e.Graphics.FillRectangle(new SolidBrush(SystemColors.Highlight),
                        e.Bounds.X, e.Bounds.Y, e.Bounds.Width, e.Bounds.Height);
                }
                else
                {
                    e.Graphics.FillRectangle(new SolidBrush(Color.LightGray),
                        e.Bounds.X, e.Bounds.Y, e.Bounds.Width, e.Bounds.Height);
                }
            }
            e.Graphics.DrawImage(im, e.Bounds.X + ((e.Bounds.Width / 2) - (w / 2)), e.Bounds.Y + 2, w, h);
            e.DrawText(TextFormatFlags.Bottom | TextFormatFlags.HorizontalCenter | TextFormatFlags.EndEllipsis);
        }
        private void treeView_playlists_DrawNode(object sender, DrawTreeNodeEventArgs e)
        {
            //clear
            e.Graphics.FillRectangle(new SolidBrush(treeView_playlists.BackColor),
                e.Bounds.X, e.Bounds.Y, e.Bounds.Width, e.Bounds.Height);

            int h = imageList_consoles.ImageSize.Height;
            int w = imageList_consoles.ImageSize.Width;
            Size CharsSize = TextRenderer.MeasureText(e.Node.Text, treeView_playlists.Font);

            if (treeView_playlists.SelectedNode == e.Node)
            {
                if (treeView_playlists.Focused)
                    e.Graphics.FillRectangle(new SolidBrush(SystemColors.Highlight),
                      e.Bounds.X + 16, e.Bounds.Y + 2, w + CharsSize.Width + 5, h);
                else
                    e.Graphics.FillRectangle(new SolidBrush(Color.LightGray),
                      e.Bounds.X + 16, e.Bounds.Y + 2, w + CharsSize.Width + 5, h);
            }
            Image im = ((TreeNode_PlayList)e.Node).PlayList.Icon;
            if (im == null)
                im = Properties.Resources.Favorites;

            e.Graphics.DrawImage(im, e.Bounds.X + 15, e.Bounds.Y + 2, w, h);
            e.Graphics.DrawString(e.Node.Text, treeView_playlists.Font, new SolidBrush(Color.Black),
                new PointF(e.Bounds.X + w + 19, e.Bounds.Y + ((e.Bounds.Height / 2) - 5)));

        }
        private void listView_roms_DrawColumnHeader(object sender, DrawListViewColumnHeaderEventArgs e)
        {
            // e.DrawDefault = true;
            e.DrawBackground();
            if (e.State == ListViewItemStates.Selected)
                e.Graphics.FillRectangle(new SolidBrush(Color.LightYellow), e.Bounds.X, e.Bounds.Y, e.Bounds.Width - 2, e.Bounds.Height - 2);
            if (e.State == ListViewItemStates.Hot)
                e.Graphics.DrawRectangle(new Pen(new SolidBrush(Color.LightYellow), 2), new Rectangle(e.Bounds.X, e.Bounds.Y, e.Bounds.Width - 2, e.Bounds.Height - 2));

            e.DrawText(TextFormatFlags.VerticalCenter | TextFormatFlags.EndEllipsis);
            if (e.Header == columnHeader_name & _SortMode == SortMode.Name)
            {
                if (_SortAlpha)
                    e.Graphics.DrawImage(Properties.Resources.Upp, e.Bounds.X + e.Bounds.Width - 20, e.Bounds.Y + 2, 15, 15);
                else
                    e.Graphics.DrawImage(Properties.Resources.Down, e.Bounds.X + e.Bounds.Width - 20, e.Bounds.Y + 2, 15, 15);
            }
            else if (e.Header == columnHeader1_fileType & _SortMode == SortMode.FileType)
            {
                if (_SortAlpha)
                    e.Graphics.DrawImage(Properties.Resources.Upp, e.Bounds.X + e.Bounds.Width - 20, e.Bounds.Y + 2, 15, 15);
                else
                    e.Graphics.DrawImage(Properties.Resources.Down, e.Bounds.X + e.Bounds.Width - 20, e.Bounds.Y + 2, 15, 15);
            }
            else if (e.Header == columnHeader_path & _SortMode == SortMode.Path)
            {
                if (_SortAlpha)
                    e.Graphics.DrawImage(Properties.Resources.Upp, e.Bounds.X + e.Bounds.Width - 20, e.Bounds.Y + 2, 15, 15);
                else
                    e.Graphics.DrawImage(Properties.Resources.Down, e.Bounds.X + e.Bounds.Width - 20, e.Bounds.Y + 2, 15, 15);
            }
            else if (e.Header == columnHeader1_playtime & _SortMode == SortMode.PlayedTime)
            {
                if (_SortAlpha)
                    e.Graphics.DrawImage(Properties.Resources.Upp, e.Bounds.X + e.Bounds.Width - 20, e.Bounds.Y + 2, 15, 15);
                else
                    e.Graphics.DrawImage(Properties.Resources.Down, e.Bounds.X + e.Bounds.Width - 20, e.Bounds.Y + 2, 15, 15);
            }
            else if (e.Header == columnHeader1_rating & _SortMode == SortMode.Rating)
            {
                if (_SortAlpha)
                    e.Graphics.DrawImage(Properties.Resources.Upp, e.Bounds.X + e.Bounds.Width - 20, e.Bounds.Y + 2, 15, 15);
                else
                    e.Graphics.DrawImage(Properties.Resources.Down, e.Bounds.X + e.Bounds.Width - 20, e.Bounds.Y + 2, 15, 15);
            }
            else if (e.Header == columnHeader_size & _SortMode == SortMode.Size)
            {
                if (_SortAlpha)
                    e.Graphics.DrawImage(Properties.Resources.Upp, e.Bounds.X + e.Bounds.Width - 20, e.Bounds.Y + 2, 15, 15);
                else
                    e.Graphics.DrawImage(Properties.Resources.Down, e.Bounds.X + e.Bounds.Width - 20, e.Bounds.Y + 2, 15, 15);
            }
        }
        private void listView_roms_DrawItem(object sender, DrawListViewItemEventArgs e)
        {
            if (listView_roms.View == View.LargeIcon)
            {
                e.Graphics.FillRectangle(new SolidBrush(listView_roms.BackColor),
                    e.Bounds.X, e.Bounds.Y, e.Bounds.Width, e.Bounds.Height);
                string path = "";
                int h = imageList1_romsLarge.ImageSize.Height;
                int w = imageList1_romsLarge.ImageSize.Width;
                Image im = null;
                if (comboBox_thumpType.SelectedItem.ToString() == "Cover")
                {
                    if (e.Item.GetType() == typeof(ListViewItem_Rom))
                    {
                        if (treeView_consoles.SelectedNode.GetType() == typeof(TreeNode_Console))
                            im = ((TreeNode_Console)treeView_consoles.SelectedNode).Console.Icon;
                        else if (treeView_consoles.SelectedNode.GetType() == typeof(TreeNode_Category))
                            im = ((TreeNode_Console)treeView_consoles.SelectedNode.Parent).Console.Icon;
                        if (((ListViewItem_Rom)e.Item).ROM.Covers == null)
                            ((ListViewItem_Rom)e.Item).ROM.Covers = new List<string>();
                        if (((ListViewItem_Rom)e.Item).ROM.Covers.Count > 0)
                            path = Utilities.GetFullPath(((ListViewItem_Rom)e.Item).ROM.Covers[0]);
                    }
                    else
                    {
                        im = ((ListViewItem_PlayListItem)e.Item).PlayListItem.Console.Icon;
                        if (((ListViewItem_PlayListItem)e.Item).PlayListItem.Rom.Covers == null)
                            ((ListViewItem_PlayListItem)e.Item).PlayListItem.Rom.Covers = new List<string>();
                        if (((ListViewItem_PlayListItem)e.Item).PlayListItem.Rom.Covers.Count > 0)
                            path = Utilities.GetFullPath(((ListViewItem_PlayListItem)e.Item).PlayListItem.Rom.Covers[0]);
                    }
                }
                else
                {
                    if (e.Item.GetType() == typeof(ListViewItem_Rom))
                    {
                        if (treeView_consoles.SelectedNode.GetType() == typeof(TreeNode_Console))
                            im = ((TreeNode_Console)treeView_consoles.SelectedNode).Console.Icon;
                        else if (treeView_consoles.SelectedNode.GetType() == typeof(TreeNode_Category))
                            im = ((TreeNode_Console)treeView_consoles.SelectedNode.Parent).Console.Icon;
                        if (((ListViewItem_Rom)e.Item).ROM.Snapshots == null)
                            ((ListViewItem_Rom)e.Item).ROM.Snapshots = new List<string>();
                        if (((ListViewItem_Rom)e.Item).ROM.Snapshots.Count > 0)
                            path = Utilities.GetFullPath(((ListViewItem_Rom)e.Item).ROM.Snapshots[0]);
                    }
                    else
                    {
                        im = ((ListViewItem_PlayListItem)e.Item).PlayListItem.Console.Icon;
                        if (((ListViewItem_PlayListItem)e.Item).PlayListItem.Rom.Snapshots == null)
                            ((ListViewItem_PlayListItem)e.Item).PlayListItem.Rom.Snapshots = new List<string>();
                        if (((ListViewItem_PlayListItem)e.Item).PlayListItem.Rom.Snapshots.Count > 0)
                            path = Utilities.GetFullPath(((ListViewItem_PlayListItem)e.Item).PlayListItem.Rom.Snapshots[0]);
                    }
                }
                if (im == null)
                    im = Properties.Resources.cpu;
                if (e.Item.Selected)
                {
                    if (listView_roms.Focused)
                        e.Graphics.FillRectangle(new SolidBrush(SystemColors.Highlight),
                          e.Bounds.X, e.Bounds.Y, e.Bounds.Width, e.Bounds.Height);
                    else
                        e.Graphics.FillRectangle(new SolidBrush(SystemColors.ButtonFace),
                    e.Bounds.X, e.Bounds.Y, e.Bounds.Width, e.Bounds.Height);
                }
                if (File.Exists(path))
                {
                    e.Graphics.DrawImage(Image.FromFile(path), e.Bounds.X + ((e.Bounds.Width / 2) - (w / 2)), e.Bounds.Y + 2, w, h);
                }
                else
                {
                    e.Graphics.DrawImage(im, e.Bounds.X + ((e.Bounds.Width / 2) - (w / 2)), e.Bounds.Y + 2, w, h);
                }
                e.DrawText(TextFormatFlags.Bottom | TextFormatFlags.HorizontalCenter | TextFormatFlags.EndEllipsis);
            }
            if (EnsureVissibleRom)
            {
                listView_roms.Items[Program.UserSettings.LastSelectedRom].EnsureVisible();
                EnsureVissibleRom = false;
            }
        }
        private void listView_roms_DrawSubItem(object sender, DrawListViewSubItemEventArgs e)
        {
            if (IsLoadingRoms)
                return;
            if (listView_roms.View == View.Details)
            {
                int h = imageList1_roms.ImageSize.Height;
                int w = imageList1_roms.ImageSize.Width;
                //clear
                #region NAME
                if (e.Header == columnHeader_name)//name
                {
                    if (tabControl3.SelectedIndex == 0)//Console or category
                    {
                        if (e.Item.Selected)
                        {
                            if (listView_roms.Focused)
                                e.Graphics.FillRectangle(new SolidBrush(SystemColors.Highlight),
                                  e.Bounds.X, e.Bounds.Y, e.Bounds.Width, e.Bounds.Height);
                            else
                                e.Graphics.FillRectangle(new SolidBrush(SystemColors.ButtonFace),
                            e.Bounds.X, e.Bounds.Y, e.Bounds.Width, e.Bounds.Height);
                        }
                        else
                        {
                            e.Graphics.FillRectangle(new SolidBrush(e.Item.BackColor),
                              e.Bounds.X, e.Bounds.Y, e.Bounds.Width, e.Bounds.Height);
                        }
                        if (treeView_consoles.SelectedNode == null)
                            return;
                        if (treeView_consoles.SelectedNode.GetType() == typeof(TreeNode_Console))
                        {
                            if (((TreeNode_Console)treeView_consoles.SelectedNode).Console.Icon != null)
                                e.Graphics.DrawImage(((TreeNode_Console)treeView_consoles.SelectedNode).Console.Icon, e.Bounds.X + 2, e.Bounds.Y, w, h);
                            else
                                e.Graphics.DrawImage(Properties.Resources.cpu, e.Bounds.X + 2, e.Bounds.Y, w, h);

                            e.Graphics.DrawString(e.Item.Text, listView_roms.Font, new SolidBrush(Color.Black),
                                new RectangleF(e.Bounds.X + w + 2, e.Bounds.Y + 3, e.Bounds.Width - (w + 2), e.Bounds.Height),
                                new StringFormat(StringFormatFlags.NoWrap));
                        }
                        else
                        {
                            if (((TreeNode_Console)treeView_consoles.SelectedNode.Parent).Console.Icon != null)
                                e.Graphics.DrawImage(((TreeNode_Console)treeView_consoles.SelectedNode.Parent).Console.Icon, e.Bounds.X + 2, e.Bounds.Y, w, h);
                            else
                                e.Graphics.DrawImage(Properties.Resources.cpu, e.Bounds.X + 2, e.Bounds.Y, w, h);

                            e.Graphics.DrawString(e.Item.Text, listView_roms.Font, new SolidBrush(Color.Black),
                                new RectangleF(e.Bounds.X + w + 2, e.Bounds.Y + 3, e.Bounds.Width - (w + 2), e.Bounds.Height),
                                new StringFormat(StringFormatFlags.NoWrap));
                        }

                    }
                    else//Playlist
                    {
                        if (e.Item.Selected)
                        {
                            if (listView_roms.Focused)
                                e.Graphics.FillRectangle(new SolidBrush(SystemColors.Highlight),
                                  e.Bounds.X, e.Bounds.Y, e.Bounds.Width, e.Bounds.Height);
                            else
                                e.Graphics.FillRectangle(new SolidBrush(SystemColors.ButtonFace),
                            e.Bounds.X, e.Bounds.Y, e.Bounds.Width, e.Bounds.Height);
                        }
                        else
                        {
                            e.Graphics.FillRectangle(new SolidBrush(e.Item.BackColor),
                              e.Bounds.X, e.Bounds.Y, e.Bounds.Width, e.Bounds.Height);
                        }
                        if (e.Item.GetType() == typeof(ListViewItem_PlayListItem))
                        {
                            Image img = ((ListViewItem_PlayListItem)e.Item).PlayListItem.Icon;
                            if (img == null)
                                img = ((ListViewItem_PlayListItem)e.Item).PlayListItem.Console.Icon;
                            if (img == null)
                                img = Properties.Resources.cpu;

                            e.Graphics.DrawImage(img, e.Bounds.X + 2, e.Bounds.Y, w, h);
                        }

                        e.Graphics.DrawString(e.Item.Text, listView_roms.Font, new SolidBrush(Color.Black),
                            new RectangleF(e.Bounds.X + w + 2, e.Bounds.Y + 3, e.Bounds.Width - (w + 2), e.Bounds.Height),
                            new StringFormat(StringFormatFlags.NoWrap));
                    }
                }
                #endregion
                #region Rom path
                else if (e.Header == columnHeader_path)//Draw exception of the rom path
                {
                    bool HAS = true;
                    string patt = "";
                    bool IsURL = false;
                    if (e.Item.GetType() == typeof(ListViewItem_Rom))
                    {
                        if (!((ListViewItem_Rom)e.Item).ROM.IsURL)
                        {
                            HAS = File.Exists(Utilities.GetFullPath(((ListViewItem_Rom)e.Item).ROM.Path));
                            patt = ((ListViewItem_Rom)e.Item).ROM.Path;
                        }
                        else
                        {
                            HAS = true; IsURL = true;
                            patt = ((ListViewItem_Rom)e.Item).ROM.URL;
                        }
                    }
                    else
                    {
                        if (!((ListViewItem_PlayListItem)e.Item).PlayListItem.Rom.IsURL)
                        {
                            HAS = File.Exists(Utilities.GetFullPath(((ListViewItem_PlayListItem)e.Item).PlayListItem.Rom.Path));
                            patt = ((ListViewItem_PlayListItem)e.Item).PlayListItem.Rom.Path;
                        }
                        else
                        {
                            HAS = true; IsURL = true;
                            patt = ((ListViewItem_PlayListItem)e.Item).PlayListItem.Rom.Path;
                        }
                    }
                    if (e.Item.Selected)
                    {
                        if (listView_roms.Focused)
                            e.Graphics.FillRectangle(new SolidBrush(SystemColors.Highlight),
                              e.Bounds.X, e.Bounds.Y, e.Bounds.Width, e.Bounds.Height);
                        else
                            e.Graphics.FillRectangle(new SolidBrush(SystemColors.ButtonFace),
                        e.Bounds.X, e.Bounds.Y, e.Bounds.Width, e.Bounds.Height);
                    }
                    else
                    {
                        e.Graphics.FillRectangle(new SolidBrush(e.Item.BackColor),
                        e.Bounds.X, e.Bounds.Y, e.Bounds.Width, e.Bounds.Height);
                    }
                    if (HAS)
                    {
                        if (!IsURL)
                        {
                            e.Graphics.DrawString(patt, listView_roms.Font, new SolidBrush(Color.Black),
                                new RectangleF(e.Bounds.X + 2, e.Bounds.Y + 2, e.Bounds.Width, e.Bounds.Height),
                                new StringFormat(StringFormatFlags.NoWrap));
                        }
                        else
                        {
                            e.Graphics.DrawImage(Properties.Resources.internettttt, e.Bounds.X + 2, e.Bounds.Y + 1, w, h);
                            e.Graphics.DrawString(patt, listView_roms.Font, new SolidBrush(Color.Black),
                                new RectangleF(e.Bounds.X + w + 2, e.Bounds.Y + 3, e.Bounds.Width - (w + 2), e.Bounds.Height),
                                new StringFormat(StringFormatFlags.NoWrap));
                        }
                    }
                    else
                    {
                        e.Graphics.DrawImage(Properties.Resources.Warning, e.Bounds.X + 2, e.Bounds.Y + 1, w, h);
                        e.Graphics.DrawString(patt, listView_roms.Font, new SolidBrush(Color.Black),
                            new RectangleF(e.Bounds.X + w + 2, e.Bounds.Y + 3, e.Bounds.Width - (w + 2), e.Bounds.Height),
                            new StringFormat(StringFormatFlags.NoWrap));
                    }
                }
                #endregion
                #region Size
                else if (e.Header == columnHeader_size)//Draw exception of the rom path
                {
                    string patt = "";
                    if (e.Item.GetType() == typeof(ListViewItem_Rom))
                    {
                        patt = ((ListViewItem_Rom)e.Item).ROM.RomSize;
                    }
                    else
                    {
                        patt = ((ListViewItem_PlayListItem)e.Item).PlayListItem.Rom.RomSize;
                    }
                    if (e.Item.Selected)
                    {
                        if (listView_roms.Focused)
                            e.Graphics.FillRectangle(new SolidBrush(SystemColors.Highlight),
                              e.Bounds.X, e.Bounds.Y, e.Bounds.Width, e.Bounds.Height);
                        else
                            e.Graphics.FillRectangle(new SolidBrush(SystemColors.ButtonFace),
                        e.Bounds.X, e.Bounds.Y, e.Bounds.Width, e.Bounds.Height);
                    }
                    else
                    {
                        e.Graphics.FillRectangle(new SolidBrush(e.Item.BackColor),
                   e.Bounds.X, e.Bounds.Y, e.Bounds.Width, e.Bounds.Height);
                    }
                    e.Graphics.DrawString(patt, listView_roms.Font, new SolidBrush(Color.Black), new RectangleF(e.Bounds.X + 2, e.Bounds.Y + 2, e.Bounds.Width, e.Bounds.Height), new StringFormat(StringFormatFlags.NoWrap));
                }
                #endregion
                #region Rating
                if (e.Header == columnHeader1_rating)//Draw rating
                {
                    RatingX = e.Bounds.X;
                    int rorating = 0;
                    if (e.Item.GetType() == typeof(ListViewItem_Rom))
                    {
                        rorating = ((ListViewItem_Rom)e.Item).ROM.Rating;
                    }
                    else
                    {
                        rorating = ((ListViewItem_PlayListItem)e.Item).PlayListItem.Rom.Rating;
                    }
                    if (e.Item.Selected)
                    {
                        if (listView_roms.Focused)
                            e.Graphics.FillRectangle(new SolidBrush(SystemColors.Highlight),
                              e.Bounds.X, e.Bounds.Y, e.Bounds.Width, e.Bounds.Height);
                        else
                            e.Graphics.FillRectangle(new SolidBrush(SystemColors.ButtonFace),
                        e.Bounds.X, e.Bounds.Y, e.Bounds.Width, e.Bounds.Height);
                    }
                    else
                    {
                        e.Graphics.FillRectangle(new SolidBrush(e.Item.BackColor),
         e.Bounds.X, e.Bounds.Y, e.Bounds.Width, e.Bounds.Height);
                    }
                    //Draw stars
                    int g = 0;
                    for (int i = 1; i < 6; i++)
                    {
                        if (i <= rorating)
                        {
                            e.Graphics.DrawImage(Properties.Resources.star, e.Bounds.X + 2 + g, e.Bounds.Y + 2, 16, 16);
                        }
                        else
                        {
                            e.Graphics.DrawImage(Properties.Resources.starunrate, e.Bounds.X + 2 + g, e.Bounds.Y + 2, 16, 16);
                        }
                        g += 15;
                    }
                }
                #endregion
                #region Played
                else if (e.Header == columnHeader1_playtime)
                {
                    string patt = "";
                    if (e.Item.GetType() == typeof(ListViewItem_Rom))
                    {
                        patt = ((ListViewItem_Rom)e.Item).ROM.PlayedTimes.ToString();
                    }
                    else
                    {
                        patt = ((ListViewItem_PlayListItem)e.Item).PlayListItem.Rom.PlayedTimes.ToString();
                    }
                    if (e.Item.Selected)
                    {
                        if (listView_roms.Focused)
                            e.Graphics.FillRectangle(new SolidBrush(SystemColors.Highlight),
                              e.Bounds.X, e.Bounds.Y, e.Bounds.Width, e.Bounds.Height);
                        else
                            e.Graphics.FillRectangle(new SolidBrush(SystemColors.ButtonFace),
                        e.Bounds.X, e.Bounds.Y, e.Bounds.Width, e.Bounds.Height);
                    }
                    else
                    {
                        e.Graphics.FillRectangle(new SolidBrush(e.Item.BackColor),
         e.Bounds.X, e.Bounds.Y, e.Bounds.Width, e.Bounds.Height);
                    }
                    e.Graphics.DrawString(patt, listView_roms.Font, new SolidBrush(Color.Black), new RectangleF(e.Bounds.X + 2, e.Bounds.Y + 2, e.Bounds.Width, e.Bounds.Height), new StringFormat(StringFormatFlags.NoWrap));
                }
                #endregion
                #region Played
                else if (e.Header == columnHeader_year)
                {
                    string patt = "";
                    if (e.Item.GetType() == typeof(ListViewItem_Rom))
                    {
                        patt = ((ListViewItem_Rom)e.Item).ROM.Year.ToString();
                    }
                    else
                    {
                        patt = ((ListViewItem_PlayListItem)e.Item).PlayListItem.Rom.Year.ToString();
                    }
                    if (e.Item.Selected)
                    {
                        if (listView_roms.Focused)
                            e.Graphics.FillRectangle(new SolidBrush(SystemColors.Highlight),
                              e.Bounds.X, e.Bounds.Y, e.Bounds.Width, e.Bounds.Height);
                        else
                            e.Graphics.FillRectangle(new SolidBrush(SystemColors.ButtonFace),
                        e.Bounds.X, e.Bounds.Y, e.Bounds.Width, e.Bounds.Height);
                    }
                    else
                    {
                        e.Graphics.FillRectangle(new SolidBrush(e.Item.BackColor),
         e.Bounds.X, e.Bounds.Y, e.Bounds.Width, e.Bounds.Height);
                    }
                    e.Graphics.DrawString(patt, listView_roms.Font, new SolidBrush(Color.Black), new RectangleF(e.Bounds.X + 2, e.Bounds.Y + 2, e.Bounds.Width, e.Bounds.Height), new StringFormat(StringFormatFlags.NoWrap));
                }
                #endregion
                #region File type
                else if (e.Header == columnHeader1_fileType)
                {
                    bool HAS = false;
                    string patt = "";
                    bool IsURL = false;
                    Image im = null;
                    if (e.Item.GetType() == typeof(ListViewItem_Rom))
                    {
                        if (!((ListViewItem_Rom)e.Item).ROM.IsURL)
                        {
                            HAS = File.Exists(Utilities.GetFullPath(((ListViewItem_Rom)e.Item).ROM.Path));
                            patt = Path.GetExtension(Utilities.GetFullPath(((ListViewItem_Rom)e.Item).ROM.Path));
                            if (HAS)
                            {
                                im = Icon.ExtractAssociatedIcon(Utilities.GetFullPath(((ListViewItem_Rom)e.Item).ROM.Path)).ToBitmap();
                            }
                        }
                        else
                        {
                            IsURL = HAS = true;
                            patt = Path.GetExtension(((ListViewItem_Rom)e.Item).ROM.URL);
                        }
                    }
                    else
                    {
                        if (!((ListViewItem_PlayListItem)e.Item).PlayListItem.Rom.IsURL)
                        {
                            HAS = File.Exists(Utilities.GetFullPath(((ListViewItem_PlayListItem)e.Item).PlayListItem.Rom.Path));
                            patt = Path.GetExtension(Utilities.GetFullPath(((ListViewItem_PlayListItem)e.Item).PlayListItem.Rom.Path));
                            if (HAS)
                            {
                                im = Icon.ExtractAssociatedIcon(Utilities.GetFullPath(((ListViewItem_PlayListItem)e.Item).PlayListItem.Rom.Path)).ToBitmap();
                            }
                        }
                        else
                        {
                            IsURL = HAS = true;
                            patt = Path.GetExtension(((ListViewItem_PlayListItem)e.Item).PlayListItem.Rom.URL);
                        }
                    }
                    if (e.Item.Selected)
                    {
                        if (listView_roms.Focused)
                            e.Graphics.FillRectangle(new SolidBrush(SystemColors.Highlight),
                              e.Bounds.X, e.Bounds.Y, e.Bounds.Width, e.Bounds.Height);
                        else
                            e.Graphics.FillRectangle(new SolidBrush(SystemColors.ButtonFace),
                        e.Bounds.X, e.Bounds.Y, e.Bounds.Width, e.Bounds.Height);
                    }
                    else
                    {
                        e.Graphics.FillRectangle(new SolidBrush(e.Item.BackColor),
         e.Bounds.X, e.Bounds.Y, e.Bounds.Width, e.Bounds.Height);
                    }
                    if (HAS)
                    {
                        if (!IsURL)
                            e.Graphics.DrawImage(im, e.Bounds.X + 2, e.Bounds.Y, w, h);
                        else
                            e.Graphics.DrawImage(Properties.Resources.internettttt, e.Bounds.X + 2, e.Bounds.Y, w, h);
                    }
                    else
                    {
                        e.Graphics.DrawImage(Properties.Resources.Warning, e.Bounds.X + 2, e.Bounds.Y, w, h);
                    }
                    e.Graphics.DrawString(patt, listView_roms.Font, new SolidBrush(Color.Black),
                    new RectangleF(e.Bounds.X + w + 2, e.Bounds.Y + 3, e.Bounds.Width - (w + 2), e.Bounds.Height),
                    new StringFormat(StringFormatFlags.NoWrap));
                }
                #endregion
                #region Has cover
                else if (e.Header == columnHeader1_hasCover)
                {
                    bool HAS = false;
                    if (e.Item.GetType() == typeof(ListViewItem_Rom))
                    {
                        if (((ListViewItem_Rom)e.Item).ROM.Covers == null)
                            ((ListViewItem_Rom)e.Item).ROM.Covers = new List<string>();
                        if (((ListViewItem_Rom)e.Item).ROM.Covers.Count > 0)
                        {
                            HAS = File.Exists(Utilities.GetFullPath(((ListViewItem_Rom)e.Item).ROM.Covers[0]));
                        }
                    }
                    else
                    {
                        if (((ListViewItem_PlayListItem)e.Item).PlayListItem.Rom.Covers == null)
                            ((ListViewItem_PlayListItem)e.Item).PlayListItem.Rom.Covers = new List<string>();
                        if (((ListViewItem_PlayListItem)e.Item).PlayListItem.Rom.Covers.Count > 0)
                        {
                            HAS = File.Exists(Utilities.GetFullPath(((ListViewItem_PlayListItem)e.Item).PlayListItem.Rom.Covers[0]));
                        }
                    }
                    if (e.Item.Selected)
                    {
                        if (listView_roms.Focused)
                            e.Graphics.FillRectangle(new SolidBrush(SystemColors.Highlight),
                              e.Bounds.X, e.Bounds.Y, e.Bounds.Width, e.Bounds.Height);
                        else
                            e.Graphics.FillRectangle(new SolidBrush(SystemColors.ButtonFace),
                        e.Bounds.X, e.Bounds.Y, e.Bounds.Width, e.Bounds.Height);
                    }
                    else
                    {
                        e.Graphics.FillRectangle(new SolidBrush(e.Item.BackColor),
                          e.Bounds.X, e.Bounds.Y, e.Bounds.Width, e.Bounds.Height);
                    }
                    if (HAS)
                        e.Graphics.DrawImage(Properties.Resources.check, e.Bounds.X + 2, e.Bounds.Y + 1, 16, 16);
                    else
                        e.Graphics.DrawImage(Properties.Resources.dele, e.Bounds.X + 2, e.Bounds.Y + 1, 16, 16);
                }
                #endregion
                #region Has snapshot
                else if (e.Header == columnHeader1_hasSnap)//Draw has snapshot
                {
                    bool HAS = false;
                    if (e.Item.GetType() == typeof(ListViewItem_Rom))
                    {
                        if (((ListViewItem_Rom)e.Item).ROM.Snapshots == null)
                            ((ListViewItem_Rom)e.Item).ROM.Snapshots = new List<string>();
                        if (((ListViewItem_Rom)e.Item).ROM.Snapshots.Count > 0)
                        {
                            HAS = File.Exists(Utilities.GetFullPath(((ListViewItem_Rom)e.Item).ROM.Snapshots[0]));
                        }
                    }
                    else
                    {
                        if (((ListViewItem_PlayListItem)e.Item).PlayListItem.Rom.Snapshots == null)
                            ((ListViewItem_PlayListItem)e.Item).PlayListItem.Rom.Snapshots = new List<string>();
                        if (((ListViewItem_PlayListItem)e.Item).PlayListItem.Rom.Snapshots.Count > 0)
                        {
                            HAS = File.Exists(Utilities.GetFullPath(((ListViewItem_PlayListItem)e.Item).PlayListItem.Rom.Snapshots[0]));
                        }
                    }
                    if (e.Item.Selected)
                    {
                        if (listView_roms.Focused)
                            e.Graphics.FillRectangle(new SolidBrush(SystemColors.Highlight),
                              e.Bounds.X, e.Bounds.Y, e.Bounds.Width, e.Bounds.Height);
                        else
                            e.Graphics.FillRectangle(new SolidBrush(SystemColors.ButtonFace),
                        e.Bounds.X, e.Bounds.Y, e.Bounds.Width, e.Bounds.Height);
                    }
                    else
                    {
                        e.Graphics.FillRectangle(new SolidBrush(e.Item.BackColor),
                          e.Bounds.X, e.Bounds.Y, e.Bounds.Width, e.Bounds.Height);
                    }
                    if (HAS)
                        e.Graphics.DrawImage(Properties.Resources.check, e.Bounds.X + 2, e.Bounds.Y + 1, 16, 16);
                    else
                        e.Graphics.DrawImage(Properties.Resources.dele, e.Bounds.X + 2, e.Bounds.Y + 1, 16, 16);
                }
                #endregion
                #region Has info
                else if (e.Header == columnHeader1_hasInfo)
                {
                    if (e.Item.Selected)
                    {
                        if (listView_roms.Focused)
                            e.Graphics.FillRectangle(new SolidBrush(SystemColors.Highlight),
                              e.Bounds.X, e.Bounds.Y, e.Bounds.Width, e.Bounds.Height);
                        else
                            e.Graphics.FillRectangle(new SolidBrush(SystemColors.ButtonFace),
                        e.Bounds.X, e.Bounds.Y, e.Bounds.Width, e.Bounds.Height);
                    }
                    else
                    {
                        e.Graphics.FillRectangle(new SolidBrush(e.Item.BackColor),
                          e.Bounds.X, e.Bounds.Y, e.Bounds.Width, e.Bounds.Height);
                    }
                    string[] infoexs = { ".rtf", ".RTF", ".txt", ".TXT" };
                    bool HAS = false;
                    if (e.Item.GetType() == typeof(ListViewItem_Rom))
                    {
                        if (treeView_consoles.SelectedNode.GetType() == typeof(TreeNode_Console))
                        {
                            TreeNode_Console con = (TreeNode_Console)treeView_consoles.SelectedNode;
                            for (int i = 0; i < infoexs.Length; i++)
                            {
                                if (File.Exists(Utilities.GetFullPath(con.Console.InfoFolder) + "\\" +
                                    Path.GetFileNameWithoutExtension(((ListViewItem_Rom)e.Item).ROM.Path) + infoexs[i]))
                                {
                                    if (infoexs[i].ToLower() == ".txt")
                                    {
                                        e.Graphics.DrawImage(Properties.Resources.File_TXT, e.Bounds.X + 2, e.Bounds.Y + 1, 16, 16);
                                    }
                                    else
                                    {
                                        e.Graphics.DrawImage(Properties.Resources.File_RTF, e.Bounds.X + 2, e.Bounds.Y + 1, 16, 16);
                                    }
                                    HAS = true;
                                    break;
                                }
                            }
                        }
                        else
                        {
                            TreeNode_Console con = (TreeNode_Console)treeView_consoles.SelectedNode.Parent;
                            for (int i = 0; i < infoexs.Length; i++)
                            {
                                if (File.Exists(Utilities.GetFullPath(con.Console.InfoFolder) + "\\" +
                                    Path.GetFileNameWithoutExtension(((ListViewItem_Rom)e.Item).ROM.Path) + infoexs[i]))
                                {
                                    if (infoexs[i].ToLower() == ".txt")
                                    {
                                        e.Graphics.DrawImage(Properties.Resources.File_TXT, e.Bounds.X + 2, e.Bounds.Y + 1, 16, 16);
                                    }
                                    else
                                    {
                                        e.Graphics.DrawImage(Properties.Resources.File_RTF, e.Bounds.X + 2, e.Bounds.Y + 1, 16, 16);
                                    }
                                    HAS = true;
                                    break;
                                }
                            }
                        }
                    }
                    else
                    {
                        ListViewItem_PlayListItem pll = (ListViewItem_PlayListItem)e.Item;
                        for (int i = 0; i < infoexs.Length; i++)
                        {
                            if (File.Exists(Utilities.GetFullPath(pll.PlayListItem.Console.InfoFolder) + "\\" +
                                Path.GetFileNameWithoutExtension(pll.PlayListItem.Rom.Path) + infoexs[i]))
                            {
                                if (infoexs[i].ToLower() == ".txt")
                                {
                                    e.Graphics.DrawImage(Properties.Resources.File_TXT, e.Bounds.X + 2, e.Bounds.Y + 1, 16, 16);
                                }
                                else
                                {
                                    e.Graphics.DrawImage(Properties.Resources.File_RTF, e.Bounds.X + 2, e.Bounds.Y + 1, 16, 16);
                                }
                                HAS = true;
                                break;
                            }
                        }
                    }
                    if (!HAS)
                        e.Graphics.DrawImage(Properties.Resources.dele, e.Bounds.X + 2, e.Bounds.Y + 1, 16, 16);
                }
                #endregion
                #region Has history
                else if (e.Header == columnHeader1_hasHistory)
                {
                    if (e.Item.Selected)
                    {
                        if (listView_roms.Focused)
                            e.Graphics.FillRectangle(new SolidBrush(SystemColors.Highlight),
                              e.Bounds.X, e.Bounds.Y, e.Bounds.Width, e.Bounds.Height);
                        else
                            e.Graphics.FillRectangle(new SolidBrush(SystemColors.ButtonFace),
                        e.Bounds.X, e.Bounds.Y, e.Bounds.Width, e.Bounds.Height);
                    }
                    else
                    {
                        e.Graphics.FillRectangle(new SolidBrush(e.Item.BackColor),
                          e.Bounds.X, e.Bounds.Y, e.Bounds.Width, e.Bounds.Height);
                    }
                    string[] infoexs = { ".rtf", ".RTF", ".txt", ".TXT" };
                    bool HAS = false;
                    if (e.Item.GetType() == typeof(ListViewItem_Rom))
                    {
                        if (treeView_consoles.SelectedNode.GetType() == typeof(TreeNode_Console))
                        {
                            TreeNode_Console con = (TreeNode_Console)treeView_consoles.SelectedNode;
                            for (int i = 0; i < infoexs.Length; i++)
                            {
                                if (File.Exists(Utilities.GetFullPath(con.Console.HistoryFolder) + "\\" +
                                    Path.GetFileNameWithoutExtension(((ListViewItem_Rom)e.Item).ROM.Path) + infoexs[i]))
                                {
                                    if (infoexs[i].ToLower() == ".txt")
                                    {
                                        e.Graphics.DrawImage(Properties.Resources.File_TXT, e.Bounds.X + 2, e.Bounds.Y + 1, 16, 16);
                                    }
                                    else
                                    {
                                        e.Graphics.DrawImage(Properties.Resources.File_RTF, e.Bounds.X + 2, e.Bounds.Y + 1, 16, 16);
                                    }
                                    HAS = true;
                                    break;
                                }
                            }
                        }
                        else
                        {
                            TreeNode_Console con = (TreeNode_Console)treeView_consoles.SelectedNode.Parent;
                            for (int i = 0; i < infoexs.Length; i++)
                            {
                                if (File.Exists(Utilities.GetFullPath(con.Console.HistoryFolder) + "\\" +
                                    Path.GetFileNameWithoutExtension(((ListViewItem_Rom)e.Item).ROM.Path) + infoexs[i]))
                                {
                                    if (infoexs[i].ToLower() == ".txt")
                                    {
                                        e.Graphics.DrawImage(Properties.Resources.File_TXT, e.Bounds.X + 2, e.Bounds.Y + 1, 16, 16);
                                    }
                                    else
                                    {
                                        e.Graphics.DrawImage(Properties.Resources.File_RTF, e.Bounds.X + 2, e.Bounds.Y + 1, 16, 16);
                                    }
                                    HAS = true;
                                    break;
                                }
                            }
                        }
                    }
                    else
                    {
                        ListViewItem_PlayListItem pll = (ListViewItem_PlayListItem)e.Item;
                        for (int i = 0; i < infoexs.Length; i++)
                        {
                            if (File.Exists(Utilities.GetFullPath(pll.PlayListItem.Console.HistoryFolder) + "\\" +
                                Path.GetFileNameWithoutExtension(pll.PlayListItem.Rom.Path) + infoexs[i]))
                            {
                                if (infoexs[i].ToLower() == ".txt")
                                {
                                    e.Graphics.DrawImage(Properties.Resources.File_TXT, e.Bounds.X + 2, e.Bounds.Y + 1, 16, 16);
                                }
                                else
                                {
                                    e.Graphics.DrawImage(Properties.Resources.File_RTF, e.Bounds.X + 2, e.Bounds.Y + 1, 16, 16);
                                }
                                HAS = true;
                                break;
                            }
                        }
                    }
                    if (!HAS)
                        e.Graphics.DrawImage(Properties.Resources.dele, e.Bounds.X + 2, e.Bounds.Y + 1, 16, 16);
                }
                #endregion
                #region Has pdf
                else if (e.Header == columnHeader1_hasManual)//Draw has pdf
                {
                    bool HAS = false;
                    if (e.Item.GetType() == typeof(ListViewItem_Rom))
                    {
                        HAS = File.Exists(Utilities.GetFullPath(((ListViewItem_Rom)e.Item).ROM.PDFPath));
                    }
                    else
                    {
                        HAS = File.Exists(Utilities.GetFullPath(((ListViewItem_PlayListItem)e.Item).PlayListItem.Rom.PDFPath));
                    }
                    if (e.Item.Selected)
                    {
                        if (listView_roms.Focused)
                            e.Graphics.FillRectangle(new SolidBrush(SystemColors.Highlight),
                              e.Bounds.X, e.Bounds.Y, e.Bounds.Width, e.Bounds.Height);
                        else
                            e.Graphics.FillRectangle(new SolidBrush(SystemColors.ButtonFace),
                        e.Bounds.X, e.Bounds.Y, e.Bounds.Width, e.Bounds.Height);
                    }
                    else
                    {
                        e.Graphics.FillRectangle(new SolidBrush(e.Item.BackColor),
                          e.Bounds.X, e.Bounds.Y, e.Bounds.Width, e.Bounds.Height);
                    }
                    if (HAS)
                        e.Graphics.DrawImage(Properties.Resources.check, e.Bounds.X + 2, e.Bounds.Y + 1, 16, 16);
                    else
                        e.Graphics.DrawImage(Properties.Resources.dele, e.Bounds.X + 2, e.Bounds.Y + 1, 16, 16);
                }
                #endregion
                #region Has video
                else if (e.Header == columnHeader_video)//Draw has video
                {
                    bool HAS = false;
                    if (e.Item.GetType() == typeof(ListViewItem_Rom))
                    {
                        if (((ListViewItem_Rom)e.Item).ROM.Videos != null)
                            if (((ListViewItem_Rom)e.Item).ROM.Videos.Count > 0)
                                HAS = File.Exists(Utilities.GetFullPath(((ListViewItem_Rom)e.Item).ROM.Videos[0]));
                    }
                    else
                    {
                        if (((ListViewItem_PlayListItem)e.Item).PlayListItem.Rom.Videos != null)
                            if (((ListViewItem_PlayListItem)e.Item).PlayListItem.Rom.Videos.Count > 0)
                                HAS = File.Exists(Utilities.GetFullPath(((ListViewItem_PlayListItem)e.Item).PlayListItem.Rom.Videos[0]));
                    }
                    if (e.Item.Selected)
                    {
                        if (listView_roms.Focused)
                            e.Graphics.FillRectangle(new SolidBrush(SystemColors.Highlight),
                              e.Bounds.X, e.Bounds.Y, e.Bounds.Width, e.Bounds.Height);
                        else
                            e.Graphics.FillRectangle(new SolidBrush(SystemColors.ButtonFace),
                        e.Bounds.X, e.Bounds.Y, e.Bounds.Width, e.Bounds.Height);
                    }
                    else
                    {
                        e.Graphics.FillRectangle(new SolidBrush(e.Item.BackColor),
                          e.Bounds.X, e.Bounds.Y, e.Bounds.Width, e.Bounds.Height);
                    }
                    if (HAS)
                        e.Graphics.DrawImage(Properties.Resources.check, e.Bounds.X + 2, e.Bounds.Y + 1, 16, 16);
                    else
                        e.Graphics.DrawImage(Properties.Resources.dele, e.Bounds.X + 2, e.Bounds.Y + 1, 16, 16);
                }
                #endregion
                #region Has sounds
                else if (e.Header == columnHeader_sounds)
                {
                    bool HAS = false;
                    if (e.Item.GetType() == typeof(ListViewItem_Rom))
                    {
                        if (((ListViewItem_Rom)e.Item).ROM.SoundRecords == null)
                            ((ListViewItem_Rom)e.Item).ROM.SoundRecords = new List<string>();
                        if (((ListViewItem_Rom)e.Item).ROM.SoundRecords.Count > 0)
                        {
                            HAS = File.Exists(Utilities.GetFullPath(((ListViewItem_Rom)e.Item).ROM.SoundRecords[0]));
                        }
                    }
                    else
                    {
                        if (((ListViewItem_PlayListItem)e.Item).PlayListItem.Rom.SoundRecords == null)
                            ((ListViewItem_PlayListItem)e.Item).PlayListItem.Rom.SoundRecords = new List<string>();
                        if (((ListViewItem_PlayListItem)e.Item).PlayListItem.Rom.SoundRecords.Count > 0)
                        {
                            HAS = File.Exists(Utilities.GetFullPath(((ListViewItem_PlayListItem)e.Item).PlayListItem.Rom.SoundRecords[0]));
                        }
                    }
                    if (e.Item.Selected)
                    {
                        if (listView_roms.Focused)
                            e.Graphics.FillRectangle(new SolidBrush(SystemColors.Highlight),
                              e.Bounds.X, e.Bounds.Y, e.Bounds.Width, e.Bounds.Height);
                        else
                            e.Graphics.FillRectangle(new SolidBrush(SystemColors.ButtonFace),
                        e.Bounds.X, e.Bounds.Y, e.Bounds.Width, e.Bounds.Height);
                    }
                    else
                    {
                        e.Graphics.FillRectangle(new SolidBrush(e.Item.BackColor),
                          e.Bounds.X, e.Bounds.Y, e.Bounds.Width, e.Bounds.Height);
                    }
                    if (HAS)
                        e.Graphics.DrawImage(Properties.Resources.check, e.Bounds.X + 2, e.Bounds.Y + 1, 16, 16);
                    else
                        e.Graphics.DrawImage(Properties.Resources.dele, e.Bounds.X + 2, e.Bounds.Y + 1, 16, 16);
                }
                #endregion
                //Gridlines
                e.Graphics.DrawLine(new Pen(new SolidBrush(Color.LightGray)), e.Bounds.X + e.Bounds.Width - 1, e.Bounds.Y,
                    e.Bounds.X + e.Bounds.Width - 1, e.Bounds.Y + e.Bounds.Height);
                e.Graphics.DrawLine(new Pen(new SolidBrush(Color.LightGray)), e.Bounds.X, e.Bounds.Y + e.Bounds.Height - 1,
                  e.Bounds.X + e.Bounds.Width, e.Bounds.Y + e.Bounds.Height - 1);
            }
        }

        //AFTER SELECT CONSOLE
        private void treeView_consoles_MouseClick(object sender, MouseEventArgs e)
        {
            _ActiveElement = ActiveElement.ConsoleOrCategory;
            if (e.Button == System.Windows.Forms.MouseButtons.Left)
            {
                if (treeView_consoles.GetNodeAt(e.Location) != null)
                {
                    treeView_consoles.SelectedNode = treeView_consoles.GetNodeAt(e.Location);
                }
                Clear();

                if (treeView_consoles.SelectedNode == null)
                    return;

                if (e.Location.X < 25)
                {
                    if (treeView_consoles.SelectedNode.IsExpanded)
                        treeView_consoles.SelectedNode.Collapse();
                    else
                        treeView_consoles.SelectedNode.Expand();
                }
                _SortAlpha = false;
                _SortMode = SortMode.None;
                ApplySortChecks();
                //CONSOLE
                if (treeView_consoles.SelectedNode.GetType() == typeof(TreeNode_Console))
                {
                    //1 refresh emulators
                    RefreshEmulatorFromConsole();
                    //2 refresh roms
                    RefreshRomsFromConsole();

                    //enable menu items
                    toolStripSeparator28.Visible = addConsoleToolStripMenuItem.Visible = addCategoryToolStripMenuItem.Visible =
                        toolStripSeparator8.Visible = romToolStripMenuItem2.Visible = addRomsToolStripMenuItem.Visible =
                        toolStripSeparator27.Visible = detectSnapshotsToolStripMenuItem1.Visible = detectCoversToolStripMenuItem1.Visible =
                   detectVideosToolStripMenuItem1.Visible = detectManualsToolStripMenuItem1.Visible = toolStripSeparator9.Visible =
              detectSoundsToolStripMenuItem.Visible = propertiesToolStripMenuItem.Visible = true;
                }
                else//CATEGORY
                {
                    RefreshEmulatorFromCategory();
                    RefreshRomsFromCategory();

                    //enable menu items
                    toolStripSeparator28.Visible = addConsoleToolStripMenuItem.Visible = addCategoryToolStripMenuItem.Visible =
                        toolStripSeparator8.Visible = romToolStripMenuItem2.Visible = addRomsToolStripMenuItem.Visible =
                        toolStripSeparator27.Visible = detectSnapshotsToolStripMenuItem1.Visible = detectCoversToolStripMenuItem1.Visible =
                  detectVideosToolStripMenuItem1.Visible = detectSoundsToolStripMenuItem.Visible = detectManualsToolStripMenuItem1.Visible =
                  toolStripSeparator9.Visible = propertiesToolStripMenuItem.Visible = false;
                }
                RomsStatusLabel1.Text = listView_roms.Items.Count.ToString();
            }
        }
        //AFTER SELECT PLAYLIST
        private void treeView_playlists_MouseClick(object sender, MouseEventArgs e)
        {
            _ActiveElement = ActiveElement.Playlist;
            if (e.Button == System.Windows.Forms.MouseButtons.Left)
            {
                if (treeView_playlists.GetNodeAt(e.Location) != null)
                {
                    treeView_playlists.SelectedNode = treeView_playlists.GetNodeAt(e.Location);
                }
                Clear(); _ActiveElement = ActiveElement.Playlist;
                if (treeView_playlists.SelectedNode == null)
                    return;
                _SortAlpha = false;
                _SortMode = SortMode.None; ApplySortChecks();
                RefreshRomsFromPlaylist();
            }
        }

        private void newProfileToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (AskToSave())
            {
                Program.Project = new Project();
                Program.ProjectPath = "";
                ClearAll();
                Frm_EnterName name = new Frm_EnterName("Enter new profile name", "");
                if (name.ShowDialog(this) == System.Windows.Forms.DialogResult.OK)
                {
                    Program.Project.Name = name.NameEntered;
                }

                Save = false;
            }
        }
        private void saveProfileToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (File.Exists(Program.ProjectPath))
            {
                if (Program.SaveProject(Program.ProjectPath))
                    Save = false;
            }
            else
            {
                SaveFileDialog Sav = new SaveFileDialog();
                Sav.Title = "Save profile";
                Sav.Filter = "Emulators Organizer Profile file (*.eop)|*.eop";
                if (Sav.ShowDialog(this) == DialogResult.OK)
                {
                    if (Program.SaveProject(Sav.FileName))
                        Save = false;
                }
            }
        }
        private void saveAsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            SaveFileDialog Sav = new SaveFileDialog();
            Sav.Title = "Save profile";
            Sav.Filter = "Emulators Organizer Profile file (*.eop)|*.eop";
            if (Sav.ShowDialog(this) == DialogResult.OK)
            {
                if (Program.SaveProject(Sav.FileName))
                    Save = false;
            }
        }
        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Close();
        }
        private void changeNameToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Frm_EnterName name = new Frm_EnterName("Enter profile name", Program.Project.Name);
            if (name.ShowDialog(this) == System.Windows.Forms.DialogResult.OK)
            {
                Program.Project.Name = name.NameEntered;
                Save = saveProfileToolStripMenuItem.Enabled;//refresh window's name
            }
        }
        private void recentProfilesToolStripMenuItem_DropDownItemClicked(object sender, ToolStripItemClickedEventArgs e)
        {
            if (File.Exists(Utilities.GetFullPath(e.ClickedItem.Text)))
            {
                if (AskToSave())
                {
                    OpenProfile(Utilities.GetFullPath(e.ClickedItem.Text));
                }
            }
            else
            {
                MessageBox.Show("This profile is not exist");
                Program.Setting.RecentProfiles.Remove(e.ClickedItem.Text);
                RefreshRecents();
            }
        }

        //Add console
        private void consoleToolStripMenuItem_Click(object sender, EventArgs e)
        {
            EmuConsole console = new EmuConsole();
            console.Name = "New Console";
            Frm_consoleProperties properties = new Frm_consoleProperties(console, Program.Project);
            if (properties.ShowDialog(this) == System.Windows.Forms.DialogResult.OK)
            {
                Program.Project.Consoles.Add(console);
                RefreshConsoles();
                Save = true;
            }
        }
        //Delete !!
        private void deleteToolStripMenuItem_Click(object sender, EventArgs e)
        {
            switch (_ActiveElement)
            {
                case ActiveElement.ConsoleOrCategory:
                    if (treeView_consoles.SelectedNode == null)
                        return;
                    if (treeView_consoles.SelectedNode.GetType() == typeof(TreeNode_Console))
                    {
                        if (MessageBox.Show(@"Are you sure you want to delete """ + treeView_consoles.SelectedNode.Text + @""" console?",
                            "Delete console", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == System.Windows.Forms.DialogResult.Yes)
                        {
                            //1 remove each rom from the playlists if found
                            foreach (PlayList playlist in Program.Project.PlayLists)
                            {
                                for (int i = 0; i < playlist.PlayListItems.Count; i++)
                                {
                                    if (playlist.PlayListItems[i].Console.Name ==
                                        ((TreeNode_Console)treeView_consoles.SelectedNode).Console.Name)
                                    {
                                        playlist.PlayListItems.Remove(playlist.PlayListItems[i]);
                                        i = -1;
                                    }
                                }
                            }
                            //2 remove the rom
                            Program.Project.Consoles.Remove(((TreeNode_Console)treeView_consoles.SelectedNode).Console);
                            ClearAll();
                            RefreshConsoles();
                            RefreshPlaylists();
                            Save = true;
                        }
                    }
                    else
                    {
                        if (MessageBox.Show(@"Are you sure you want to delete """ + treeView_consoles.SelectedNode.Text + @""" category?",
                                "Delete category", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == System.Windows.Forms.DialogResult.Yes)
                        {
                            Clear();
                            ((TreeNode_Console)treeView_consoles.SelectedNode.Parent).Console.Categories.Remove(((TreeNode_Category)treeView_consoles.SelectedNode).CategoryIn);
                            ((TreeNode_Console)treeView_consoles.SelectedNode.Parent).RefreshCategories();
                            Save = true;
                        }
                    }
                    break;
                case ActiveElement.Playlist:
                    if (treeView_playlists.SelectedNode == null)
                        return;
                    if (MessageBox.Show(@"Are you sure you want to delete """ + treeView_playlists.SelectedNode.Text + @""" playlist?",
                           "Delete playlist", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == System.Windows.Forms.DialogResult.Yes)
                    {
                        Program.Project.PlayLists.Remove(((TreeNode_PlayList)treeView_playlists.SelectedNode).PlayList);
                        Clear(); RefreshPlaylists();
                        Save = true;
                    }
                    break;
                case ActiveElement.Emulator:
                    if (listView_emulators.SelectedItems.Count == 1)
                    {
                        if (treeView_consoles.SelectedNode == null)
                        {
                            MessageBox.Show("you must select the console you want to delete emulator from.");
                            return;
                        }
                        if (treeView_consoles.SelectedNode.GetType() != typeof(TreeNode_Console))
                        {
                            MessageBox.Show("you must select the console you want to delete emulator from."); return;
                        }
                        if (tabControl3.SelectedIndex != 0)
                        {
                            MessageBox.Show("you must select the console you want to delete emulator from."); return;
                        }
                        if (MessageBox.Show(@"Are you sure you want to delete """ + listView_emulators.SelectedItems[0].Text + @""" emulator?",
                                "Delete emulator", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == System.Windows.Forms.DialogResult.Yes)
                        {
                            ((TreeNode_Console)treeView_consoles.SelectedNode).Console.Emulators.Remove(((ListViewItem_Emulator)listView_emulators.SelectedItems[0]).Emu);
                            listView_emulators.Items.Clear();
                            RefreshEmulatorFromConsole();
                            Save = true;
                        }
                    }
                    break;
                case ActiveElement.RomOrPlaylistItem:
                    if (tabControl3.SelectedIndex == 0)//Console or category
                    {
                        if (treeView_consoles.SelectedNode == null)
                            return;
                        if (treeView_consoles.SelectedNode.GetType() == typeof(TreeNode_Console))
                        {
                            if (listView_roms.SelectedItems.Count > 0)
                            {
                                if (MessageBox.Show(@"Are you sure you want to delete selected rom(s) ?" +
                                    "\nThese rom(s) will be deleted from playlists as well.",
                                    "Delete roms", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == System.Windows.Forms.DialogResult.Yes)
                                {
                                    for (int i = 0; i < listView_roms.SelectedItems.Count; i++)
                                    {
                                        //1 remove the rom from the playlists if found
                                        foreach (PlayList playlist in Program.Project.PlayLists)
                                        {
                                            foreach (PlayListItem item in playlist.PlayListItems)
                                            {
                                                if (item.Rom.Path == ((ListViewItem_Rom)listView_roms.SelectedItems[i]).ROM.Path)
                                                {
                                                    playlist.PlayListItems.Remove(item);
                                                    break;
                                                }
                                            }
                                        }
                                        //2 remove the rom ...
                                        ((TreeNode_Console)treeView_consoles.SelectedNode).Console.Roms.Remove(((ListViewItem_Rom)listView_roms.SelectedItems[i]).ROM);
                                        listView_roms.Items.Remove(listView_roms.SelectedItems[i]);
                                        i = -1;
                                    }
                                    Save = true;
                                }
                            }
                        }
                        else//category
                        {
                            if (listView_roms.SelectedItems.Count > 0)
                            {
                                if (MessageBox.Show(@"Are you sure you want to remove selected rom(s) from selected category ?",
                                     "Delete roms", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == System.Windows.Forms.DialogResult.Yes)
                                {
                                    for (int i = 0; i < listView_roms.SelectedItems.Count; i++)
                                    {
                                        //((TreeNode_Console)treeView_consoles.SelectedNode).Console.Roms.Remove(((ListViewItem_Rom)listView_roms.SelectedItems[i]).ROM);
                                        //listView_roms.Items.Remove(listView_roms.SelectedItems[i]);
                                        ((ListViewItem_Rom)listView_roms.SelectedItems[i]).ROM.Categories.Remove(((TreeNode_Category)treeView_consoles.SelectedNode).CategoryIn.Name);
                                        listView_roms.Items.Remove(listView_roms.SelectedItems[i]);
                                        i = -1;
                                    }
                                    Save = true;
                                }
                            }
                        }
                    }
                    else//playlist
                    {
                        if (treeView_consoles.SelectedNode == null)
                            return;
                        if (listView_roms.SelectedItems.Count > 0)
                        {
                            if (MessageBox.Show(@"Are you sure you want to delete selected rom(s) from selected playlist ?",
                                     "Delete roms", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == System.Windows.Forms.DialogResult.Yes)
                            {
                                for (int i = 0; i < listView_roms.SelectedItems.Count; i++)
                                {
                                    ((TreeNode_PlayList)treeView_playlists.SelectedNode).PlayList.PlayListItems.Remove(((ListViewItem_PlayListItem)listView_roms.SelectedItems[i]).PlayListItem);
                                    listView_roms.Items.Remove(listView_roms.SelectedItems[i]);
                                    i = -1;
                                }
                                Save = true;
                            }
                        }
                    }
                    break;
            }
        }
        //Rename console or category
        private void treeView_consoles_AfterLabelEdit(object sender, NodeLabelEditEventArgs e)
        {
            if (e.Label == "" | e.Label == null)
            {
                e.CancelEdit = true;
                return;
            }
            if (treeView_consoles.SelectedNode.GetType() == typeof(TreeNode_Console))
            {
                foreach (EmuConsole con in Program.Project.Consoles)
                {
                    if (e.Label == con.Name & ((TreeNode_Console)treeView_consoles.SelectedNode).Console != con)
                    {
                        MessageBox.Show("Name already taken !!");
                        e.CancelEdit = true;
                        return;
                    }
                }
                ((TreeNode_Console)treeView_consoles.SelectedNode).Console.Name = e.Label;
                ((TreeNode_Console)treeView_consoles.SelectedNode).RefreshName();
                Save = true;
            }
            else
            {
                foreach (Category con in ((TreeNode_Console)treeView_consoles.SelectedNode.Parent).Console.Categories)
                {
                    if (e.Label == con.Name & ((TreeNode_Category)treeView_consoles.SelectedNode).CategoryIn != con)
                    {
                        MessageBox.Show("Name already taken !!");
                        e.CancelEdit = true;
                        return;
                    }
                }
                ((TreeNode_Category)treeView_consoles.SelectedNode).CategoryIn.Name = e.Label;
                ((TreeNode_Category)treeView_consoles.SelectedNode).RefreshName();
                Save = true;
            }
        }
        private void renameToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            switch (_ActiveElement)
            {
                case ActiveElement.ConsoleOrCategory:
                    if (treeView_consoles.SelectedNode != null)
                        treeView_consoles.SelectedNode.BeginEdit();
                    break;
                case ActiveElement.Playlist:
                    if (treeView_playlists.SelectedNode != null)
                        treeView_playlists.SelectedNode.BeginEdit();
                    break;
                case ActiveElement.Emulator:
                    if (listView_emulators.SelectedItems.Count == 1)
                        listView_emulators.SelectedItems[0].BeginEdit();
                    break;
                case ActiveElement.RomOrPlaylistItem:
                    if (listView_roms.SelectedItems.Count == 1)
                        listView_roms.SelectedItems[0].BeginEdit();
                    break;
            }
        }
        //properties
        private void propertiesToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            switch (_ActiveElement)
            {
                case ActiveElement.ConsoleOrCategory:
                    if (treeView_consoles.SelectedNode == null)
                        return;
                    if (treeView_consoles.SelectedNode.GetType() == typeof(TreeNode_Console))
                    {
                        Frm_consoleProperties properties = new Frm_consoleProperties(((TreeNode_Console)treeView_consoles.SelectedNode).Console, Program.Project);
                        if (properties.ShowDialog(this) == System.Windows.Forms.DialogResult.OK)
                        {
                            ((TreeNode_Console)treeView_consoles.SelectedNode).RefreshName();
                            Save = true;
                        }
                    }
                    break;
                case ActiveElement.Emulator:
                    if (listView_emulators.SelectedItems.Count == 1)
                    {
                        if (tabControl3.SelectedIndex == 0)
                        {
                            if (treeView_consoles.SelectedNode == null)
                                return;
                            if (treeView_consoles.SelectedNode.GetType() == typeof(TreeNode_Console))
                            {
                                Frm_emulatorProperties em = new Frm_emulatorProperties(((ListViewItem_Emulator)listView_emulators.SelectedItems[0]).Emu,
                                    ((TreeNode_Console)treeView_consoles.SelectedNode).Console);
                                if (em.ShowDialog(this) == System.Windows.Forms.DialogResult.OK)
                                {
                                    ((ListViewItem_Emulator)listView_emulators.SelectedItems[0]).RefreshName();
                                    commandlinesEditor1.CommandLinesGroups = ((ListViewItem_Emulator)listView_emulators.SelectedItems[0]).Emu.CommandLinesGroups;
                                    commandlinesEditor1.Refresh();
                                    Save = true;
                                }
                            }
                            else
                            {
                                Frm_emulatorProperties em = new Frm_emulatorProperties(((ListViewItem_Emulator)listView_emulators.SelectedItems[0]).Emu,
                                    ((TreeNode_Console)treeView_consoles.SelectedNode.Parent).Console);
                                if (em.ShowDialog(this) == System.Windows.Forms.DialogResult.OK)
                                {
                                    ((ListViewItem_Emulator)listView_emulators.SelectedItems[0]).RefreshName();
                                    commandlinesEditor1.CommandLinesGroups = ((ListViewItem_Emulator)listView_emulators.SelectedItems[0]).Emu.CommandLinesGroups;
                                    commandlinesEditor1.Refresh();
                                    Save = true;
                                }
                            }
                        }
                        else
                        {
                            Frm_emulatorProperties em = new Frm_emulatorProperties(((ListViewItem_Emulator)listView_emulators.SelectedItems[0]).Emu,
                                   ((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Console);
                            if (em.ShowDialog(this) == System.Windows.Forms.DialogResult.OK)
                            {
                                ((ListViewItem_Emulator)listView_emulators.SelectedItems[0]).RefreshName();
                                commandlinesEditor1.CommandLinesGroups = ((ListViewItem_Emulator)listView_emulators.SelectedItems[0]).Emu.CommandLinesGroups;
                                commandlinesEditor1.Refresh();
                                Save = true;
                            }
                        }
                    }
                    break;
                case ActiveElement.RomOrPlaylistItem:
                    if (listView_roms.SelectedItems.Count == 1)
                    {
                        if (listView_roms.SelectedItems[0].GetType() == typeof(ListViewItem_Rom))
                        {
                            EmuConsole console = null;
                            if (treeView_consoles.SelectedNode == null)
                                return;
                            if (treeView_consoles.SelectedNode.GetType() == typeof(TreeNode_Console))
                                console = ((TreeNode_Console)treeView_consoles.SelectedNode).Console;
                            else
                                console = ((TreeNode_Console)treeView_consoles.SelectedNode.Parent).Console;

                            Frm_RomProperties properties = new Frm_RomProperties(((ListViewItem_Rom)listView_roms.SelectedItems[0]).ROM, console);
                            properties.ShowDialog(this);
                            Save = true;
                        }
                        else
                        {
                            Frm_RomProperties properties =
                                new Frm_RomProperties(((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Rom,
                                    ((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Console);
                            properties.ShowDialog(this);
                            Save = true;
                        }
                    }
                    break;
            }
        }
        private void treeView_consoles_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            propertiesToolStripMenuItem1_Click(this, null);
        }
        private void changeIconToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (treeView_consoles.SelectedNode == null)
                return;
            if (_ActiveElement != ActiveElement.ConsoleOrCategory)
                return;
            if (treeView_consoles.SelectedNode.GetType() == typeof(TreeNode_Console))
            {
                OpenFileDialog Op = new OpenFileDialog();
                Op.Title = "Browse for an icon";
                Op.Filter = "Icon (*.jpg;*.png;*.bmp;*.gif;*.jpeg;*.tiff;*.tga;*.ico;*.exe)|*.jpg;*.png;*.bmp;*.gif;*.jpeg;*.tiff;*.tga;*.ico;*.exe";
                Op.Multiselect = false;
                if (Op.ShowDialog(this) == DialogResult.OK)
                {
                    if (Path.GetExtension(Op.FileName).ToLower() == ".exe" | Path.GetExtension(Op.FileName).ToLower() == ".ico")
                    { ((TreeNode_Console)treeView_consoles.SelectedNode).Console.Icon = Icon.ExtractAssociatedIcon(Op.FileName).ToBitmap(); }
                    else { ((TreeNode_Console)treeView_consoles.SelectedNode).Console.Icon = Image.FromFile(Op.FileName); }
                    treeView_consoles.Refresh();
                    Save = true;
                }
            }
            else
            {
                OpenFileDialog Op = new OpenFileDialog();
                Op.Title = "Browse for an icon";
                Op.Filter = "Icon (*.jpg;*.png;*.bmp;*.gif;*.jpeg;*.tiff;*.tga;*.ico;*.exe)|*.jpg;*.png;*.bmp;*.gif;*.jpeg;*.tiff;*.tga;*.ico;*.exe";
                Op.Multiselect = false;
                if (Op.ShowDialog(this) == DialogResult.OK)
                {
                    if (Path.GetExtension(Op.FileName).ToLower() == ".exe" | Path.GetExtension(Op.FileName).ToLower() == ".ico")
                    { ((TreeNode_Category)treeView_consoles.SelectedNode).CategoryIn.Icon = Icon.ExtractAssociatedIcon(Op.FileName).ToBitmap(); }
                    else { ((TreeNode_Category)treeView_consoles.SelectedNode).CategoryIn.Icon = Image.FromFile(Op.FileName); }
                    treeView_consoles.Refresh();
                    Save = true;
                }
            }
        }
        private void tabPage7_Enter(object sender, EventArgs e)
        {
            panel1_consolesSize.Visible = true;
        }
        private void tabPage7_Leave(object sender, EventArgs e)
        {
            panel1_consolesSize.Visible = false;
        }
        private void button1_Click(object sender, EventArgs e)
        {
            if (treeView_consoles.SelectedNode == null)
                return;
            if (treeView_consoles.SelectedNode.GetType() == typeof(TreeNode_Console))
            {
                TreeNode_Console selected = (TreeNode_Console)treeView_consoles.SelectedNode;
                int index = treeView_consoles.SelectedNode.Index;

                Program.Project.Consoles.RemoveAt(index);
                treeView_consoles.Nodes.RemoveAt(index);

                if (index > 0)
                    index--;

                Program.Project.Consoles.Insert(index, selected.Console);
                treeView_consoles.Nodes.Insert(index, selected);

                treeView_consoles.SelectedNode = selected;
                Save = true;
            }
            else//category
            {
                TreeNode_Category selected = (TreeNode_Category)treeView_consoles.SelectedNode;
                int index = treeView_consoles.SelectedNode.Index;

                ((TreeNode_Console)treeView_consoles.SelectedNode.Parent).Console.Categories.RemoveAt(index);
                treeView_consoles.SelectedNode.Parent.Nodes.RemoveAt(index);

                if (index > 0)
                    index--;

                ((TreeNode_Console)treeView_consoles.SelectedNode.Parent).Console.Categories.Insert(index, selected.CategoryIn);
                treeView_consoles.SelectedNode.Parent.Nodes.Insert(index, selected);

                treeView_consoles.SelectedNode = selected;
                Save = true;
            }
        }
        private void button2_Click(object sender, EventArgs e)
        {
            if (treeView_consoles.SelectedNode == null)
                return;
            if (treeView_consoles.SelectedNode.GetType() == typeof(TreeNode_Console))
            {
                TreeNode_Console selected = (TreeNode_Console)treeView_consoles.SelectedNode;
                int index = treeView_consoles.SelectedNode.Index;

                Program.Project.Consoles.RemoveAt(index);
                treeView_consoles.Nodes.RemoveAt(index);

                if (index < treeView_consoles.Nodes.Count)
                    index++;

                Program.Project.Consoles.Insert(index, selected.Console);
                treeView_consoles.Nodes.Insert(index, selected);

                treeView_consoles.SelectedNode = selected;
                Save = true;
            }
            else//category
            {
                TreeNode_Category selected = (TreeNode_Category)treeView_consoles.SelectedNode;
                int index = treeView_consoles.SelectedNode.Index;

                ((TreeNode_Console)treeView_consoles.SelectedNode.Parent).Console.Categories.RemoveAt(index);
                treeView_consoles.SelectedNode.Parent.Nodes.RemoveAt(index);

                if (index < treeView_consoles.SelectedNode.Parent.Nodes.Count)
                    index++;

                ((TreeNode_Console)treeView_consoles.SelectedNode.Parent).Console.Categories.Insert(index, selected.CategoryIn);
                treeView_consoles.SelectedNode.Parent.Nodes.Insert(index, selected);

                treeView_consoles.SelectedNode = selected;
                Save = true;
            }
        }
        //Add category
        private void categoryToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (treeView_consoles.SelectedNode == null)
                return;
            if (treeView_consoles.SelectedNode.GetType() == typeof(TreeNode_Category))
                return;

            Frm_EnterName name = new Frm_EnterName("Enter new category name", "");
            if (name.ShowDialog(this) == System.Windows.Forms.DialogResult.OK)
            {
                int i = 0;
                foreach (Category cat in ((TreeNode_Console)treeView_consoles.SelectedNode).Console.Categories)
                {
                    if (cat.Name.ToLower() == name.NameEntered.ToLower())
                    {
                        treeView_consoles.SelectedNode = treeView_consoles.SelectedNode.Nodes[i];
                        treeView_consoles.SelectedNode.EnsureVisible();
                        return;
                    }
                    i++;
                }
                Category category = new Category();
                category.Name = name.NameEntered;
                category.Icon = ((TreeNode_Console)treeView_consoles.SelectedNode).Console.Icon;
                ((TreeNode_Console)treeView_consoles.SelectedNode).Console.Categories.Add(category);
                ((TreeNode_Console)treeView_consoles.SelectedNode).RefreshCategories();
                Save = true;
            }
        }
        private void trackBar1_Scroll(object sender, EventArgs e)
        {
            imageList_consoles.ImageSize = new Size(trackBar1.Value, trackBar1.Value);
            trackBar3.Value = trackBar2.Value = trackBar1.Value;
        }
        private void trackBar2_Scroll(object sender, EventArgs e)
        {
            imageList_consoles.ImageSize = new Size(trackBar2.Value, trackBar2.Value);
            trackBar3.Value = trackBar1.Value = trackBar2.Value;
        }
        private void tabPage8_Enter(object sender, EventArgs e)
        {
            panel1_playlist.Visible = true;
        }
        private void button4_Click(object sender, EventArgs e)
        {
            if (treeView_playlists.SelectedNode == null)
                return;
            TreeNode_PlayList selected = (TreeNode_PlayList)treeView_playlists.SelectedNode;
            int index = treeView_playlists.SelectedNode.Index;

            Program.Project.PlayLists.RemoveAt(index);
            treeView_playlists.Nodes.RemoveAt(index);

            if (index > 0)
                index--;

            Program.Project.PlayLists.Insert(index, selected.PlayList);
            treeView_playlists.Nodes.Insert(index, selected);

            treeView_playlists.SelectedNode = selected;
            Save = true;
        }
        private void button3_Click(object sender, EventArgs e)
        {
            if (treeView_playlists.SelectedNode == null)
                return;
            TreeNode_PlayList selected = (TreeNode_PlayList)treeView_playlists.SelectedNode;
            int index = treeView_playlists.SelectedNode.Index;

            Program.Project.PlayLists.RemoveAt(index);
            treeView_playlists.Nodes.RemoveAt(index);

            if (index < treeView_playlists.Nodes.Count)
                index++;

            Program.Project.PlayLists.Insert(index, selected.PlayList);
            treeView_playlists.Nodes.Insert(index, selected);

            treeView_playlists.SelectedNode = selected;
            Save = true;
        }
        //Add playlist
        private void playlistToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Frm_EnterName name = new Frm_EnterName("Enter playlist name", "New playlist");
            if (name.ShowDialog(this) == System.Windows.Forms.DialogResult.OK)
            {
                int i = 0;
                foreach (PlayList pl in Program.Project.PlayLists)
                {
                    if (pl.Name.ToLower() == name.NameEntered.ToLower())
                    {
                        treeView_playlists.SelectedNode = treeView_playlists.Nodes[i];
                        treeView_playlists.SelectedNode.EnsureVisible();
                        return;
                    }
                    i++;
                }
                PlayList playlist = new PlayList();
                playlist.Name = name.NameEntered;
                Program.Project.PlayLists.Add(playlist);
                RefreshPlaylists();
                Save = true;
            }
        }
        private void tabPage8_Leave(object sender, EventArgs e)
        {
            panel1_playlist.Visible = false;
        }
        //Rename playlist
        private void treeView_playlists_AfterLabelEdit(object sender, NodeLabelEditEventArgs e)
        {
            if (e.Label == "" | e.Label == null)
            {
                e.CancelEdit = true;
                return;
            }

            foreach (PlayList pl in Program.Project.PlayLists)
            {
                if (e.Label == pl.Name & ((TreeNode_PlayList)treeView_playlists.SelectedNode).PlayList != pl)
                {
                    MessageBox.Show("Name already taken !!");
                    e.CancelEdit = true;
                    return;
                }
            }
            ((TreeNode_PlayList)treeView_playlists.SelectedNode).PlayList.Name = e.Label;
            ((TreeNode_PlayList)treeView_playlists.SelectedNode).RefreshName();
            Save = true;
        }
        private void listView_emulators_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (listView_emulators.SelectedItems.Count == 1)
            {
                toolStripButton5_UseEmulatorCommandlines.Checked = true;
                toolStripButton6_UseRomCommandlines.Checked = false;
                CheckCommandlinesPanel(true);
                if (toolStripButton6_UseRomCommandlines.Enabled)
                {
                    toolStripButton6_Click(this, null);
                }
                //config files
                cnt_ConfigurationFileEditor1.Clear();
                listBox_configFiles.Items.Clear();
                if (((ListViewItem_Emulator)listView_emulators.SelectedItems[0]).Emu.ConfigFiles == null)
                    ((ListViewItem_Emulator)listView_emulators.SelectedItems[0]).Emu.ConfigFiles = new List<ConfigurationFile>();
                foreach (ConfigurationFile file in ((ListViewItem_Emulator)listView_emulators.SelectedItems[0]).Emu.ConfigFiles)
                    listBox_configFiles.Items.Add(file);

                _ActiveElement = ActiveElement.Emulator;
            }
            else
            {
                //config files
                cnt_ConfigurationFileEditor1.Clear();
                listBox_configFiles.Items.Clear();
            }
        }
        private void listView_emulators_AfterLabelEdit(object sender, LabelEditEventArgs e)
        {
            if (e.Label == "" | e.Label == null)
            {
                e.CancelEdit = true; return;
            }
            if (listView_emulators.SelectedItems.Count == 1)
            {
                foreach (ListViewItem_Emulator IT in listView_emulators.Items)
                {
                    if (IT.Text.ToLower() == e.Label.ToLower())
                    {
                        MessageBox.Show("Name already taken !!");
                        e.CancelEdit = true; return;
                    }
                }
                ((ListViewItem_Emulator)listView_emulators.SelectedItems[0]).Emu.Name = e.Label;
                ((ListViewItem_Emulator)listView_emulators.SelectedItems[0]).RefreshName();
                Save = true;
            }
        }
        //ADD EMULATOR
        private void emulatorToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (treeView_consoles.SelectedNode == null)
            {
                MessageBox.Show("you must select the console you want to add emulator for.");
                return;
            }
            if (treeView_consoles.SelectedNode.GetType() != typeof(TreeNode_Console))
            {
                MessageBox.Show("you must select the console you want to add emulator for."); return;
            }
            if (tabControl3.SelectedIndex != 0)
            {
                MessageBox.Show("you must select the console you want to add emulator for."); return;
            }
            Emulator newEmulator = new Emulator();
            Frm_emulatorProperties em = new Frm_emulatorProperties(newEmulator,
                ((TreeNode_Console)treeView_consoles.SelectedNode).Console);
            if (em.ShowDialog(this) == System.Windows.Forms.DialogResult.OK)
            {
                ((TreeNode_Console)treeView_consoles.SelectedNode).Console.Emulators.Add(newEmulator);
                listView_emulators.Items.Clear();
                RefreshEmulatorFromConsole();
                listView_emulators.Items[listView_emulators.Items.Count - 1].Selected = true;
                listView_emulators.Items[listView_emulators.Items.Count - 1].EnsureVisible();
                Save = true;
            }
        }
        private void listView_emulators_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            if (listView_emulators.SelectedItems.Count == 1)
            {
                try
                {
                    Process.Start(Utilities.GetFullPath(((ListViewItem_Emulator)listView_emulators.SelectedItems[0]).Emu.Path));
                }
                catch (Exception ex)
                { MessageBox.Show(ex.Message); }
            }
        }
        private void toolStripButton5_Click(object sender, EventArgs e)
        {
            splitContainer5.Panel2Collapsed = !toolStripButton_commandlinesShowHide.Checked;
        }
        private void toolStripButton6_Click(object sender, EventArgs e)
        {
            toolStripButton5_UseEmulatorCommandlines.Checked = false;
            toolStripButton6_UseRomCommandlines.Checked = true; CheckCommandlinesPanel(false);
        }
        private void toolStripButton5_Click_1(object sender, EventArgs e)
        {
            toolStripButton5_UseEmulatorCommandlines.Checked = true;
            toolStripButton6_UseRomCommandlines.Checked = false; CheckCommandlinesPanel(true);
        }
        private void detailsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            listView_roms.View = View.Details;
            detailsToolStripMenuItem1.Checked = detailsToolStripMenuItem.Checked = true;
            largeIconsToolStripMenuItem.Checked = largIconsToolStripMenuItem.Checked = false;
            ViewChanged();
        }
        private void largIconsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            listView_roms.View = View.LargeIcon;
            detailsToolStripMenuItem1.Checked = detailsToolStripMenuItem.Checked = false;
            largeIconsToolStripMenuItem.Checked = largIconsToolStripMenuItem.Checked = true;
            ViewChanged();
        }
        private void listView_roms_MouseClick(object sender, MouseEventArgs e)
        {
            if (listView_roms.View == View.Details)
            {
                if (listView_roms.SelectedItems.Count == 1)
                {
                    if (e.X >= RatingX
                        & e.X <= RatingX + columnHeader1_rating.Width
                        & e.Y >= listView_roms.SelectedItems[0].Position.Y
                        & e.Y <= listView_roms.SelectedItems[0].Position.Y + 17)
                    {
                        IsRating = true;
                        //calculat offset
                        int offset = e.X - RatingX;
                        int rat = 0;
                        if (offset < 15)
                            rat = 1;
                        else if (offset >= 15 & offset < 30)
                            rat = 2;
                        else if (offset >= 30 & offset < 45)
                            rat = 3;
                        else if (offset >= 45 & offset < 60)
                            rat = 4;
                        else if (offset >= 60 & offset < 75)
                            rat = 5;
                        else if (offset >= 75)
                            rat = 0;
                        if (listView_roms.SelectedItems[0].GetType() == typeof(ListViewItem_Rom))
                        {
                            ListViewItem_Rom roooo = (ListViewItem_Rom)listView_roms.SelectedItems[0];
                            roooo.ROM.Rating = rat;
                            if (roooo.Index + 1 < listView_roms.Items.Count)
                                listView_roms.RedrawItems(roooo.Index, roooo.Index + 1, false);
                            else
                                listView_roms.RedrawItems(roooo.Index, roooo.Index, false);
                        }
                        else
                        {
                            ListViewItem_PlayListItem roooo = (ListViewItem_PlayListItem)listView_roms.SelectedItems[0];
                            roooo.PlayListItem.Rom.Rating = rat;
                            if (roooo.Index + 1 < listView_roms.Items.Count)
                                listView_roms.RedrawItems(roooo.Index, roooo.Index + 1, false);
                            else
                                listView_roms.RedrawItems(roooo.Index, roooo.Index, false);
                        }
                        Save = true;
                    }
                    else
                    {
                        IsRating = false;
                    }
                }
            }
        }
        private void listView_roms_AfterLabelEdit(object sender, LabelEditEventArgs e)
        {
            if (e.Label == "" | e.Label == null)
            {
                e.CancelEdit = true;
                return;
            }

            if (listView_roms.SelectedItems[0].GetType() == typeof(ListViewItem_Rom))
            {
                if (treeView_consoles.SelectedNode.GetType() == typeof(TreeNode_Console))
                {
                    foreach (Rom rom in ((TreeNode_Console)treeView_consoles.SelectedNode).Console.Roms)
                    {
                        if (e.Label.ToLower() == rom.Name.ToLower())
                        {
                            e.CancelEdit = true;
                            MessageBox.Show("Name already taken by another rom");
                            return;
                        }
                    }
                }
                else
                {
                    foreach (Rom rom in ((TreeNode_Console)treeView_consoles.SelectedNode.Parent).Console.Roms)
                    {
                        if (e.Label.ToLower() == rom.Name.ToLower())
                        {
                            e.CancelEdit = true;
                            MessageBox.Show("Name already taken by another rom");
                            return;
                        }
                    }
                }
                ((ListViewItem_Rom)listView_roms.SelectedItems[0]).ROM.Name = e.Label;
                Save = true;
            }
            else
            {
                foreach (Rom rom in ((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Console.Roms)
                {
                    if (e.Label.ToLower() == rom.Name.ToLower())
                    {
                        e.CancelEdit = true;
                        MessageBox.Show("Name already taken by another rom in the parent console '" + ((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Console.Name + "'");
                        return;
                    }
                }
                ((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Rom.Name = e.Label;
                Save = true;
            }
            e.CancelEdit = false;
            Save = true;
        }
        private void listView_roms_BeforeLabelEdit(object sender, LabelEditEventArgs e)
        {
            if (IsRating)
                e.CancelEdit = true;
        }
        //PLAY ROM
        private void listView_roms_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            RunRom();
        }
        private void toolStripButton8_EnableEmulator_CheckedChanged(object sender, EventArgs e)
        {
            toolStripButton7_EnableCommandlines.Enabled = toolStripButton8_EnableEmulator.Checked;
            useEmulatorToolStripMenuItem.Checked = toolStripButton8_EnableEmulator.Checked;
        }
        //After select rom...
        private void listView_roms_SelectedIndexChanged(object sender, EventArgs e)
        {
            _ActiveElement = ActiveElement.RomOrPlaylistItem;
            countDown = 2;
            timer1_AfterSelectRoms.Start();
        }
        private void listView_roms_Click(object sender, EventArgs e)
        {
            _ActiveElement = ActiveElement.RomOrPlaylistItem;
        }
        private void listView_roms_ColumnReordered(object sender, ColumnReorderedEventArgs e)
        {
            if (e.Header == columnHeader_name | e.NewDisplayIndex == 0)
                e.Cancel = true;
        }
        private void playToolStripMenuItem_Click(object sender, EventArgs e)
        {
            RunRom();
        }
        private void listView_roms_KeyUp(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Return)
                RunRom();
        }
        private void tabControl3_SelectedIndexChanged(object sender, EventArgs e)
        {
            Clear();
            if (tabControl3.SelectedIndex == 0)
            {
                toolStrip4_copypaste.Visible = false;
                toolStripSeparator33.Visible = copyToolStripMenuItem.Visible = cutToolStripMenuItem.Visible =
                cutToolStripMenuItem1.Visible = copyToolStripMenuItem1.Visible = pasteBeforeToolStripMenuItem1.Visible =
           toolStripSeparator34.Visible = pasteAfterToolStripMenuItem.Visible = pasteBeforeToolStripMenuItem.Visible =
           pasteToolStripMenuItem.Visible = false;

                if (treeView_consoles.SelectedNode != null)
                {
                    _ActiveElement = ActiveElement.ConsoleOrCategory; _SortAlpha = false;
                    _SortMode = SortMode.None;
                    //CONSOLE
                    if (treeView_consoles.SelectedNode.GetType() == typeof(TreeNode_Console))
                    {
                        //1 refresh emulators
                        RefreshEmulatorFromConsole();
                        //2 refresh roms
                        RefreshRomsFromConsole();
                    }
                    else//CATEGORY
                    {
                        RefreshEmulatorFromCategory();
                        RefreshRomsFromCategory();
                    }
                }
            }
            else
            {
                playlistToolStripMenuItem2.DropDownItems.Clear();
                categoryToolStripMenuItem2.DropDownItems.Clear();
                playlistToolStripMenuItem2.Enabled = categoryToolStripMenuItem2.Enabled = false;
                toolStripSeparator33.Visible = copyToolStripMenuItem.Visible = cutToolStripMenuItem.Visible =
                cutToolStripMenuItem1.Visible = copyToolStripMenuItem1.Visible = pasteBeforeToolStripMenuItem1.Visible =
           toolStripSeparator34.Visible = pasteAfterToolStripMenuItem.Visible = pasteBeforeToolStripMenuItem.Visible =
           pasteToolStripMenuItem.Visible = true;
                toolStrip4_copypaste.Visible = true;
                Clear(); _ActiveElement = ActiveElement.Playlist;
                if (treeView_playlists.SelectedNode == null)
                    return;
                _SortAlpha = false;
                _SortMode = SortMode.None;
                RefreshRomsFromPlaylist();
            }
        }
        private void listView_emulators_Click(object sender, EventArgs e)
        {
            if (listView_emulators.SelectedItems.Count == 1)
            {
                _ActiveElement = ActiveElement.Emulator;
                toolStripButton5_UseEmulatorCommandlines.Checked = true;
                toolStripButton6_UseRomCommandlines.Checked = false;
                CheckCommandlinesPanel(true);
                if (toolStripButton6_UseRomCommandlines.Enabled)
                {
                    toolStripButton6_Click(this, null);
                }
            }
        }
        private void romStatus1_RatingChanged(object sender, EventArgs e)
        {
            if (listView_roms.SelectedItems[0].Index + 1 < listView_roms.Items.Count)
                listView_roms.RedrawItems(listView_roms.SelectedItems[0].Index, listView_roms.SelectedItems[0].Index + 1, false);
            else
                listView_roms.RedrawItems(listView_roms.SelectedItems[0].Index, listView_roms.SelectedItems[0].Index, false);

            Save = true;
        }
        //Add roms
        private void romsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (tabControl3.SelectedIndex == 0)
            {
                if (treeView_consoles.SelectedNode == null)
                {
                    MessageBox.Show("You must select console to add roms.");
                    return;
                }
                if (treeView_consoles.SelectedNode.GetType() == typeof(TreeNode_Category))
                {
                    MessageBox.Show("You must select console to add roms.");
                    return;
                }
                Frm_AddRoms addroms = new Frm_AddRoms(((TreeNode_Console)treeView_consoles.SelectedNode).Console);
                if (addroms.ShowDialog(this) == System.Windows.Forms.DialogResult.OK)
                {
                    Clear();
                    RefreshEmulatorFromConsole();
                    RefreshRomsFromConsole();
                    Save = true;
                }
            }
            else
            {
                MessageBox.Show("You must select console to add roms.");
            }
        }
        private void detectSnapshotsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (tabControl3.SelectedIndex == 0)
            {
                if (treeView_consoles.SelectedNode == null)
                {
                    MessageBox.Show("You must select console to detect snapshots.");
                    return;
                }
                if (treeView_consoles.SelectedNode.GetType() == typeof(TreeNode_Category))
                {
                    MessageBox.Show("You must select console to detect snapshots.");
                    return;
                }
                Frm_DetectSnapshotsCovers detect = new Frm_DetectSnapshotsCovers("Detect snapshots",
                    ((TreeNode_Console)treeView_consoles.SelectedNode).Console, false, false);
                if (detect.ShowDialog(this) == System.Windows.Forms.DialogResult.OK)
                {
                    Save = true;
                    listView_roms.Refresh();
                }
            }
            else
            {
                MessageBox.Show("You must select console to detect snapshots.");
            }
        }
        private void detectCoversToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (tabControl3.SelectedIndex == 0)
            {
                if (treeView_consoles.SelectedNode == null)
                {
                    MessageBox.Show("You must select console to detect covers.");
                    return;
                }
                if (treeView_consoles.SelectedNode.GetType() == typeof(TreeNode_Category))
                {
                    MessageBox.Show("You must select console to detect covers.");
                    return;
                }
                Frm_DetectSnapshotsCovers detect = new Frm_DetectSnapshotsCovers("Detect covers",
                    ((TreeNode_Console)treeView_consoles.SelectedNode).Console, true, false);
                if (detect.ShowDialog(this) == System.Windows.Forms.DialogResult.OK)
                { Save = true; listView_roms.Refresh(); }
            }
            else
            {
                MessageBox.Show("You must select console to detect covers.");
            }
        }
        private void detectManualsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (tabControl3.SelectedIndex == 0)
            {
                if (treeView_consoles.SelectedNode == null)
                {
                    MessageBox.Show("You must select console to detect manuals.");
                    return;
                }
                if (treeView_consoles.SelectedNode.GetType() == typeof(TreeNode_Category))
                {
                    MessageBox.Show("You must select console to detect manuals.");
                    return;
                }
                Frm_DetectSnapshotsCovers detect = new Frm_DetectSnapshotsCovers("Detect manuals",
                    ((TreeNode_Console)treeView_consoles.SelectedNode).Console, true, true);
                if (detect.ShowDialog(this) == System.Windows.Forms.DialogResult.OK)
                {
                    Save = true;
                    listView_roms.Refresh();
                }
            }
            else
            {
                MessageBox.Show("You must select console to detect manuals.");
            }
        }
        private void cntr_ImagesBrowser_snaps_ImageAdded(object sender, EventArgs e)
        {
            Save = true;
            //refresh other controls
            countDown = 2;
            timer1_AfterSelectRoms.Start();
        }
        private void cntr_ImagesBrowser_snaps_ImageRemoved(object sender, EventArgs e)
        {
            Save = true;
            //refresh other controls
            countDown = 2;
            timer1_AfterSelectRoms.Start();
        }
        //open
        private void contextMenuStrip1_roms_Opening(object sender, CancelEventArgs e)
        {
            if (tabControl3.SelectedIndex == 0)
            {
                if (treeView_consoles.SelectedNode != null)
                {
                    if (treeView_consoles.SelectedNode.GetType() == typeof(TreeNode_Console))
                        toolStripSeparator42.Visible = addRomToolStripMenuItem.Visible = addRomsfolderScanToolStripMenuItem.Visible = true;
                    else
                        toolStripSeparator42.Visible = addRomToolStripMenuItem.Visible = addRomsfolderScanToolStripMenuItem.Visible = false;
                }
                else
                    toolStripSeparator42.Visible = addRomToolStripMenuItem.Visible = addRomsfolderScanToolStripMenuItem.Visible = false;
                changeIconToolStripMenuItem1.Visible = ResetIconMenuItem1.Visible = toolStripSeparator26.Visible = false;
            }
            else
            {
                toolStripSeparator42.Visible = addRomToolStripMenuItem.Visible = addRomsfolderScanToolStripMenuItem.Visible = false;
                changeIconToolStripMenuItem1.Visible = ResetIconMenuItem1.Visible = toolStripSeparator26.Visible = true;
            }
            playlistToolStripMenuItem2.DropDownItems.Clear();
            categoryToolStripMenuItem2.DropDownItems.Clear();
            if (treeView_consoles.SelectedNode == null)
                return;
            //categories
            if (treeView_consoles.SelectedNode.GetType() == typeof(TreeNode_Console))
            {
                foreach (Category cat in ((TreeNode_Console)treeView_consoles.SelectedNode).Console.Categories)
                {
                    categoryToolStripMenuItem2.DropDownItems.Add(cat.Name);
                }
            }
            else
            {
                foreach (Category cat in ((TreeNode_Console)treeView_consoles.SelectedNode.Parent).Console.Categories)
                {
                    categoryToolStripMenuItem2.DropDownItems.Add(cat.Name);
                }
            }
            categoryToolStripMenuItem2.Enabled = (categoryToolStripMenuItem2.DropDownItems.Count != 0);
            //playlists
            foreach (PlayList playlist in Program.Project.PlayLists)
            {
                playlistToolStripMenuItem2.DropDownItems.Add(playlist.Name);
            }
            playlistToolStripMenuItem2.Enabled = (playlistToolStripMenuItem2.DropDownItems.Count != 0);


        }
        //send to category
        private void categoryToolStripMenuItem2_DropDownItemClicked(object sender, ToolStripItemClickedEventArgs e)
        {
            if (listView_roms.SelectedItems.Count == 0)
                return;
            foreach (ListViewItem_Rom ro in listView_roms.SelectedItems)
            {
                if (ro.ROM.Categories == null)
                    ro.ROM.Categories = new List<string>();
                bool canadd = true;
                //see if this category is already exist
                foreach (string cat in ro.ROM.Categories)
                {
                    if (cat == e.ClickedItem.Text)
                    { canadd = false; break; }
                }
                if (canadd)
                {
                    ro.ROM.Categories.Add(e.ClickedItem.Text);
                }
            }
            Save = true;
            StatusLabel.Text = "Done, " + listView_roms.SelectedItems.Count.ToString() +
              " rom(s) added on " + e.ClickedItem.Text + " category.";
        }
        private void playlistToolStripMenuItem2_DropDownItemClicked(object sender, ToolStripItemClickedEventArgs e)
        {
            if (listView_roms.SelectedItems.Count == 0)
                return;
            if (treeView_consoles.SelectedNode == null)
                return;
            if (tabControl3.SelectedIndex != 0)
            {
                MessageBox.Show("You must select roms from console");
                return;
            }
            TreeNode_Console TR;
            if (treeView_consoles.SelectedNode.GetType() == typeof(TreeNode_Console))
                TR = (TreeNode_Console)treeView_consoles.SelectedNode;
            else
                TR = (TreeNode_Console)treeView_consoles.SelectedNode.Parent;
            foreach (PlayList PL in Program.Project.PlayLists)
            {
                if (PL.Name == e.ClickedItem.Text)
                {
                    foreach (ListViewItem_Rom roo in listView_roms.SelectedItems)
                    {
                        PlayListItem IT = new PlayListItem();
                        IT.Rom = roo.ROM;
                        IT.Console = TR.Console;
                        bool found = false;
                        foreach (PlayListItem ittt in PL.PlayListItems)
                        {
                            if (IT.Rom.Path == ittt.Rom.Path)
                            { found = true; break; }
                        }
                        if (!found)
                            PL.PlayListItems.Add(IT);
                    }
                    break;
                }
            }
            Save = true;
            StatusLabel.Text = "Roms sent to playlist";
        }
        //real delete
        private void realDeleteToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            if (listView_roms.SelectedItems.Count == 0)
                return;
            if (tabControl3.SelectedIndex == 0)
            {
                bool DeleteSnapsCoversManuals = false;
                if (MessageBox.Show("Do you want to delete snapshots, covers, videos, sounds and manuals of rom(s) too ?", "Delete roms associated files", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == System.Windows.Forms.DialogResult.Yes)
                { DeleteSnapsCoversManuals = true; }
                DialogResult mess = MessageBox.Show("This will delete selected rom FILES from the REAL STORAGE disk.\nAre you sure you want to do that ?", "Real delete roms", MessageBoxButtons.YesNo, MessageBoxIcon.Warning);
                if (mess == DialogResult.Yes)
                {
                    int unremoved = 0;
                    TreeNode_Console TR;
                    if (treeView_consoles.SelectedNode.GetType() == typeof(TreeNode_Console))
                        TR = (TreeNode_Console)treeView_consoles.SelectedNode;
                    else
                        TR = (TreeNode_Console)treeView_consoles.SelectedNode.Parent;
                    for (int i = 0; i < listView_roms.SelectedItems.Count; i++)
                    {
                        ListViewItem_Rom ro = (ListViewItem_Rom)listView_roms.SelectedItems[i];
                        //Delete the roms from the disk
                        if (File.Exists(Utilities.GetFullPath(ro.ROM.Path)))
                        {
                            FileInfo fiinf = new FileInfo(Utilities.GetFullPath(ro.ROM.Path));
                            try
                            {
                                fiinf.IsReadOnly = false;
                                File.Delete(Utilities.GetFullPath(ro.ROM.Path));

                            }
                            catch { unremoved++; }
                        }
                        if (DeleteSnapsCoversManuals)
                        {
                            cntr_ImagesBrowser_covers.Rom = null;
                            cntr_ImagesBrowser_snaps.Rom = null;
                            cntr_ImagesBrowser_covers2.Rom = null;
                            cntr_ImagesBrowser_snaps2.Rom = null;
                            pdFviewer1.PDFPath = "";
                            videoPlayer1.Rom = null;
                            soundPlayer1.Rom = null;
                            romStatus1.Rom = null;
                            foreach (string snap in ro.ROM.Snapshots)
                            {
                                try
                                {
                                    FileInfo fiinf = new FileInfo(Utilities.GetFullPath(snap));
                                    fiinf.IsReadOnly = false;
                                    File.Delete(Utilities.GetFullPath(snap));
                                }
                                catch { }
                            }
                            foreach (string snap in ro.ROM.Covers)
                            {
                                try
                                {
                                    FileInfo fiinf = new FileInfo(Utilities.GetFullPath(snap));
                                    fiinf.IsReadOnly = false;
                                    File.Delete(Utilities.GetFullPath(snap));
                                }
                                catch { }
                            }
                            foreach (string snap in ro.ROM.SoundRecords)
                            {
                                try
                                {
                                    FileInfo fiinf = new FileInfo(Utilities.GetFullPath(snap));
                                    fiinf.IsReadOnly = false;
                                    File.Delete(Utilities.GetFullPath(snap));
                                }
                                catch { }
                            }
                            try
                            {
                                FileInfo fiinf = new FileInfo(Utilities.GetFullPath(ro.ROM.PDFPath));
                                fiinf.IsReadOnly = false;
                                File.Delete(Utilities.GetFullPath(ro.ROM.PDFPath));
                            }
                            catch { }
                            try
                            {
                                foreach (string snap in ro.ROM.Videos)
                                {
                                    try
                                    {
                                        FileInfo fiinf = new FileInfo(Utilities.GetFullPath(snap));
                                        fiinf.IsReadOnly = false;
                                        File.Delete(Utilities.GetFullPath(snap));
                                    }
                                    catch { }
                                }
                            }
                            catch { }
                            //info and history
                            try
                            {
                                FileInfo fiinf = new FileInfo(Utilities.GetFullPath(TR.Console.InfoFolder) + "\\"
                                    + Path.GetFileNameWithoutExtension(ro.ROM.Path) + ".txt");
                                fiinf.IsReadOnly = false;
                                File.Delete(Utilities.GetFullPath(TR.Console.InfoFolder) + "\\"
                                    + Path.GetFileNameWithoutExtension(ro.ROM.Path) + ".txt");
                            }
                            catch { }
                            try
                            {
                                FileInfo fiinf = new FileInfo(Utilities.GetFullPath(TR.Console.InfoFolder) + "\\"
                                    + Path.GetFileNameWithoutExtension(ro.ROM.Path) + ".rtf");
                                fiinf.IsReadOnly = false;
                                File.Delete(Utilities.GetFullPath(TR.Console.InfoFolder) + "\\"
                                    + Path.GetFileNameWithoutExtension(ro.ROM.Path) + ".rtf");
                            }
                            catch { }
                        }
                        //Delete the roms from the collection
                        //search playlists for this rom to remove it from playlists as well ...
                        foreach (PlayList playlist in Program.Project.PlayLists)
                        {
                            for (int j = 0; j < playlist.PlayListItems.Count; i++)
                            {
                                if (playlist.PlayListItems[j].Console.Name == TR.Console.Name)
                                {
                                    playlist.PlayListItems.Remove(playlist.PlayListItems[j]);
                                    j = -1;
                                }
                            }
                        }
                        TR.Console.Roms.Remove(ro.ROM);
                        listView_roms.Items.Remove(listView_roms.SelectedItems[i]);
                        i = -1;
                    }
                    Save = true;
                    if (unremoved == 0)
                        StatusLabel.Text = "Done";
                    else
                        StatusLabel.Text = "Done, could not delete " + unremoved.ToString() + " rom(s) from the disk";
                }
            }
            else
            {
                if (treeView_playlists.SelectedNode == null)
                    return;
                bool DeleteSnapsCoversManuals = false;
                if (MessageBox.Show("Do you want to delete snapshots, covers, videos, sounds and manuals of rom(s) too ?", "Delete roms associated files", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == System.Windows.Forms.DialogResult.Yes)
                { DeleteSnapsCoversManuals = true; }
                DialogResult mess = MessageBox.Show("This will delete selected rom FILES from the REAL STORAGE disk.\nAre you sure you want to do that ?", "Real delete roms", MessageBoxButtons.YesNo, MessageBoxIcon.Warning);
                if (mess == DialogResult.Yes)
                {
                    int unremoved = 0;
                    TreeNode_PlayList TR = (TreeNode_PlayList)treeView_playlists.SelectedNode;
                    for (int i = 0; i < listView_roms.SelectedItems.Count; i++)
                    {
                        ListViewItem_PlayListItem ro = (ListViewItem_PlayListItem)listView_roms.SelectedItems[i];
                        //Delete the roms from the disk
                        if (File.Exists(Utilities.GetFullPath(ro.PlayListItem.Rom.Path)))
                        {
                            FileInfo fiinf = new FileInfo(Utilities.GetFullPath(ro.PlayListItem.Rom.Path));
                            try
                            {
                                fiinf.IsReadOnly = false;
                                File.Delete(Utilities.GetFullPath(ro.PlayListItem.Rom.Path));

                            }
                            catch { unremoved++; }
                        }
                        if (DeleteSnapsCoversManuals)
                        {
                            cntr_ImagesBrowser_covers.Rom = null;
                            cntr_ImagesBrowser_snaps.Rom = null;
                            cntr_ImagesBrowser_covers2.Rom = null;
                            cntr_ImagesBrowser_snaps2.Rom = null;
                            pdFviewer1.PDFPath = "";
                            videoPlayer1.Rom = null;
                            soundPlayer1.Rom = null;
                            romStatus1.Rom = null;
                            foreach (string snap in ro.PlayListItem.Rom.Snapshots)
                            {
                                try
                                {
                                    FileInfo fiinf = new FileInfo(Utilities.GetFullPath(snap));
                                    fiinf.IsReadOnly = false;
                                    File.Delete(Utilities.GetFullPath(snap));
                                }
                                catch { }
                            }
                            foreach (string snap in ro.PlayListItem.Rom.Covers)
                            {
                                try
                                {
                                    FileInfo fiinf = new FileInfo(Utilities.GetFullPath(snap));
                                    fiinf.IsReadOnly = false;
                                    File.Delete(Utilities.GetFullPath(snap));
                                }
                                catch { }
                            }
                            foreach (string snap in ro.PlayListItem.Rom.SoundRecords)
                            {
                                try
                                {
                                    FileInfo fiinf = new FileInfo(Utilities.GetFullPath(snap));
                                    fiinf.IsReadOnly = false;
                                    File.Delete(Utilities.GetFullPath(snap));
                                }
                                catch { }
                            }
                            try
                            {
                                FileInfo fiinf = new FileInfo(Utilities.GetFullPath(ro.PlayListItem.Rom.PDFPath));
                                fiinf.IsReadOnly = false;
                                File.Delete(Utilities.GetFullPath(ro.PlayListItem.Rom.PDFPath));
                            }
                            catch { }
                            try
                            {
                                foreach (string snap in ro.PlayListItem.Rom.Videos)
                                {
                                    try
                                    {
                                        FileInfo fiinf = new FileInfo(Utilities.GetFullPath(snap));
                                        fiinf.IsReadOnly = false;
                                        File.Delete(Utilities.GetFullPath(snap));
                                    }
                                    catch { }
                                }
                            }
                            catch { }
                            //info and history
                            try
                            {
                                FileInfo fiinf = new FileInfo(Utilities.GetFullPath(ro.PlayListItem.Console.InfoFolder) + "\\"
                                    + Path.GetFileNameWithoutExtension(ro.PlayListItem.Rom.Path) + ".txt");
                                fiinf.IsReadOnly = false;
                                File.Delete(Utilities.GetFullPath(ro.PlayListItem.Console.InfoFolder) + "\\"
                                    + Path.GetFileNameWithoutExtension(ro.PlayListItem.Rom.Path) + ".txt");
                            }
                            catch { }
                            try
                            {
                                FileInfo fiinf = new FileInfo(Utilities.GetFullPath(ro.PlayListItem.Console.InfoFolder) + "\\"
                                    + Path.GetFileNameWithoutExtension(ro.PlayListItem.Rom.Path) + ".rtf");
                                fiinf.IsReadOnly = false;
                                File.Delete(Utilities.GetFullPath(ro.PlayListItem.Console.InfoFolder) + "\\"
                                    + Path.GetFileNameWithoutExtension(ro.PlayListItem.Rom.Path) + ".rtf");
                            }
                            catch { }
                        }
                        //Delete the roms from the collection
                        ro.PlayListItem.Console.Roms.Remove(ro.PlayListItem.Rom);
                        TR.PlayList.PlayListItems.Remove(ro.PlayListItem);
                        listView_roms.Items.Remove(listView_roms.SelectedItems[i]);
                        i = -1;
                    }
                    Save = true;
                    if (unremoved == 0)
                        StatusLabel.Text = "Done";
                    else
                        StatusLabel.Text = "Done, could not delete " + unremoved.ToString() + " rom(s) from the disk";
                }
            }
        }
        private void realRenameToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (listView_roms.SelectedItems.Count != 1)
                return;
            if (tabControl3.SelectedIndex == 0)
            {
                TreeNode_Console TR;
                if (treeView_consoles.SelectedNode.GetType() == typeof(TreeNode_Console))
                    TR = (TreeNode_Console)treeView_consoles.SelectedNode;
                else
                    TR = (TreeNode_Console)treeView_consoles.SelectedNode.Parent;
                ListViewItem_Rom IT = (ListViewItem_Rom)listView_roms.SelectedItems[0];
                string nn = Path.GetFileNameWithoutExtension(IT.ROM.Path);
                string ext = Path.GetExtension(IT.ROM.Path);
                if (!File.Exists(Utilities.GetFullPath(IT.ROM.Path)))
                {
                    MessageBox.Show("You can't rename this rom because of this rom file doesn't exist");
                    return;
                }
                Frm_EnterName nam = new Frm_EnterName("Enter the new rom name", nn);
                if (nam.ShowDialog(this) == System.Windows.Forms.DialogResult.OK)
                {
                    if (nam.NameEntered == nn)
                        return;

                    if (nam.NameEntered == "")
                        return;
                    bool _FileRenamed = false;
                    bool _SnapsRenamed = false;
                    bool _CoversRenamed = false;
                    bool _ManualRenamed = false;
                    bool _InfoRenamed = false;
                    bool _HistoryRenamed = false;
                    bool _VideoRenamed = false;
                    bool _SoundRenamed = false;
                    try
                    {
                        string fol = Path.GetDirectoryName(Utilities.GetFullPath(IT.ROM.Path));
                        if (fol == "")
                            fol = Path.GetPathRoot(Utilities.GetFullPath(IT.ROM.Path));

                        string Orgenal = Utilities.GetFullPath(IT.ROM.Path);
                        string Neww = fol + "\\" + nam.NameEntered + ext;
                        File.Copy(Orgenal, Neww);
                        FileInfo inf = new FileInfo(Orgenal);
                        inf.IsReadOnly = false;
                        File.Delete(Orgenal);

                        IT.ROM.Name = nam.NameEntered;
                        IT.ROM.Path = Neww;
                        IT.RefreshName();
                        _FileRenamed = true;
                        if (MessageBox.Show("Do you want to rename snapshots, covers, info, history, video, sounds and manual of this rom too ?", "Rename rom associated files", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == System.Windows.Forms.DialogResult.Yes)
                        {
                            cntr_ImagesBrowser_covers.Rom = null;
                            cntr_ImagesBrowser_snaps.Rom = null;
                            cntr_ImagesBrowser_covers2.Rom = null;
                            cntr_ImagesBrowser_snaps2.Rom = null;
                            pdFviewer1.PDFPath = "";
                            videoPlayer1.Rom = null;
                            soundPlayer1.Rom = null;
                            romStatus1.Rom = null;
                            //Snapshots
                            for (int i = 0; i < IT.ROM.Snapshots.Count; i++)
                            {
                                if (File.Exists(Utilities.GetFullPath(IT.ROM.Snapshots[i])))
                                {
                                    nn = Path.GetFileNameWithoutExtension(IT.ROM.Snapshots[i]);
                                    ext = Path.GetExtension(IT.ROM.Snapshots[i]);
                                    fol = Path.GetDirectoryName(Utilities.GetFullPath(IT.ROM.Snapshots[i]));
                                    if (fol == "")
                                        fol = Path.GetPathRoot(Utilities.GetFullPath(IT.ROM.Snapshots[i]));

                                    Orgenal = Utilities.GetFullPath(IT.ROM.Snapshots[i]);
                                    int j = 1;
                                    while (File.Exists(fol + "\\" + nam.NameEntered + "(" + j + ")" + ext))
                                        j++;
                                    Neww = fol + "\\" + nam.NameEntered + "(" + j + ")" + ext;
                                    File.Copy(Orgenal, Neww);
                                    inf = new FileInfo(Orgenal);
                                    inf.IsReadOnly = false;
                                    File.Delete(Orgenal);
                                    IT.ROM.Snapshots[i] = Neww;
                                }
                                else
                                {
                                }
                            }
                            _SnapsRenamed = true;
                            //Covers
                            for (int i = 0; i < IT.ROM.Covers.Count; i++)
                            {
                                if (File.Exists(Utilities.GetFullPath(IT.ROM.Covers[i])))
                                {
                                    nn = Path.GetFileNameWithoutExtension(IT.ROM.Covers[i]);
                                    ext = Path.GetExtension(IT.ROM.Covers[i]);
                                    fol = Path.GetDirectoryName(Utilities.GetFullPath(IT.ROM.Covers[i]));
                                    if (fol == "")
                                        fol = Path.GetPathRoot(Utilities.GetFullPath(IT.ROM.Covers[i]));

                                    Orgenal = Utilities.GetFullPath(IT.ROM.Covers[i]);
                                    int j = 1;
                                    while (File.Exists(fol + "\\" + nam.NameEntered + "(" + j + ")" + ext))
                                        j++;
                                    Neww = fol + "\\" + nam.NameEntered + "(" + j + ")" + ext;
                                    File.Copy(Orgenal, Neww);
                                    inf = new FileInfo(Orgenal);
                                    inf.IsReadOnly = false;
                                    File.Delete(Orgenal);
                                    IT.ROM.Covers[i] = Neww;
                                }
                            }
                            _CoversRenamed = true;
                            //sounds
                            for (int i = 0; i < IT.ROM.SoundRecords.Count; i++)
                            {
                                if (File.Exists(Utilities.GetFullPath(IT.ROM.SoundRecords[i])))
                                {
                                    nn = Path.GetFileNameWithoutExtension(IT.ROM.SoundRecords[i]);
                                    ext = Path.GetExtension(IT.ROM.SoundRecords[i]);
                                    fol = Path.GetDirectoryName(Utilities.GetFullPath(IT.ROM.SoundRecords[i]));
                                    if (fol == "")
                                        fol = Path.GetPathRoot(Utilities.GetFullPath(IT.ROM.SoundRecords[i]));

                                    Orgenal = Utilities.GetFullPath(IT.ROM.SoundRecords[i]);
                                    int j = 1;
                                    while (File.Exists(fol + "\\" + nam.NameEntered + "(" + j + ")" + ext))
                                        j++;
                                    Neww = fol + "\\" + nam.NameEntered + "(" + j + ")" + ext;
                                    File.Copy(Orgenal, Neww);
                                    inf = new FileInfo(Orgenal);
                                    inf.IsReadOnly = false;
                                    File.Delete(Orgenal);
                                    IT.ROM.SoundRecords[i] = Neww;
                                }
                            }
                            _SoundRenamed = true;
                            //Manual
                            if (File.Exists(Utilities.GetFullPath(IT.ROM.PDFPath)))
                            {
                                nn = Path.GetFileNameWithoutExtension(IT.ROM.PDFPath);
                                ext = Path.GetExtension(IT.ROM.PDFPath);
                                fol = Path.GetDirectoryName(Utilities.GetFullPath(IT.ROM.PDFPath));
                                if (fol == "")
                                    fol = Path.GetPathRoot(Utilities.GetFullPath(IT.ROM.PDFPath));

                                Orgenal = Utilities.GetFullPath(IT.ROM.PDFPath);
                                int o = 1;
                                while (File.Exists(fol + "\\" + nam.NameEntered + "(" + o + ")" + ext))
                                    o++;
                                Neww = fol + "\\" + nam.NameEntered + "(" + o + ")" + ext;
                                File.Copy(Orgenal, Neww);
                                inf = new FileInfo(Orgenal);
                                inf.IsReadOnly = false;
                                File.Delete(Orgenal);
                                IT.ROM.PDFPath = Neww;
                            }
                            _ManualRenamed = true;
                            //video
                            /*if (File.Exists(Utilities.GetFullPath(IT.ROM.VideoPath)))
                            {
                                nn = Path.GetFileNameWithoutExtension(IT.ROM.VideoPath);
                                ext = Path.GetExtension(IT.ROM.VideoPath);
                                fol = Path.GetDirectoryName(Utilities.GetFullPath(IT.ROM.VideoPath));
                                if (fol == "")
                                    fol = Path.GetPathRoot(Utilities.GetFullPath(IT.ROM.VideoPath));

                                Orgenal = Utilities.GetFullPath(IT.ROM.VideoPath);
                                int o = 1;
                                while (File.Exists(fol + "\\" + nam.NameEntered + "(" + o + ")" + ext))
                                    o++;
                                Neww = fol + "\\" + nam.NameEntered + "(" + o + ")" + ext;
                                File.Copy(Orgenal, Neww);
                                inf = new FileInfo(Orgenal);
                                inf.IsReadOnly = false;
                                File.Delete(Orgenal);
                                IT.ROM.VideoPath = Neww;
                            }*/
                            for (int i = 0; i < IT.ROM.Videos.Count; i++)
                            {
                                if (File.Exists(Utilities.GetFullPath(IT.ROM.Videos[i])))
                                {
                                    nn = Path.GetFileNameWithoutExtension(IT.ROM.Videos[i]);
                                    ext = Path.GetExtension(IT.ROM.Videos[i]);
                                    fol = Path.GetDirectoryName(Utilities.GetFullPath(IT.ROM.Videos[i]));
                                    if (fol == "")
                                        fol = Path.GetPathRoot(Utilities.GetFullPath(IT.ROM.Videos[i]));

                                    Orgenal = Utilities.GetFullPath(IT.ROM.Videos[i]);
                                    int j = 1;
                                    while (File.Exists(fol + "\\" + nam.NameEntered + "(" + j + ")" + ext))
                                        j++;
                                    Neww = fol + "\\" + nam.NameEntered + "(" + j + ")" + ext;
                                    File.Copy(Orgenal, Neww);
                                    inf = new FileInfo(Orgenal);
                                    inf.IsReadOnly = false;
                                    File.Delete(Orgenal);
                                    IT.ROM.Videos[i] = Neww;
                                }
                            }
                            _VideoRenamed = true;
                            //Info text file
                            string pat = Utilities.GetFullPath(TR.Console.InfoFolder) + "\\" + Path.GetFileNameWithoutExtension(IT.ROM.Path) + ".txt";
                            if (File.Exists(pat))
                            {
                                nn = Path.GetFileNameWithoutExtension(pat);
                                ext = Path.GetExtension(pat);
                                fol = Path.GetDirectoryName(pat);
                                if (fol == "")
                                    fol = Path.GetPathRoot(pat);

                                Orgenal = Utilities.GetFullPath(pat);
                                Neww = fol + "\\" + nam.NameEntered + ext;
                                File.Copy(Orgenal, Neww);
                                inf = new FileInfo(Orgenal);
                                inf.IsReadOnly = false;
                                File.Delete(Orgenal);
                            }
                            _InfoRenamed = true;
                            //History
                            pat = Utilities.GetFullPath(TR.Console.HistoryFolder) + "\\" + Path.GetFileNameWithoutExtension(IT.ROM.Path) + ".txt";
                            if (File.Exists(pat))
                            {
                                nn = Path.GetFileNameWithoutExtension(pat);
                                ext = Path.GetExtension(pat);
                                fol = Path.GetDirectoryName(pat);
                                if (fol == "")
                                    fol = Path.GetPathRoot(pat);

                                Orgenal = Utilities.GetFullPath(pat);
                                Neww = fol + "\\" + nam.NameEntered + ext;
                                File.Copy(Orgenal, Neww);
                                inf = new FileInfo(Orgenal);
                                inf.IsReadOnly = false;
                                File.Delete(Orgenal);
                            }
                            _HistoryRenamed = true;
                        }
                        Save = true;
                        listView_roms_SelectedIndexChanged(this, null);
                    }
                    catch
                    {
                        MessageBox.Show("Can't rename this file !!\nMaybe the name you've entered is already exist in the folder where this rom is placed or 'access is denied' ....\n\n"
                            + "Rom file renamed = " + (_FileRenamed ? "Yes" : "NO")
                            + "Snapshots renamed = " + (_SnapsRenamed ? "Yes" : "NO")
                            + "Covers renamed = " + (_CoversRenamed ? "Yes" : "NO")
                            + "Manual renamed = " + (_ManualRenamed ? "Yes" : "NO")
                            + "Video renamed = " + (_VideoRenamed ? "Yes" : "NO")
                            + "sound files renamed = " + (_SoundRenamed ? "Yes" : "NO")
                            + "Info text file renamed = " + (_InfoRenamed ? "Yes" : "NO")
                            + "History text file renamed = " + (_HistoryRenamed ? "Yes" : "NO"));
                    }
                }
            }
            else
            {
                ListViewItem_PlayListItem IT = (ListViewItem_PlayListItem)listView_roms.SelectedItems[0];
                string nn = Path.GetFileNameWithoutExtension(IT.PlayListItem.Rom.Path);
                string ext = Path.GetExtension(IT.PlayListItem.Rom.Path);
                if (!File.Exists(Utilities.GetFullPath(IT.PlayListItem.Rom.Path)))
                {
                    MessageBox.Show("You can't rename this rom because of this rom file doesn't exist");
                    return;
                }
                Frm_EnterName nam = new Frm_EnterName("Enter the new rom name", nn);
                if (nam.ShowDialog(this) == System.Windows.Forms.DialogResult.OK)
                {
                    if (nam.NameEntered == nn)
                        return;

                    if (nam.NameEntered == "")
                        return;
                    bool _FileRenamed = false;
                    bool _SnapsRenamed = false;
                    bool _CoversRenamed = false;
                    bool _ManualRenamed = false;
                    bool _InfoRenamed = false;
                    bool _HistoryRenamed = false;
                    bool _VideoRenamed = false;
                    bool _SoundRenamed = false;
                    try
                    {
                        string fol = Path.GetDirectoryName(Utilities.GetFullPath(IT.PlayListItem.Rom.Path));
                        if (fol == "")
                            fol = Path.GetPathRoot(Utilities.GetFullPath(IT.PlayListItem.Rom.Path));

                        string Orgenal = Utilities.GetFullPath(IT.PlayListItem.Rom.Path);
                        string Neww = fol + "\\" + nam.NameEntered + ext;
                        File.Copy(Orgenal, Neww);
                        FileInfo inf = new FileInfo(Orgenal);
                        inf.IsReadOnly = false;
                        File.Delete(Orgenal);

                        IT.PlayListItem.Rom.Name = nam.NameEntered;
                        IT.PlayListItem.Rom.Path = Neww;
                        IT.RefreshName();
                        _FileRenamed = true;
                        if (MessageBox.Show("Do you want to rename snapshots, covers, sounds and manual of this rom too ?", "Rename rom associated files", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == System.Windows.Forms.DialogResult.Yes)
                        {
                            cntr_ImagesBrowser_covers.Rom = null;
                            cntr_ImagesBrowser_snaps.Rom = null;
                            cntr_ImagesBrowser_covers2.Rom = null;
                            cntr_ImagesBrowser_snaps2.Rom = null;
                            pdFviewer1.PDFPath = "";
                            videoPlayer1.Rom = null;
                            soundPlayer1.Rom = null;
                            romStatus1.Rom = null;
                            //Snapshots
                            for (int i = 0; i < IT.PlayListItem.Rom.Snapshots.Count; i++)
                            {
                                if (File.Exists(Utilities.GetFullPath(IT.PlayListItem.Rom.Snapshots[i])))
                                {
                                    nn = Path.GetFileNameWithoutExtension(IT.PlayListItem.Rom.Snapshots[i]);
                                    ext = Path.GetExtension(IT.PlayListItem.Rom.Snapshots[i]);
                                    fol = Path.GetDirectoryName(Utilities.GetFullPath(IT.PlayListItem.Rom.Snapshots[i]));
                                    if (fol == "")
                                        fol = Path.GetPathRoot(Utilities.GetFullPath(IT.PlayListItem.Rom.Snapshots[i]));

                                    Orgenal = Utilities.GetFullPath(IT.PlayListItem.Rom.Snapshots[i]);
                                    int j = 1;
                                    while (File.Exists(fol + "\\" + nam.NameEntered + "(" + j + ")" + ext))
                                        j++;
                                    Neww = fol + "\\" + nam.NameEntered + "(" + j + ")" + ext;
                                    File.Copy(Orgenal, Neww);
                                    inf = new FileInfo(Orgenal);
                                    inf.IsReadOnly = false;
                                    File.Delete(Orgenal);
                                    IT.PlayListItem.Rom.Snapshots[i] = Neww;
                                }
                                else
                                {
                                }
                            }
                            _SnapsRenamed = true;
                            //Covers
                            for (int i = 0; i < IT.PlayListItem.Rom.Covers.Count; i++)
                            {
                                if (File.Exists(Utilities.GetFullPath(IT.PlayListItem.Rom.Covers[i])))
                                {
                                    nn = Path.GetFileNameWithoutExtension(IT.PlayListItem.Rom.Covers[i]);
                                    ext = Path.GetExtension(IT.PlayListItem.Rom.Covers[i]);
                                    fol = Path.GetDirectoryName(Utilities.GetFullPath(IT.PlayListItem.Rom.Covers[i]));
                                    if (fol == "")
                                        fol = Path.GetPathRoot(Utilities.GetFullPath(IT.PlayListItem.Rom.Covers[i]));

                                    Orgenal = Utilities.GetFullPath(IT.PlayListItem.Rom.Covers[i]);
                                    int j = 1;
                                    while (File.Exists(fol + "\\" + nam.NameEntered + "(" + j + ")" + ext))
                                        j++;
                                    Neww = fol + "\\" + nam.NameEntered + "(" + j + ")" + ext;
                                    File.Copy(Orgenal, Neww);
                                    inf = new FileInfo(Orgenal);
                                    inf.IsReadOnly = false;
                                    File.Delete(Orgenal);
                                    IT.PlayListItem.Rom.Covers[i] = Neww;
                                }
                            }
                            _CoversRenamed = true;
                            //Sounds
                            for (int i = 0; i < IT.PlayListItem.Rom.SoundRecords.Count; i++)
                            {
                                if (File.Exists(Utilities.GetFullPath(IT.PlayListItem.Rom.SoundRecords[i])))
                                {
                                    nn = Path.GetFileNameWithoutExtension(IT.PlayListItem.Rom.SoundRecords[i]);
                                    ext = Path.GetExtension(IT.PlayListItem.Rom.SoundRecords[i]);
                                    fol = Path.GetDirectoryName(Utilities.GetFullPath(IT.PlayListItem.Rom.SoundRecords[i]));
                                    if (fol == "")
                                        fol = Path.GetPathRoot(Utilities.GetFullPath(IT.PlayListItem.Rom.SoundRecords[i]));

                                    Orgenal = Utilities.GetFullPath(IT.PlayListItem.Rom.SoundRecords[i]);
                                    int j = 1;
                                    while (File.Exists(fol + "\\" + nam.NameEntered + "(" + j + ")" + ext))
                                        j++;
                                    Neww = fol + "\\" + nam.NameEntered + "(" + j + ")" + ext;
                                    File.Copy(Orgenal, Neww);
                                    inf = new FileInfo(Orgenal);
                                    inf.IsReadOnly = false;
                                    File.Delete(Orgenal);
                                    IT.PlayListItem.Rom.SoundRecords[i] = Neww;
                                }
                            }
                            _SoundRenamed = true;
                            //Manual
                            if (File.Exists(Utilities.GetFullPath(IT.PlayListItem.Rom.PDFPath)))
                            {
                                nn = Path.GetFileNameWithoutExtension(IT.PlayListItem.Rom.PDFPath);
                                ext = Path.GetExtension(IT.PlayListItem.Rom.PDFPath);
                                fol = Path.GetDirectoryName(Utilities.GetFullPath(IT.PlayListItem.Rom.PDFPath));
                                if (fol == "")
                                    fol = Path.GetPathRoot(Utilities.GetFullPath(IT.PlayListItem.Rom.PDFPath));

                                Orgenal = Utilities.GetFullPath(IT.PlayListItem.Rom.PDFPath);
                                int o = 1;
                                while (File.Exists(fol + "\\" + nam.NameEntered + "(" + o + ")" + ext))
                                    o++;
                                Neww = fol + "\\" + nam.NameEntered + "(" + o + ")" + ext;
                                File.Copy(Orgenal, Neww);
                                inf = new FileInfo(Orgenal);
                                inf.IsReadOnly = false;
                                File.Delete(Orgenal);
                                IT.PlayListItem.Rom.PDFPath = Neww;
                            }
                            _ManualRenamed = true;
                            //Video
                            /*if (File.Exists(Utilities.GetFullPath(IT.PlayListItem.Rom.VideoPath)))
                            {
                                nn = Path.GetFileNameWithoutExtension(IT.PlayListItem.Rom.VideoPath);
                                ext = Path.GetExtension(IT.PlayListItem.Rom.VideoPath);
                                fol = Path.GetDirectoryName(Utilities.GetFullPath(IT.PlayListItem.Rom.VideoPath));
                                if (fol == "")
                                    fol = Path.GetPathRoot(Utilities.GetFullPath(IT.PlayListItem.Rom.VideoPath));

                                Orgenal = Utilities.GetFullPath(IT.PlayListItem.Rom.VideoPath);
                                int o = 1;
                                while (File.Exists(fol + "\\" + nam.NameEntered + "(" + o + ")" + ext))
                                    o++;
                                Neww = fol + "\\" + nam.NameEntered + "(" + o + ")" + ext;
                                File.Copy(Orgenal, Neww);
                                inf = new FileInfo(Orgenal);
                                inf.IsReadOnly = false;
                                File.Delete(Orgenal);
                                IT.PlayListItem.Rom.VideoPath = Neww;
                            }*/
                            for (int i = 0; i < IT.PlayListItem.Rom.Videos.Count; i++)
                            {
                                if (File.Exists(Utilities.GetFullPath(IT.PlayListItem.Rom.Videos[i])))
                                {
                                    nn = Path.GetFileNameWithoutExtension(IT.PlayListItem.Rom.Videos[i]);
                                    ext = Path.GetExtension(IT.PlayListItem.Rom.Videos[i]);
                                    fol = Path.GetDirectoryName(Utilities.GetFullPath(IT.PlayListItem.Rom.Videos[i]));
                                    if (fol == "")
                                        fol = Path.GetPathRoot(Utilities.GetFullPath(IT.PlayListItem.Rom.Videos[i]));

                                    Orgenal = Utilities.GetFullPath(IT.PlayListItem.Rom.Videos[i]);
                                    int j = 1;
                                    while (File.Exists(fol + "\\" + nam.NameEntered + "(" + j + ")" + ext))
                                        j++;
                                    Neww = fol + "\\" + nam.NameEntered + "(" + j + ")" + ext;
                                    File.Copy(Orgenal, Neww);
                                    inf = new FileInfo(Orgenal);
                                    inf.IsReadOnly = false;
                                    File.Delete(Orgenal);
                                    IT.PlayListItem.Rom.Videos[i] = Neww;
                                }
                            }
                            _VideoRenamed = true;
                            //Info text file
                            string pat = Utilities.GetFullPath(IT.PlayListItem.Console.InfoFolder) + "\\" + Path.GetFileNameWithoutExtension(IT.PlayListItem.Rom.Path) + ".txt";
                            if (File.Exists(pat))
                            {
                                nn = Path.GetFileNameWithoutExtension(pat);
                                ext = Path.GetExtension(pat);
                                fol = Path.GetDirectoryName(pat);
                                if (fol == "")
                                    fol = Path.GetPathRoot(pat);

                                Orgenal = Utilities.GetFullPath(pat);
                                Neww = fol + "\\" + nam.NameEntered + ext;
                                File.Copy(Orgenal, Neww);
                                inf = new FileInfo(Orgenal);
                                inf.IsReadOnly = false;
                                File.Delete(Orgenal);
                            }
                            _InfoRenamed = true;
                            //History
                            pat = Utilities.GetFullPath(IT.PlayListItem.Console.HistoryFolder) + "\\" + Path.GetFileNameWithoutExtension(IT.PlayListItem.Rom.Path) + ".txt";
                            if (File.Exists(pat))
                            {
                                nn = Path.GetFileNameWithoutExtension(pat);
                                ext = Path.GetExtension(pat);
                                fol = Path.GetDirectoryName(pat);
                                if (fol == "")
                                    fol = Path.GetPathRoot(pat);

                                Orgenal = Utilities.GetFullPath(pat);
                                Neww = fol + "\\" + nam.NameEntered + ext;
                                File.Copy(Orgenal, Neww);
                                inf = new FileInfo(Orgenal);
                                inf.IsReadOnly = false;
                                File.Delete(Orgenal);
                            }
                            _HistoryRenamed = true;
                        }
                        Save = true;
                        listView_roms_SelectedIndexChanged(this, null);
                    }
                    catch
                    {
                        MessageBox.Show("Can't rename this file !!\nMaybe the name you've entered is already exist in the folder where this rom is placed or 'access is denied' ....\n\n"
                          + "Rom file renamed = " + (_FileRenamed ? "Yes" : "NO")
                          + "Snapshots renamed = " + (_SnapsRenamed ? "Yes" : "NO")
                          + "Covers renamed = " + (_CoversRenamed ? "Yes" : "NO")
                          + "Manual renamed = " + (_ManualRenamed ? "Yes" : "NO")
                          + "Video renamed = " + (_VideoRenamed ? "Yes" : "NO")
                          + "Sound files renamed = " + (_SoundRenamed ? "Yes" : "NO")
                          + "Info text file renamed = " + (_InfoRenamed ? "Yes" : "NO")
                          + "History text file renamed = " + (_HistoryRenamed ? "Yes" : "NO"));
                    }
                }
            }
        }
        private void selectallToolStripMenuItem_Click(object sender, EventArgs e)
        {
            for (int i = 0; i < listView_roms.Items.Count; i++)
            {
                listView_roms.Items[i].Selected = true;
            }
        }
        private void clearMissedRomsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (listView_roms.Items.Count == 0)
                return;
            if (tabControl3.SelectedIndex != 0)
            {
                MessageBox.Show("Please select console !!");
                return;
            }
            if (treeView_consoles.SelectedNode == null)
            {
                MessageBox.Show("Please select console !!");
                return;
            }
            if (treeView_consoles.SelectedNode.GetType() != typeof(TreeNode_Console))
            {
                MessageBox.Show("You can only clear a console");
                return;
            }
            int removed = 0;
            DialogResult mess = MessageBox.Show("Are you sure ?", "Clear missed roms", MessageBoxButtons.YesNo, MessageBoxIcon.Warning);
            if (mess == DialogResult.Yes)
            {
                TreeNode_Console TR = (TreeNode_Console)treeView_consoles.SelectedNode;
                for (int i = 0; i < listView_roms.Items.Count; i++)
                {
                    ListViewItem_Rom ro = (ListViewItem_Rom)listView_roms.Items[i];
                    if (!File.Exists(Utilities.GetFullPath(ro.ROM.Path)))
                    {
                        TR.Console.Roms.Remove(ro.ROM);
                        listView_roms.Items.Remove(listView_roms.Items[i]);
                        i = -1;
                        removed++;
                    }
                }
                Save = true;
                if (removed > 0)
                    StatusLabel.Text = "Done, " + removed.ToString() + " roms are missed and removed from the list";
                else
                    StatusLabel.Text = "No missed rom detected";
            }
        }
        private void clearRomsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (listView_roms.Items.Count == 0)
                return;
            if (tabControl3.SelectedIndex != 0)
            {
                MessageBox.Show("Please select console !!");
                return;
            }
            if (treeView_consoles.SelectedNode == null)
            {
                MessageBox.Show("Please select console !!");
                return;
            }
            if (treeView_consoles.SelectedNode.GetType() != typeof(TreeNode_Console))
            {
                MessageBox.Show("You can only clear a console");
                return;
            }
            DialogResult mess = MessageBox.Show("Are you sure ?", "Clear all the roms", MessageBoxButtons.YesNo, MessageBoxIcon.Warning);
            if (mess == DialogResult.Yes)
            {
                TreeNode_Console TR = (TreeNode_Console)treeView_consoles.SelectedNode;
                TR.Console.Roms.Clear();
                listView_roms.Items.Clear();

                Save = true;
                StatusLabel.Text = "Done";
            }
        }
        //Copy paste delete
        private void copyToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (_ActiveElement == ActiveElement.Info)
                infoEditor1_info.CopyToClipBoard();
            else if (_ActiveElement == ActiveElement.History)
                infoEditor1_history.CopyToClipBoard();
            else if (_ActiveElement == ActiveElement.RomOrPlaylistItem)
            {
                if (tabControl3.SelectedIndex != 1)
                {
                    MessageBox.Show("Please select playlist !!");
                    return;
                }
                if (treeView_playlists.SelectedNode == null)
                    return;
                if (listView_roms.SelectedItems.Count == 0)
                    return;
                _CopiedPlaylistItems.Clear();
                foreach (ListViewItem_PlayListItem IT in listView_roms.SelectedItems)
                {
                    PlayListItem ittt = new PlayListItem();
                    ittt.Console = IT.PlayListItem.Console;
                    ittt.Name = IT.PlayListItem.Name;
                    ittt.Rom = IT.PlayListItem.Rom;
                    _CopiedPlaylistItems.Add(ittt);
                }
                StatusLabel.Text = _CopiedPlaylistItems.Count.ToString() + " rom(s) copied.";
            }
            else
            {
                //richTextBox1.Copy();
            }
        }
        private void cutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (_ActiveElement == ActiveElement.Info)
                infoEditor1_info.CutToClipBoard();
            else if (_ActiveElement == ActiveElement.History)
                infoEditor1_history.CutToClipBoard();
            else if (_ActiveElement == ActiveElement.RomOrPlaylistItem)
            {
                if (tabControl3.SelectedIndex != 1)
                {
                    MessageBox.Show("Please select playlist !!");
                    return;
                }
                if (treeView_playlists.SelectedNode == null)
                    return;
                if (listView_roms.SelectedItems.Count == 0)
                    return;
                if (treeView_playlists.SelectedNode.GetType() == typeof(TreeNode_PlayList))
                {
                    //Copy
                    _CopiedPlaylistItems.Clear();
                    foreach (ListViewItem_PlayListItem IT in listView_roms.SelectedItems)
                    {
                        PlayListItem ittt = new PlayListItem();
                        ittt.Console = IT.PlayListItem.Console;
                        ittt.Name = IT.PlayListItem.Name;
                        ittt.Rom = IT.PlayListItem.Rom;
                        _CopiedPlaylistItems.Add(ittt);
                    }
                    StatusLabel.Text = _CopiedPlaylistItems.Count.ToString() + " rom(s) cut.";
                    //Delete
                    TreeNode_PlayList TR = (TreeNode_PlayList)treeView_playlists.SelectedNode;
                    for (int i = 0; i < listView_roms.SelectedItems.Count; i++)
                    {
                        ListViewItem_PlayListItem ro = (ListViewItem_PlayListItem)listView_roms.SelectedItems[i];
                        TR.PlayList.PlayListItems.Remove(ro.PlayListItem);
                        listView_roms.Items.Remove(listView_roms.SelectedItems[i]);
                        i = -1;
                    }
                    Save = true;
                }
            }
            else
            {
                // richTextBox1.Cut();
            }
        }
        private void pasteBeforeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (tabControl3.SelectedIndex != 1)
                return;
            if (treeView_playlists.SelectedNode == null)
                return;
            if (_CopiedPlaylistItems.Count == 0)
                return;
            if (listView_roms.SelectedItems.Count != 1)
            {
                MessageBox.Show("You must select one rom to paste before !!");
                return;
            }
            ListViewItem_PlayListItem SelectedItem = (ListViewItem_PlayListItem)listView_roms.SelectedItems[0];

            TreeNode_PlayList TR = (TreeNode_PlayList)treeView_playlists.SelectedNode;
            foreach (PlayListItem IT in _CopiedPlaylistItems)
            {
                bool found = false;
                foreach (PlayListItem ITii in TR.PlayList.PlayListItems)
                {
                    if (IT.Name == ITii.Name & IT.Console == ITii.Console & IT.Rom == ITii.Rom)
                    { found = true; break; }
                }
                if (!found)
                {
                    TR.PlayList.PlayListItems.Insert(SelectedItem.Index, IT);
                    ListViewItem_PlayListItem pl = new ListViewItem_PlayListItem();
                    pl.PlayListItem = IT;
                    pl.RefreshName();
                    pl.Selected = true;
                    listView_roms.Items.Insert(SelectedItem.Index, pl);
                }
                SelectedItem.Selected = false;
                Save = true;
            }
        }
        private void pasteToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (_ActiveElement == ActiveElement.Info)
                infoEditor1_info.PasteFromClipBoard();
            else if (_ActiveElement == ActiveElement.History)
                infoEditor1_history.PasteFromClipBoard();
            else
            {
                if (tabControl3.SelectedIndex != 1)
                    return;
                if (treeView_playlists.SelectedNode == null)
                    return;
                if (_CopiedPlaylistItems.Count == 0)
                    return;
                TreeNode_PlayList TR = (TreeNode_PlayList)treeView_playlists.SelectedNode;
                if (listView_roms.SelectedItems.Count >= 1)
                {
                    ListViewItem_PlayListItem SelectedItem = (ListViewItem_PlayListItem)listView_roms.SelectedItems[listView_roms.SelectedItems.Count - 1];

                    foreach (PlayListItem IT in _CopiedPlaylistItems)
                    {
                        bool found = false;
                        foreach (PlayListItem ITii in TR.PlayList.PlayListItems)
                        {
                            if (IT.Name == ITii.Name & IT.Console == ITii.Console & IT.Rom == ITii.Rom)
                            { found = true; break; }
                        }
                        if (!found)
                        {
                            TR.PlayList.PlayListItems.Insert(SelectedItem.Index, IT);
                            ListViewItem_PlayListItem pl = new ListViewItem_PlayListItem();
                            pl.PlayListItem = IT;
                            pl.RefreshName();
                            pl.Selected = true;
                            listView_roms.Items.Insert(SelectedItem.Index + 1, pl);
                        }
                        SelectedItem.Selected = false;
                    }
                    Save = true;
                }
                else
                {
                    foreach (PlayListItem IT in _CopiedPlaylistItems)
                    {
                        bool found = false;
                        foreach (PlayListItem ITii in TR.PlayList.PlayListItems)
                        {
                            if (IT.Name == ITii.Name & IT.Console == ITii.Console & IT.Rom == ITii.Rom)
                            { found = true; break; }
                        }
                        if (!found)
                        {
                            TR.PlayList.PlayListItems.Add(IT);
                            ListViewItem_PlayListItem pl = new ListViewItem_PlayListItem();
                            pl.PlayListItem = IT;
                            pl.RefreshName();
                            pl.Selected = true;
                            listView_roms.Items.Add(pl);
                        }
                    }
                    Save = true;
                }
            }
        }
        private void compressToolToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Frm_CompressTool compress = new Frm_CompressTool();
            compress.ShowDialog(this);
        }
        private void buildISOToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (Save)
            {
                if (MessageBox.Show("You must save profile first, save it ?", "Build ISO", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == System.Windows.Forms.DialogResult.Yes)
                {
                    if (File.Exists(Program.ProjectPath))
                    {
                        if (Program.SaveProject(Program.ProjectPath))
                            Save = false;
                    }
                    else
                    {
                        SaveFileDialog Sav = new SaveFileDialog();
                        Sav.Title = "Save profile";
                        Sav.Filter = "Emulators Organizer Profile file (*.eop)|*.eop";
                        if (Sav.ShowDialog(this) == DialogResult.OK)
                        {
                            if (Program.SaveProject(Sav.FileName))
                                Save = false;
                        }
                        else
                            return;
                    }
                }
                else
                {
                    return;
                }
            }
            Frm_BuildIso iso = new Frm_BuildIso(Program.Project);
            iso.ShowDialog(this);
        }
        private void sendToToolStripMenuItem2_Click(object sender, EventArgs e)
        {
            if (listView_roms.SelectedItems.Count == 0)
                return;
            List<Rom> roms = new List<Rom>();
            List<EmuConsole> consoles = new List<EmuConsole>();
            if (tabControl3.SelectedIndex == 0)
            {
                if (treeView_consoles.SelectedNode == null)
                    return;
                EmuConsole cons = treeView_consoles.SelectedNode.GetType() == typeof(TreeNode_Console) ?
                    ((TreeNode_Console)treeView_consoles.SelectedNode).Console :
                    ((TreeNode_Console)treeView_consoles.SelectedNode.Parent).Console;
                foreach (ListViewItem_Rom ro in listView_roms.SelectedItems)
                {
                    roms.Add(ro.ROM);
                    consoles.Add(cons);
                }
            }
            else
            {
                if (treeView_playlists.SelectedNode == null)
                    return;
                foreach (ListViewItem_PlayListItem pl in listView_roms.SelectedItems)
                {
                    roms.Add(pl.PlayListItem.Rom);
                    consoles.Add(pl.PlayListItem.Console);
                }
            }
            Frm_SendTo sen = new Frm_SendTo(roms.ToArray(), consoles.ToArray());
            sen.ShowDialog(this);
            if (sen.DeleteRoms)
            {
                if (tabControl3.SelectedIndex == 0)//Console or category
                {
                    if (treeView_consoles.SelectedNode == null)
                        return;
                    if (treeView_consoles.SelectedNode.GetType() == typeof(TreeNode_Console))
                    {
                        if (listView_roms.SelectedItems.Count > 0)
                        {
                            for (int i = 0; i < listView_roms.SelectedItems.Count; i++)
                            {
                                //1 remove the rom from the playlists if found
                                foreach (PlayList playlist in Program.Project.PlayLists)
                                {
                                    foreach (PlayListItem item in playlist.PlayListItems)
                                    {
                                        if (item.Rom.Path == ((ListViewItem_Rom)listView_roms.SelectedItems[i]).ROM.Path)
                                        {
                                            playlist.PlayListItems.Remove(item);
                                            break;
                                        }
                                    }
                                }
                                //2 remove the rom ...
                                ((TreeNode_Console)treeView_consoles.SelectedNode).Console.Roms.Remove(((ListViewItem_Rom)listView_roms.SelectedItems[i]).ROM);
                                listView_roms.Items.Remove(listView_roms.SelectedItems[i]);
                                i = -1;
                            }
                            Save = true;
                        }
                    }
                    else//category
                    {
                        if (listView_roms.SelectedItems.Count > 0)
                        {
                            for (int i = 0; i < listView_roms.SelectedItems.Count; i++)
                            {
                                //((TreeNode_Console)treeView_consoles.SelectedNode).Console.Roms.Remove(((ListViewItem_Rom)listView_roms.SelectedItems[i]).ROM);
                                //listView_roms.Items.Remove(listView_roms.SelectedItems[i]);
                                ((ListViewItem_Rom)listView_roms.SelectedItems[i]).ROM.Categories.Remove(((TreeNode_Category)treeView_consoles.SelectedNode).CategoryIn.Name);
                                listView_roms.Items.Remove(listView_roms.SelectedItems[i]);
                                i = -1;
                            }
                            Save = true;
                        }
                    }
                }
                else//playlist
                {
                    if (treeView_consoles.SelectedNode == null)
                        return;
                    if (listView_roms.SelectedItems.Count > 0)
                    {
                        for (int i = 0; i < listView_roms.SelectedItems.Count; i++)
                        {
                            ((TreeNode_PlayList)treeView_playlists.SelectedNode).PlayList.PlayListItems.Remove(((ListViewItem_PlayListItem)listView_roms.SelectedItems[i]).PlayListItem);
                            listView_roms.Items.Remove(listView_roms.SelectedItems[i]);
                            i = -1;
                        }
                        Save = true;
                    }
                }
            }
        }
        private void applyNameToFilesToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (listView_roms.Items.Count == 0)
            {
                MessageBox.Show("There's no rom to apply name for !!?", "Apply name to all", MessageBoxButtons.OK, MessageBoxIcon.Hand);
                return;
            }
            if (tabControl3.SelectedIndex == 0)
            {
                if (MessageBox.Show("This will rename the REAL ROM FILES and can't be undone, Are you sure you want to do this ?", "Apply name to all", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No)
                    return;
                bool RenameSnapsCovers = false;
                if (MessageBox.Show("Do you want to rename snapshots, covers, sounds and manual of selected rom(s) too ?", "Rename rom associated files", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == System.Windows.Forms.DialogResult.Yes)
                {
                    RenameSnapsCovers = true;
                    cntr_ImagesBrowser_covers.Rom = null;
                    cntr_ImagesBrowser_snaps.Rom = null;
                    cntr_ImagesBrowser_covers2.Rom = null;
                    cntr_ImagesBrowser_snaps2.Rom = null;
                    pdFviewer1.PDFPath = "";
                    videoPlayer1.Rom = null;
                    soundPlayer1.Rom = null;
                    romStatus1.Rom = null;
                }
                int Unrenamed = 0;
                foreach (ListViewItem_Rom ro in listView_roms.Items)
                {
                    try
                    {
                        string nn = ro.ROM.Name;
                        string ext = Path.GetExtension(ro.ROM.Path);
                        string fol = Path.GetDirectoryName(ro.ROM.Path);
                        if (fol == "")
                            fol = Path.GetPathRoot(ro.ROM.Path);
                        string Orgenal = ro.ROM.Path;
                        string Neww = fol + "\\" + ro.ROM.Name + ext;
                        if (Orgenal == Neww)
                            continue;
                        File.Copy(Orgenal, Neww);
                        FileInfo inf = new FileInfo(Orgenal);
                        inf.IsReadOnly = false;
                        File.Delete(Orgenal);
                        ro.ROM.Path = Neww;
                        ro.RefreshName();
                        ro.BackColor = Color.White;
                        if (RenameSnapsCovers)
                        {
                            //Snapshots
                            for (int i = 0; i < ro.ROM.Snapshots.Count; i++)
                            {
                                if (File.Exists(Utilities.GetFullPath(ro.ROM.Snapshots[i])))
                                {
                                    nn = Path.GetFileNameWithoutExtension(ro.ROM.Snapshots[i]);
                                    ext = Path.GetExtension(ro.ROM.Snapshots[i]);
                                    fol = Path.GetDirectoryName(Utilities.GetFullPath(ro.ROM.Snapshots[i]));
                                    if (fol == "")
                                        fol = Path.GetPathRoot(Utilities.GetFullPath(ro.ROM.Snapshots[i]));

                                    Orgenal = Utilities.GetFullPath(ro.ROM.Snapshots[i]);
                                    int j = 1;
                                    while (File.Exists(fol + "\\" + ro.ROM.Name + "(" + j + ")" + ext))
                                        j++;
                                    Neww = fol + "\\" + ro.ROM.Name + "(" + j + ")" + ext;
                                    File.Copy(Orgenal, Neww);
                                    inf = new FileInfo(Orgenal);
                                    inf.IsReadOnly = false;
                                    File.Delete(Orgenal);
                                    ro.ROM.Snapshots[i] = Neww;
                                }
                                else
                                {
                                }
                            }
                            //Covers
                            for (int i = 0; i < ro.ROM.Covers.Count; i++)
                            {
                                if (File.Exists(Utilities.GetFullPath(ro.ROM.Covers[i])))
                                {
                                    nn = Path.GetFileNameWithoutExtension(ro.ROM.Covers[i]);
                                    ext = Path.GetExtension(ro.ROM.Covers[i]);
                                    fol = Path.GetDirectoryName(Utilities.GetFullPath(ro.ROM.Covers[i]));
                                    if (fol == "")
                                        fol = Path.GetPathRoot(Utilities.GetFullPath(ro.ROM.Covers[i]));

                                    Orgenal = Utilities.GetFullPath(ro.ROM.Covers[i]);
                                    int j = 1;
                                    while (File.Exists(fol + "\\" + ro.ROM.Name + "(" + j + ")" + ext))
                                        j++;
                                    Neww = fol + "\\" + ro.ROM.Name + "(" + j + ")" + ext;
                                    File.Copy(Orgenal, Neww);
                                    inf = new FileInfo(Orgenal);
                                    inf.IsReadOnly = false;
                                    File.Delete(Orgenal);
                                    ro.ROM.Covers[i] = Neww;
                                }
                            }
                            //Sound
                            for (int i = 0; i < ro.ROM.SoundRecords.Count; i++)
                            {
                                if (File.Exists(Utilities.GetFullPath(ro.ROM.SoundRecords[i])))
                                {
                                    nn = Path.GetFileNameWithoutExtension(ro.ROM.SoundRecords[i]);
                                    ext = Path.GetExtension(ro.ROM.SoundRecords[i]);
                                    fol = Path.GetDirectoryName(Utilities.GetFullPath(ro.ROM.SoundRecords[i]));
                                    if (fol == "")
                                        fol = Path.GetPathRoot(Utilities.GetFullPath(ro.ROM.SoundRecords[i]));

                                    Orgenal = Utilities.GetFullPath(ro.ROM.SoundRecords[i]);
                                    int j = 1;
                                    while (File.Exists(fol + "\\" + ro.ROM.Name + "(" + j + ")" + ext))
                                        j++;
                                    Neww = fol + "\\" + ro.ROM.Name + "(" + j + ")" + ext;
                                    File.Copy(Orgenal, Neww);
                                    inf = new FileInfo(Orgenal);
                                    inf.IsReadOnly = false;
                                    File.Delete(Orgenal);
                                    ro.ROM.SoundRecords[i] = Neww;
                                }
                            }
                            //Manual
                            if (File.Exists(Utilities.GetFullPath(ro.ROM.PDFPath)))
                            {
                                nn = Path.GetFileNameWithoutExtension(ro.ROM.PDFPath);
                                ext = Path.GetExtension(ro.ROM.PDFPath);
                                fol = Path.GetDirectoryName(Utilities.GetFullPath(ro.ROM.PDFPath));
                                if (fol == "")
                                    fol = Path.GetPathRoot(Utilities.GetFullPath(ro.ROM.PDFPath));

                                Orgenal = Utilities.GetFullPath(ro.ROM.PDFPath);
                                int o = 1;
                                while (File.Exists(fol + "\\" + ro.ROM.Name + "(" + o + ")" + ext))
                                    o++;
                                Neww = fol + "\\" + ro.ROM.Name + "(" + o + ")" + ext;
                                File.Copy(Orgenal, Neww);
                                inf = new FileInfo(Orgenal);
                                inf.IsReadOnly = false;
                                File.Delete(Orgenal);
                                ro.ROM.PDFPath = Neww;
                            }
                            //Video
                            for (int i = 0; i < ro.ROM.Videos.Count; i++)
                            {
                                if (File.Exists(Utilities.GetFullPath(ro.ROM.Videos[i])))
                                {
                                    nn = Path.GetFileNameWithoutExtension(ro.ROM.Videos[i]);
                                    ext = Path.GetExtension(ro.ROM.Videos[i]);
                                    fol = Path.GetDirectoryName(Utilities.GetFullPath(ro.ROM.Videos[i]));
                                    if (fol == "")
                                        fol = Path.GetPathRoot(Utilities.GetFullPath(ro.ROM.Videos[i]));

                                    Orgenal = Utilities.GetFullPath(ro.ROM.Videos[i]);
                                    int j = 1;
                                    while (File.Exists(fol + "\\" + ro.ROM.Name + "(" + j + ")" + ext))
                                        j++;
                                    Neww = fol + "\\" + ro.ROM.Name + "(" + j + ")" + ext;
                                    File.Copy(Orgenal, Neww);
                                    inf = new FileInfo(Orgenal);
                                    inf.IsReadOnly = false;
                                    File.Delete(Orgenal);
                                    ro.ROM.Videos[i] = Neww;
                                }
                            }
                        }
                    }
                    catch
                    {
                        Unrenamed++;
                        ro.BackColor = Color.Red;
                    }
                }
                Save = true;
                if (Unrenamed > 0)
                    StatusLabel.Text = "Done, could not rename " + Unrenamed.ToString() + " rom(s), they colored red in the list.";
                else
                    StatusLabel.Text = "Done";
            }
            else
            {
                if (MessageBox.Show("This will rename the REAL ROM FILES and can't be undone, Are you sure you want to do this ?", "Apply name to all", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No)
                    return;
                bool RenameSnapsCovers = false;
                if (MessageBox.Show("Do you want to rename snapshots, covers, sounds and manual of selected rom(s) too ?", "Rename rom associated files", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == System.Windows.Forms.DialogResult.Yes)
                {
                    RenameSnapsCovers = true; cntr_ImagesBrowser_covers.Rom = null;
                    cntr_ImagesBrowser_snaps.Rom = null;
                    cntr_ImagesBrowser_covers2.Rom = null;
                    cntr_ImagesBrowser_snaps2.Rom = null;
                    pdFviewer1.PDFPath = ""; videoPlayer1.Rom = null; soundPlayer1.Rom = null;
                    romStatus1.Rom = null;
                }
                int Unrenamed = 0;
                foreach (ListViewItem_PlayListItem ro in listView_roms.Items)
                {
                    try
                    {
                        string nn = ro.PlayListItem.Rom.Name;
                        string ext = Path.GetExtension(ro.PlayListItem.Rom.Path);
                        string fol = Path.GetDirectoryName(ro.PlayListItem.Rom.Path);
                        if (fol == "")
                            fol = Path.GetPathRoot(ro.PlayListItem.Rom.Path);
                        string Orgenal = ro.PlayListItem.Rom.Path;
                        string Neww = fol + "\\" + ro.PlayListItem.Rom.Name + ext;
                        if (Orgenal == Neww)
                            continue;
                        File.Copy(Orgenal, Neww);
                        FileInfo inf = new FileInfo(Orgenal);
                        inf.IsReadOnly = false;
                        File.Delete(Orgenal);
                        ro.PlayListItem.Rom.Path = Neww;
                        ro.RefreshName();
                        ro.BackColor = Color.White;
                        if (RenameSnapsCovers)
                        {
                            //Snapshots
                            for (int i = 0; i < ro.PlayListItem.Rom.Snapshots.Count; i++)
                            {
                                if (File.Exists(Utilities.GetFullPath(ro.PlayListItem.Rom.Snapshots[i])))
                                {
                                    nn = Path.GetFileNameWithoutExtension(ro.PlayListItem.Rom.Snapshots[i]);
                                    ext = Path.GetExtension(ro.PlayListItem.Rom.Snapshots[i]);
                                    fol = Path.GetDirectoryName(Utilities.GetFullPath(ro.PlayListItem.Rom.Snapshots[i]));
                                    if (fol == "")
                                        fol = Path.GetPathRoot(Utilities.GetFullPath(ro.PlayListItem.Rom.Snapshots[i]));

                                    Orgenal = Utilities.GetFullPath(ro.PlayListItem.Rom.Snapshots[i]);
                                    int j = 1;
                                    while (File.Exists(fol + "\\" + ro.PlayListItem.Rom.Name + "(" + j + ")" + ext))
                                        j++;
                                    Neww = fol + "\\" + ro.PlayListItem.Rom.Name + "(" + j + ")" + ext;
                                    File.Copy(Orgenal, Neww);
                                    inf = new FileInfo(Orgenal);
                                    inf.IsReadOnly = false;
                                    File.Delete(Orgenal);
                                    ro.PlayListItem.Rom.Snapshots[i] = Neww;
                                }
                                else
                                {
                                }
                            }
                            //Covers
                            for (int i = 0; i < ro.PlayListItem.Rom.Covers.Count; i++)
                            {
                                if (File.Exists(Utilities.GetFullPath(ro.PlayListItem.Rom.Covers[i])))
                                {
                                    nn = Path.GetFileNameWithoutExtension(ro.PlayListItem.Rom.Covers[i]);
                                    ext = Path.GetExtension(ro.PlayListItem.Rom.Covers[i]);
                                    fol = Path.GetDirectoryName(Utilities.GetFullPath(ro.PlayListItem.Rom.Covers[i]));
                                    if (fol == "")
                                        fol = Path.GetPathRoot(Utilities.GetFullPath(ro.PlayListItem.Rom.Covers[i]));

                                    Orgenal = Utilities.GetFullPath(ro.PlayListItem.Rom.Covers[i]);
                                    int j = 1;
                                    while (File.Exists(fol + "\\" + ro.PlayListItem.Rom.Name + "(" + j + ")" + ext))
                                        j++;
                                    Neww = fol + "\\" + ro.PlayListItem.Rom.Name + "(" + j + ")" + ext;
                                    File.Copy(Orgenal, Neww);
                                    inf = new FileInfo(Orgenal);
                                    inf.IsReadOnly = false;
                                    File.Delete(Orgenal);
                                    ro.PlayListItem.Rom.Covers[i] = Neww;
                                }
                            }
                            //Sound
                            for (int i = 0; i < ro.PlayListItem.Rom.SoundRecords.Count; i++)
                            {
                                if (File.Exists(Utilities.GetFullPath(ro.PlayListItem.Rom.SoundRecords[i])))
                                {
                                    nn = Path.GetFileNameWithoutExtension(ro.PlayListItem.Rom.SoundRecords[i]);
                                    ext = Path.GetExtension(ro.PlayListItem.Rom.SoundRecords[i]);
                                    fol = Path.GetDirectoryName(Utilities.GetFullPath(ro.PlayListItem.Rom.SoundRecords[i]));
                                    if (fol == "")
                                        fol = Path.GetPathRoot(Utilities.GetFullPath(ro.PlayListItem.Rom.SoundRecords[i]));

                                    Orgenal = Utilities.GetFullPath(ro.PlayListItem.Rom.SoundRecords[i]);
                                    int j = 1;
                                    while (File.Exists(fol + "\\" + ro.PlayListItem.Rom.Name + "(" + j + ")" + ext))
                                        j++;
                                    Neww = fol + "\\" + ro.PlayListItem.Rom.Name + "(" + j + ")" + ext;
                                    File.Copy(Orgenal, Neww);
                                    inf = new FileInfo(Orgenal);
                                    inf.IsReadOnly = false;
                                    File.Delete(Orgenal);
                                    ro.PlayListItem.Rom.SoundRecords[i] = Neww;
                                }
                            }
                            //Manual
                            if (File.Exists(Utilities.GetFullPath(ro.PlayListItem.Rom.PDFPath)))
                            {
                                nn = Path.GetFileNameWithoutExtension(ro.PlayListItem.Rom.PDFPath);
                                ext = Path.GetExtension(ro.PlayListItem.Rom.PDFPath);
                                fol = Path.GetDirectoryName(Utilities.GetFullPath(ro.PlayListItem.Rom.PDFPath));
                                if (fol == "")
                                    fol = Path.GetPathRoot(Utilities.GetFullPath(ro.PlayListItem.Rom.PDFPath));

                                Orgenal = Utilities.GetFullPath(ro.PlayListItem.Rom.PDFPath);
                                int o = 1;
                                while (File.Exists(fol + "\\" + ro.PlayListItem.Rom.Name + "(" + o + ")" + ext))
                                    o++;
                                Neww = fol + "\\" + ro.PlayListItem.Rom.Name + "(" + o + ")" + ext;
                                File.Copy(Orgenal, Neww);
                                inf = new FileInfo(Orgenal);
                                inf.IsReadOnly = false;
                                File.Delete(Orgenal);
                                ro.PlayListItem.Rom.PDFPath = Neww;
                            }
                            //Video
                            for (int i = 0; i < ro.PlayListItem.Rom.Videos.Count; i++)
                            {
                                if (File.Exists(Utilities.GetFullPath(ro.PlayListItem.Rom.Videos[i])))
                                {
                                    nn = Path.GetFileNameWithoutExtension(ro.PlayListItem.Rom.Videos[i]);
                                    ext = Path.GetExtension(ro.PlayListItem.Rom.Videos[i]);
                                    fol = Path.GetDirectoryName(Utilities.GetFullPath(ro.PlayListItem.Rom.Videos[i]));
                                    if (fol == "")
                                        fol = Path.GetPathRoot(Utilities.GetFullPath(ro.PlayListItem.Rom.Videos[i]));

                                    Orgenal = Utilities.GetFullPath(ro.PlayListItem.Rom.Videos[i]);
                                    int j = 1;
                                    while (File.Exists(fol + "\\" + ro.PlayListItem.Rom.Name + "(" + j + ")" + ext))
                                        j++;
                                    Neww = fol + "\\" + ro.PlayListItem.Rom.Name + "(" + j + ")" + ext;
                                    File.Copy(Orgenal, Neww);
                                    inf = new FileInfo(Orgenal);
                                    inf.IsReadOnly = false;
                                    File.Delete(Orgenal);
                                    ro.PlayListItem.Rom.Videos[i] = Neww;
                                }
                            }
                        }
                    }
                    catch
                    {
                        Unrenamed++;
                        ro.BackColor = Color.Red;
                    }
                }
                Save = true;
                if (Unrenamed > 0)
                    StatusLabel.Text = "Done, could not rename " + Unrenamed.ToString() + " rom(s), they colored red in the list.";
                else
                    StatusLabel.Text = "Done";
            }
        }
        private void ComboBox1_find_Validated(object sender, EventArgs e)
        {
            foreach (string item in ComboBox1_find.Items)
            {
                if (item == ComboBox1_find.Text)
                    return;
            }
            ComboBox1_find.Items.Add(ComboBox1_find.Text);
        }
        private void nextSearchResultToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (findForm != null)
            {
                if (findForm.Visible)
                {
                    findForm.FindNext();
                }
            }
        }
        private void groupBox2_Enter(object sender, EventArgs e)
        {
            panel1_emulators.Visible = true;
        }
        private void groupBox2_Leave(object sender, EventArgs e)
        {
            panel1_emulators.Visible = false;
        }
        private void trackBar3_Scroll(object sender, EventArgs e)
        {
            imageList_consoles.ImageSize = new Size(trackBar1.Value, trackBar1.Value);
            trackBar2.Value = trackBar1.Value = trackBar3.Value;
        }
        private void button6_Click(object sender, EventArgs e)
        {
            if (listView_emulators.SelectedItems.Count != 1)
                return;
            if (tabControl3.SelectedIndex != 0)
            {
                MessageBox.Show("You must select the parent console");
                return;
            }
            if (treeView_consoles.SelectedNode == null)
                return;
            if (treeView_consoles.SelectedNode.GetType() != typeof(TreeNode_Console))
                return;

            ListViewItem_Emulator selected = (ListViewItem_Emulator)listView_emulators.SelectedItems[0];
            int index = selected.Index;

            TreeNode_Console tr = (TreeNode_Console)treeView_consoles.SelectedNode;

            tr.Console.Emulators.RemoveAt(index);

            if (index > 0)
                index--;

            tr.Console.Emulators.Insert(index, selected.Emu);

            listView_emulators.Items.Clear();
            TreeNode_Console TR = (TreeNode_Console)treeView_consoles.SelectedNode;
            int Index = 0;
            foreach (Emulator Emu in TR.Console.Emulators)
            {
                ListViewItem_Emulator em = new ListViewItem_Emulator();
                em.Emu = Emu;
                listView_emulators.Items.Add(em);
                Index++;
            }

            listView_emulators.Items[index].Selected = true;
            Save = true;
        }
        private void button5_Click(object sender, EventArgs e)
        {
            if (listView_emulators.SelectedItems.Count != 1)
                return;
            if (tabControl3.SelectedIndex != 0)
            {
                MessageBox.Show("You must select the parent console");
                return;
            }
            if (treeView_consoles.SelectedNode == null)
                return;
            if (treeView_consoles.SelectedNode.GetType() != typeof(TreeNode_Console))
                return;

            ListViewItem_Emulator selected = (ListViewItem_Emulator)listView_emulators.SelectedItems[0];
            int index = selected.Index;

            TreeNode_Console tr = (TreeNode_Console)treeView_consoles.SelectedNode;

            tr.Console.Emulators.RemoveAt(index);

            if (index < listView_emulators.Items.Count - 1)
                index++;

            tr.Console.Emulators.Insert(index, selected.Emu);

            listView_emulators.Items.Clear();
            TreeNode_Console TR = (TreeNode_Console)treeView_consoles.SelectedNode;
            int Index = 0;
            foreach (Emulator Emu in TR.Console.Emulators)
            {
                ListViewItem_Emulator em = new ListViewItem_Emulator();
                em.Emu = Emu;
                listView_emulators.Items.Add(em);
                Index++;
            }

            listView_emulators.Items[index].Selected = true;
            Save = true;
        }
        private void listView_roms_MouseMove(object sender, MouseEventArgs e)
        {
            if (e.X > 7 & e.Y <= 27)
            {
                listView_roms.ContextMenuStrip = contextMenuStrip1_columns;
            }
            else
                listView_roms.ContextMenuStrip = contextMenuStrip1_roms;
        }
        private void fileTypeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Program.UserSettings.Column_filetype_visible = fileTypeToolStripMenuItem.Checked;
            RefreshColumns();
        }
        private void playedTimeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Program.UserSettings.Column_playedtime_visible = playedTimeToolStripMenuItem.Checked;
            RefreshColumns();
        }
        private void pathToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Program.UserSettings.Column_path_visible = pathToolStripMenuItem.Checked;
            RefreshColumns();
        }
        private void ratingToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Program.UserSettings.Column_rating_visible = ratingToolStripMenuItem.Checked;
            RefreshColumns();
        }
        private void sizeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Program.UserSettings.Column_Size_Visible = sizeToolStripMenuItem.Checked;
            RefreshColumns();
        }
        private void toolStripButton12_Click(object sender, EventArgs e)
        {
            if (listView_roms.SelectedItems.Count == 1)
            {
                if (listView_roms.SelectedItems[0].GetType() == typeof(ListViewItem_Rom))
                {
                    Frm_EnterName nam = new Frm_EnterName("Enter the link", "http://");
                    if (nam.ShowDialog(this) == System.Windows.Forms.DialogResult.OK)
                    {
                        foreach (string link in ((ListViewItem_Rom)listView_roms.SelectedItems[0]).ROM.Links)
                        {
                            if (nam.NameEntered == link)
                                return;
                        }
                        ((ListViewItem_Rom)listView_roms.SelectedItems[0]).ROM.Links.Add(nam.NameEntered);
                        treeView1.Nodes.Clear();
                        foreach (string link in ((ListViewItem_Rom)listView_roms.SelectedItems[0]).ROM.Links)
                            treeView1.Nodes.Add(link);
                        Save = true;
                    }
                }
                else
                {
                    Frm_EnterName nam = new Frm_EnterName("Enter the link", "http://");
                    if (nam.ShowDialog(this) == System.Windows.Forms.DialogResult.OK)
                    {
                        foreach (string link in ((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Rom.Links)
                        {
                            if (nam.NameEntered == link)
                                return;
                        }
                        ((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Rom.Links.Add(nam.NameEntered);
                        treeView1.Nodes.Clear();
                        foreach (string link in ((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Rom.Links)
                            treeView1.Nodes.Add(link);
                        Save = true;
                    }
                }
            }
        }
        private void toolStripButton13_Click(object sender, EventArgs e)
        {
            for (int i = 0; i < treeView1.Nodes.Count; i++)
            {
                if (treeView1.Nodes[i].Checked)
                {
                    treeView1.Nodes.Remove(treeView1.Nodes[i]); i = -1;
                }
            }
            if (listView_roms.SelectedItems.Count == 1)
            {
                if (listView_roms.SelectedItems[0].GetType() == typeof(ListViewItem_Rom))
                {
                    ((ListViewItem_Rom)listView_roms.SelectedItems[0]).ROM.Links.Clear();
                    for (int i = 0; i < treeView1.Nodes.Count; i++)
                    {
                        ((ListViewItem_Rom)listView_roms.SelectedItems[0]).ROM.Links.Add(treeView1.Nodes[i].Text);
                    } Save = true;
                }
                else
                {
                    ((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Rom.Links.Clear();
                    for (int i = 0; i < treeView1.Nodes.Count; i++)
                    {
                        ((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Rom.Links.Add(treeView1.Nodes[i].Text);
                    } Save = true;
                }
            }
        }
        //After select rom...
        private void timer1_AfterSelectRoms_Tick(object sender, EventArgs e)
        {
            if (countDown > 0)
                countDown--;
            else
            {
                timer1_AfterSelectRoms.Stop();
                _ActiveElement = ActiveElement.RomOrPlaylistItem;
                toolStripButton5_UseEmulatorCommandlines.Checked = true;
                toolStripButton6_UseRomCommandlines.Checked = false;
                CheckCommandlinesPanel(true);
                if (toolStripButton6_UseRomCommandlines.Enabled)
                {
                    toolStripButton6_Click(this, null);
                }
                if (tabControl3.SelectedIndex == 1)
                    listView_emulators.Items.Clear();
                if (listView_roms.SelectedItems.Count == 1)
                {
                    if (listView_roms.SelectedItems[0].GetType() == typeof(ListViewItem_Rom))
                    {
                        //statues
                        romStatus1.Rom = ((ListViewItem_Rom)listView_roms.SelectedItems[0]).ROM;
                        //cover and snap
                        cntr_ImagesBrowser_snaps.Rom = ((ListViewItem_Rom)listView_roms.SelectedItems[0]).ROM;
                        cntr_ImagesBrowser_covers.Rom = ((ListViewItem_Rom)listView_roms.SelectedItems[0]).ROM;
                        cntr_ImagesBrowser_snaps2.Rom = ((ListViewItem_Rom)listView_roms.SelectedItems[0]).ROM;
                        cntr_ImagesBrowser_covers2.Rom = ((ListViewItem_Rom)listView_roms.SelectedItems[0]).ROM;
                        //pdf
                        if (tabControl_up_right.SelectedIndex == 3)
                            pdFviewer1.PDFPath = ((ListViewItem_Rom)listView_roms.SelectedItems[0]).ROM.PDFPath;
                        //video
                        videoPlayer1.Rom = ((ListViewItem_Rom)listView_roms.SelectedItems[0]).ROM;
                        //sound
                        soundPlayer1.Rom = ((ListViewItem_Rom)listView_roms.SelectedItems[0]).ROM;

                        //auto start
                        if (videoPlayer1.AutoStart & tabControl_down_right.SelectedIndex == 2)
                        {
                            soundPlayer1.Stop();
                        }
                        else if (soundPlayer1.AutoStart)
                        {
                            videoPlayer1.Stop();
                        }
                        //info and history
                        if (treeView_consoles.SelectedNode != null)
                        {
                            if (treeView_consoles.SelectedNode.GetType() == typeof(TreeNode_Console))
                            {
                                infoEditor1_info.InfoFolderPath = Utilities.GetFullPath(((TreeNode_Console)treeView_consoles.SelectedNode).Console.InfoFolder);
                                infoEditor1_history.HistoryFolderPath = Utilities.GetFullPath(((TreeNode_Console)treeView_consoles.SelectedNode).Console.HistoryFolder);
                            }
                            else
                            {
                                infoEditor1_info.InfoFolderPath = Utilities.GetFullPath(((TreeNode_Console)treeView_consoles.SelectedNode.Parent).Console.InfoFolder);
                                infoEditor1_history.HistoryFolderPath = Utilities.GetFullPath(((TreeNode_Console)treeView_consoles.SelectedNode.Parent).Console.HistoryFolder);
                            }
                            infoEditor1_info.RomName = Path.GetFileNameWithoutExtension(((ListViewItem_Rom)listView_roms.SelectedItems[0]).ROM.Path);
                            infoEditor1_history.RomName = Path.GetFileNameWithoutExtension(((ListViewItem_Rom)listView_roms.SelectedItems[0]).ROM.Path);
                        }
                        else
                        {
                            infoEditor1_info.RomName = infoEditor1_history.RomName = null;
                        }
                        //links
                        if (((ListViewItem_Rom)listView_roms.SelectedItems[0]).ROM.Links == null)
                            ((ListViewItem_Rom)listView_roms.SelectedItems[0]).ROM.Links = new List<string>();
                        treeView1.Nodes.Clear();
                        foreach (string link in ((ListViewItem_Rom)listView_roms.SelectedItems[0]).ROM.Links)
                            treeView1.Nodes.Add(link);
                    }
                    else
                    {
                        romStatus1.Rom = ((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Rom;
                        cntr_ImagesBrowser_snaps.Rom = ((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Rom;
                        cntr_ImagesBrowser_covers.Rom = ((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Rom;
                        cntr_ImagesBrowser_snaps2.Rom = ((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Rom;
                        cntr_ImagesBrowser_covers2.Rom = ((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Rom;
                        //pdf
                        if (tabControl_up_right.SelectedIndex == 3)
                            pdFviewer1.PDFPath = ((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Rom.PDFPath;
                        //video
                        videoPlayer1.Rom = ((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Rom;
                        //sound
                        soundPlayer1.Rom = ((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Rom;
                        foreach (Emulator emu in ((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Console.Emulators)
                        {
                            ListViewItem_Emulator IT = new ListViewItem_Emulator();
                            IT.Emu = emu;
                            listView_emulators.Items.Add(IT);
                        }
                        if (listView_emulators.Items.Count > 0)
                            listView_emulators.Items[0].Selected = true;

                        infoEditor1_info.InfoFolderPath = Utilities.GetFullPath(((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Console.InfoFolder);
                        infoEditor1_history.HistoryFolderPath = Utilities.GetFullPath(((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Console.HistoryFolder);
                        infoEditor1_info.RomName = Path.GetFileNameWithoutExtension(((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Rom.Path);
                        infoEditor1_history.RomName = Path.GetFileNameWithoutExtension(((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Rom.Path);
                        //links
                        if (((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Rom.Links == null)
                            ((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Rom.Links = new List<string>();
                        treeView1.Nodes.Clear();
                        foreach (string link in ((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Rom.Links)
                            treeView1.Nodes.Add(link);
                        _ActiveElement = ActiveElement.RomOrPlaylistItem;
                    }
                }
                else
                {
                    treeView1.Nodes.Clear();
                    pdFviewer1.PDFPath = "";
                    videoPlayer1.Rom = null;
                    soundPlayer1.Rom = null;
                    infoEditor1_info.InfoFolderPath = infoEditor1_history.HistoryFolderPath =
                    infoEditor1_info.RomName = infoEditor1_history.RomName = null;
                    romStatus1.Rom = null;
                    cntr_ImagesBrowser_snaps.Rom = null;
                    cntr_ImagesBrowser_covers.Rom = null;
                    cntr_ImagesBrowser_snaps2.Rom = null;
                    cntr_ImagesBrowser_covers2.Rom = null;
                }
                RomsStatusLabel1.Text = listView_roms.SelectedItems.Count + " / " + listView_roms.Items.Count;
            }
        }
        private void trackBar4_Scroll(object sender, EventArgs e)
        {
            toolTip1.SetToolTip(trackBar4, trackBar4.Value + "x" + trackBar4.Value);
        }
        private void comboBox_thumpType_SelectedIndexChanged(object sender, EventArgs e)
        {
            listView_roms.Refresh();
        }
        private void historyToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Program.UserSettings.Column_hasHistory_visible = historyToolStripMenuItem.Checked;
            RefreshColumns();
        }
        private void coverToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Program.UserSettings.Column_hasCover_visible = coverToolStripMenuItem.Checked;
            RefreshColumns();
        }
        private void infoToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Program.UserSettings.Column_hasInfo_visible = infoToolStripMenuItem.Checked;
            RefreshColumns();
        }
        private void snapshotToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Program.UserSettings.Column_hasSnap_visible = snapshotToolStripMenuItem.Checked;
            RefreshColumns();
        }
        private void manualToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Program.UserSettings.Column_hasManual_visible = manualToolStripMenuItem.Checked;
            RefreshColumns();
        }
        private void trackBar4_MouseUp(object sender, MouseEventArgs e)
        {
            imageList1_romsLarge.ImageSize = new Size(trackBar4.Value, trackBar4.Value);
        }
        private void listView_roms_ColumnClick(object sender, ColumnClickEventArgs e)
        {
            if (e.Column == columnHeader_name.Index)
            {
                _SortMode = SortMode.Name;
                _SortAlpha = !_SortAlpha;
                SortElements();
            }
            else if (e.Column == columnHeader_path.Index)
            {
                _SortMode = SortMode.Path;
                _SortAlpha = !_SortAlpha;
                SortElements();
            }
            else if (e.Column == columnHeader_size.Index)
            {
                _SortMode = SortMode.Size;
                _SortAlpha = !_SortAlpha;
                SortElements();
            }
            else if (e.Column == columnHeader1_fileType.Index)
            {
                _SortMode = SortMode.FileType;
                _SortAlpha = !_SortAlpha;
                SortElements();
            }
            else if (e.Column == columnHeader1_playtime.Index)
            {
                _SortMode = SortMode.PlayedTime;
                _SortAlpha = !_SortAlpha;
                SortElements();
            }
            else if (e.Column == columnHeader1_rating.Index)
            {
                _SortMode = SortMode.Rating;
                _SortAlpha = !_SortAlpha;
                SortElements();
            }
        }
        private void optionsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Frm_Options op = new Frm_Options();
            if (op.ShowDialog(this) == System.Windows.Forms.DialogResult.OK)
            {
                romStatus1.ShowSnapshot = Program.UserSettings.UseSnapInStatus;
                cntr_ImagesBrowser_snaps.StreatchByDefault = Program.UserSettings.forceStretchSnapshots;
                cntr_ImagesBrowser_covers.StreatchByDefault = Program.UserSettings.forceStretchCovers;
                cntr_ImagesBrowser_snaps2.StreatchByDefault = Program.UserSettings.forceStretchSnapshots;
                cntr_ImagesBrowser_covers2.StreatchByDefault = Program.UserSettings.forceStretchCovers;
            }
        }
        //DRAG AND DROP
        private void listView_roms_ItemDrag(object sender, ItemDragEventArgs e)
        {
            if (listView_roms.SelectedItems.Count == 0)
                return;
            if (tabControl3.SelectedIndex == 0)
            {
                if (treeView_consoles.SelectedNode == null)
                    return;

                DragedItems.Clear();
                if (treeView_consoles.SelectedNode.GetType() == typeof(TreeNode_Console))
                {
                    List<string> lisss = new List<string>();
                    foreach (ListViewItem_Rom IT in listView_roms.SelectedItems)
                    {
                        lisss.Add(IT.ROM.Path);
                        DragedItems.Add(IT);
                    }
                    DragOut = true;
                    DoDragDrop(new DataObject(DataFormats.FileDrop, lisss.ToArray()), DragDropEffects.Copy | DragDropEffects.Move);
                    DragOut = false;
                    foreach (ListViewItem_Rom ro in DragedItems)
                    {
                        if (!ro.ROM.IsURL & !File.Exists(ro.ROM.Path))
                        {
                            listView_roms.Items.Remove(ro);
                            ((TreeNode_Console)treeView_consoles.SelectedNode).Console.Roms.Remove(ro.ROM);
                            Save = true;
                        }
                    }
                }
            }
            else
            {
                if (treeView_playlists.SelectedNode == null)
                    return;

                List<string> lisss = new List<string>();
                foreach (ListViewItem_PlayListItem IT in listView_roms.SelectedItems)
                {
                    lisss.Add(IT.PlayListItem.Rom.Path);
                    DragedItems.Add(IT);
                }

                DragedItems.Clear();
                foreach (ListViewItem_PlayListItem IT in listView_roms.SelectedItems)
                    DragedItems.Add(IT);
                DragOut = false;
                DoDragDrop(new DataObject(DataFormats.FileDrop, lisss.ToArray()), DragDropEffects.Copy | DragDropEffects.Move);
            }
        }
        private void listView_roms_DragOver(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(typeof(ListViewItem_PlayListItem)) & DragedItems.Count > 0)
            {
                e.Effect = DragDropEffects.Move;
            }
            else if (e.Data.GetDataPresent(DataFormats.FileDrop))
            {
                if (!DragOut & (e.AllowedEffect & DragDropEffects.Copy) == DragDropEffects.Copy)
                {
                    e.Effect = DragDropEffects.Copy;
                }
                else
                {
                    e.Effect = DragDropEffects.None;
                }
            }
        }
        private void listView_roms_DragDrop(object sender, DragEventArgs e)
        {
            List<string> DragedFiles = new List<string>();
            if (tabControl3.SelectedIndex == 1 & DragedItems.Count > 0)
            {
                CanDrop = true;
                FilesDrag = false;
            }
            else
            {
                CanDrop = true;
                FilesDrag = true;
                DragedFiles.AddRange((string[])e.Data.GetData(DataFormats.FileDrop));
            }
            if (CanDrop)//Files drag
            {
                if (FilesDrag)
                {
                    CanDrop = false; FilesDrag = false;
                    if (tabControl3.SelectedIndex != 0)
                        return;
                    if (treeView_consoles.SelectedNode == null)
                        return;
                    if (treeView_consoles.SelectedNode.GetType() != typeof(TreeNode_Console))
                        return;
                    TreeNode_Console TR = (TreeNode_Console)treeView_consoles.SelectedNode;
                    DialogResult mess = MessageBox.Show("Do you want to replace the same roms in the list ?", "Replace rom files", MessageBoxButtons.YesNoCancel, MessageBoxIcon.Question);
                    bool Replace = (mess == DialogResult.Yes);
                    if (mess == DialogResult.Cancel)
                    { return; }
                    //Calculate folders files
                    List<string> DragedFolders = new List<string>();
                    for (int i = 0; i < DragedFiles.Count; i++)
                    {
                        if (Directory.Exists(DragedFiles[i]))
                        {
                            DragedFolders.Add(DragedFiles[i]);
                            DragedFiles.RemoveAt(i);
                            i = -1;
                        }
                    }
                    foreach (string folder in DragedFolders)
                    {
                        string[] nnn = Directory.GetFiles(folder, "*", SearchOption.AllDirectories);
                        DragedFiles.AddRange(nnn);
                    }
                    this.Enabled = false;
                    //add the roms if available
                    foreach (string R in DragedFiles)
                    {
                        foreach (string ext in TR.Console.Extensions)
                        {
                            if (ext.ToLower() == Path.GetExtension(R).ToLower())
                            {
                                Rom ro = new Rom();
                                ro.Name = Path.GetFileNameWithoutExtension(R);
                                ro.Path = R;
                                if (Path.GetDirectoryName(R).Length >= Program.StartUpDir.Length)
                                {
                                    if (Path.GetDirectoryName(R).Substring(0, Program.StartUpDir.Length) == Program.StartUpDir)
                                    {
                                        ro.Path = "." + R.Substring(Program.StartUpDir.Length);
                                    }
                                }
                                ro.RomSize = Utilities.GetFileSize(R);
                                if (Replace)
                                {
                                    for (int g = 0; g < TR.Console.Roms.Count; g++)
                                    {
                                        if (ro.Path == TR.Console.Roms[g].Path)
                                        {
                                            TR.Console.Roms.RemoveAt(g);
                                            listView_roms.Items.RemoveAt(g);
                                            g = -1;
                                        }
                                    }
                                }
                                TR.Console.Roms.Add(ro);
                                ListViewItem_Rom roooom = new ListViewItem_Rom();
                                roooom.ImageIndex = treeView_consoles.SelectedNode.Index;
                                roooom.BackColor = Color.LightYellow;
                                roooom.ROM = ro;
                                roooom.RefreshName();
                                listView_roms.Items.Add(roooom);
                                roooom.EnsureVisible();
                                break;
                            }
                        }
                    }
                    this.Enabled = true;
                    Save = true;
                }
            }
            DragOut = false;
        }
        private void listView_roms_MouseUp(object sender, MouseEventArgs e)
        {
            if (CanDrop)
            {
                CanDrop = false; FilesDrag = false;
                DestinationItem = (ListViewItem_PlayListItem)listView_roms.GetItemAt(e.X, e.Y);
                TreeNode_PlayList TR = (TreeNode_PlayList)treeView_playlists.SelectedNode;
                if (DestinationItem != null)
                {
                    //insert em
                    foreach (ListViewItem_PlayListItem DragedItem in DragedItems)
                    {
                        if (DragedItem != DestinationItem)
                        {
                            ListViewItem_PlayListItem NEW = new ListViewItem_PlayListItem();
                            NEW.PlayListItem = DragedItem.PlayListItem;
                            NEW.ImageIndex = DragedItem.ImageIndex;
                            NEW.Selected = true;
                            //Remove the orgenal
                            TR.PlayList.PlayListItems.Remove(DragedItem.PlayListItem);
                            listView_roms.Items.Remove(DragedItem);
                            //Insert    
                            NEW.BackColor = Color.LightYellow;
                            listView_roms.Items.Insert(DestinationItem.Index, NEW);
                            TR.PlayList.PlayListItems.Insert(DestinationItem.Index - 1, NEW.PlayListItem);
                        }
                    }
                }
                else//Add em
                {
                    foreach (ListViewItem_PlayListItem DragedItem in DragedItems)
                    {
                        ListViewItem_PlayListItem NEW = new ListViewItem_PlayListItem();
                        NEW.PlayListItem = DragedItem.PlayListItem;
                        NEW.ImageIndex = DragedItem.ImageIndex;
                        NEW.Selected = true;
                        //Remove the orgenal
                        TR.PlayList.PlayListItems.Remove(DragedItem.PlayListItem);
                        listView_roms.Items.Remove(DragedItem);
                        //add
                        NEW.BackColor = Color.LightYellow;
                        listView_roms.Items.Add(NEW);
                        TR.PlayList.PlayListItems.Add(NEW.PlayListItem);
                    }
                }
                Save = true;
            }
        }
        private void nameToolStripMenuItem_Click(object sender, EventArgs e)
        {
            _SortMode = SortMode.Name;
            _SortAlpha = !_SortAlpha;
            SortElements();
        }
        private void fileTypeToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            _SortMode = SortMode.FileType;
            _SortAlpha = !_SortAlpha;
            SortElements();
        }
        private void sizeToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            _SortMode = SortMode.Size;
            _SortAlpha = !_SortAlpha;
            SortElements();
        }
        private void pathToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            _SortMode = SortMode.Path;
            _SortAlpha = !_SortAlpha;
            SortElements();
        }
        private void playedTimesToolStripMenuItem_Click(object sender, EventArgs e)
        {
            _SortMode = SortMode.PlayedTime;
            _SortAlpha = !_SortAlpha;
            SortElements();
        }
        private void ratingToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            _SortMode = SortMode.Rating;
            _SortAlpha = !_SortAlpha;
            SortElements();
        }
        private void pdFviewer1_PDFchangeFileRequest(object sender, EventArgs e)
        {
            if (listView_roms.SelectedItems.Count == 1)
            {
                OpenFileDialog op = new OpenFileDialog();
                op.Filter = "(*.pdf)|*.pdf;*.PDF";
                if (listView_roms.SelectedItems[0].GetType() == typeof(ListViewItem_Rom))
                {
                    if (op.ShowDialog(this) == System.Windows.Forms.DialogResult.OK)
                    {
                        ((ListViewItem_Rom)listView_roms.SelectedItems[0]).ROM.PDFPath = op.FileName;
                        pdFviewer1.PDFPath = ((ListViewItem_Rom)listView_roms.SelectedItems[0]).ROM.PDFPath;
                        Save = true;
                    }
                }
                else
                {
                    if (op.ShowDialog(this) == System.Windows.Forms.DialogResult.OK)
                    {
                        ((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Rom.PDFPath = op.FileName;
                        pdFviewer1.PDFPath = ((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Rom.PDFPath;
                        Save = true;
                    }
                }
            }
            else
                MessageBox.Show("Please select one rom first");
        }
        private void pdFviewer1_PDFdeleteFileRequest(object sender, EventArgs e)
        {
            if (listView_roms.SelectedItems.Count == 1)
            {
                if (MessageBox.Show("Are you sure ?", "Delete pdf", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == System.Windows.Forms.DialogResult.Yes)
                {
                    if (listView_roms.SelectedItems[0].GetType() == typeof(ListViewItem_Rom))
                    {
                        ((ListViewItem_Rom)listView_roms.SelectedItems[0]).ROM.PDFPath = "";
                    }
                    else
                    {
                        ((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Rom.PDFPath = "";
                    }
                    pdFviewer1.PDFPath = "";
                    Save = true;
                }
            }
            else
                MessageBox.Show("Please select one rom first");
        }
        private void pdFviewer1_PDFrealDeleteFileRequest(object sender, EventArgs e)
        {
            if (listView_roms.SelectedItems.Count == 1)
            {
                if (MessageBox.Show("Are you sure ?\nTHIS IS REAL AND CAN'T BE UNDONE !!", "Delete pdf", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == System.Windows.Forms.DialogResult.Yes)
                {
                    if (listView_roms.SelectedItems[0].GetType() == typeof(ListViewItem_Rom))
                    {
                        try
                        {
                            File.Delete(((ListViewItem_Rom)listView_roms.SelectedItems[0]).ROM.PDFPath);
                            ((ListViewItem_Rom)listView_roms.SelectedItems[0]).ROM.PDFPath = "";
                            pdFviewer1.PDFPath = "";
                        }
                        catch { MessageBox.Show("Can't delete file !!?"); }
                    }
                    else
                    {
                        try
                        {
                            File.Delete(((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Rom.PDFPath);
                            ((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Rom.PDFPath = "";
                            pdFviewer1.PDFPath = "";
                        }
                        catch { MessageBox.Show("Can't delete file !!?"); }
                    }

                    Save = true;
                }
            }
            else
                MessageBox.Show("Please select one rom first");
        }
        private void romToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (tabControl3.SelectedIndex == 0)
            {
                if (treeView_consoles.SelectedNode == null)
                {
                    MessageBox.Show("You must select console to add rom");
                    return;
                }
                if (treeView_consoles.SelectedNode.GetType() != typeof(TreeNode_Console))
                {
                    MessageBox.Show("You must select console to add rom");
                    return;
                }
                Frm_AddRom addrom = new Frm_AddRom(((TreeNode_Console)treeView_consoles.SelectedNode).Console);
                if (addrom.ShowDialog(this) == System.Windows.Forms.DialogResult.OK)
                {
                    Clear();
                    _ActiveElement = ActiveElement.ConsoleOrCategory;
                    _SortAlpha = false;
                    _SortMode = SortMode.None; ApplySortChecks();
                    //1 refresh emulators
                    RefreshEmulatorFromConsole();
                    //2 refresh roms
                    RefreshRomsFromConsole();
                    Save = true;

                }
            }
            else
                MessageBox.Show("You must select console to add rom");
        }
        private void timer1_autominimize_Tick(object sender, EventArgs e)
        {
            foreach (Process process in Process.GetProcesses())
            {
                if (process.ProcessName == EmuProccess.ProcessName)
                    return;
            }
            timer1_autominimize.Stop();
            this.WindowState = FormWindowState.Normal;
        }
        private void searchInternetForSnapshotsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (listView_roms.SelectedItems.Count == 1)
            {
                string selectedPath = "";
                if (tabControl3.SelectedIndex == 0)
                {
                    if (treeView_consoles.SelectedNode.GetType() == typeof(TreeNode_Console))
                    {
                        if (((TreeNode_Console)treeView_consoles.SelectedNode).Console.ImageDirs == null)
                            ((TreeNode_Console)treeView_consoles.SelectedNode).Console.ImageDirs = new List<string>();
                        if (((TreeNode_Console)treeView_consoles.SelectedNode).Console.ImageDirs.Count > 0)
                            selectedPath = ((TreeNode_Console)treeView_consoles.SelectedNode).Console.ImageDirs[0];
                    }
                    else
                    {
                        if (((TreeNode_Console)treeView_consoles.SelectedNode.Parent).Console.ImageDirs == null)
                            ((TreeNode_Console)treeView_consoles.SelectedNode.Parent).Console.ImageDirs = new List<string>();
                        if (((TreeNode_Console)treeView_consoles.SelectedNode.Parent).Console.ImageDirs.Count > 0)
                            selectedPath = ((TreeNode_Console)treeView_consoles.SelectedNode.Parent).Console.ImageDirs[0];
                    }
                }
                else
                {
                    if (((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Console.ImageDirs == null)
                        ((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Console.ImageDirs = new List<string>();
                    if (((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Console.ImageDirs.Count > 0)
                        selectedPath = ((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Console.ImageDirs[0];
                }
                if (listView_roms.SelectedItems[0].GetType() == typeof(ListViewItem_Rom))
                {
                    string Send = ((ListViewItem_Rom)listView_roms.SelectedItems[0]).ROM.Name;
                    switch (Program.UserSettings.GoogleSearchMode)
                    {
                        case GoogleSearchMode.RomName_Console: Send += " " +
                            ((treeView_consoles.SelectedNode.GetType() == typeof(TreeNode_Console)) ?
                            ((TreeNode_Console)treeView_consoles.SelectedNode).Console.Name :
                            ((TreeNode_Console)treeView_consoles.SelectedNode.Parent).Console.Name);
                            break;
                        case GoogleSearchMode.RomName_Console_Emulator:
                            if (listView_emulators.SelectedItems.Count == 1)
                            {
                                Send += " " +
                    ((treeView_consoles.SelectedNode.GetType() == typeof(TreeNode_Console)) ?
                    ((TreeNode_Console)treeView_consoles.SelectedNode).Console.Name :
                    ((TreeNode_Console)treeView_consoles.SelectedNode.Parent).Console.Name) + " " +
                    ((ListViewItem_Emulator)listView_emulators.SelectedItems[0]).Emu.Name;
                            }
                            break;
                    }
                    Send += " " + Program.UserSettings.GoogleSearchPerfix;
                    Frm_ImageSearcher searcher = new Frm_ImageSearcher(Send, selectedPath);
                    searcher.OptionRequest += new EventHandler(searcher_OptionRequest);
                    if (searcher.ShowDialog(this) == System.Windows.Forms.DialogResult.OK)
                    {
                        selectedPath = searcher.FolderSelected;
                        Frm_DownloadFiles down = new Frm_DownloadFiles(searcher.FolderSelected, searcher.SelectedImageLinks.ToArray(), ((ListViewItem_Rom)listView_roms.SelectedItems[0]).ROM.Name);
                        if (down.ShowDialog(this) == System.Windows.Forms.DialogResult.OK)
                        {
                            if (MessageBox.Show("Clear snapshots collection of this rom first ?", "Clear snapshots", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == System.Windows.Forms.DialogResult.Yes)
                            {
                                ((ListViewItem_Rom)listView_roms.SelectedItems[0]).ROM.Snapshots.Clear();
                                ((ListViewItem_Rom)listView_roms.SelectedItems[0]).ROM.Snapshots = down.DownloadedPaths;
                                cntr_ImagesBrowser_snaps.Rom = ((ListViewItem_Rom)listView_roms.SelectedItems[0]).ROM;
                                cntr_ImagesBrowser_snaps2.Rom = ((ListViewItem_Rom)listView_roms.SelectedItems[0]).ROM;
                                Save = true;
                            }
                            else
                            {
                                foreach (string file in down.DownloadedPaths)
                                    ((ListViewItem_Rom)listView_roms.SelectedItems[0]).ROM.Snapshots.Add(file);
                                cntr_ImagesBrowser_snaps.Rom = ((ListViewItem_Rom)listView_roms.SelectedItems[0]).ROM;
                                cntr_ImagesBrowser_snaps2.Rom = ((ListViewItem_Rom)listView_roms.SelectedItems[0]).ROM;
                                Save = true;
                            }
                        }
                    }
                }
                else
                {
                    if (((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Console.ImageDirs.Count > 0)
                        selectedPath = ((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Console.ImageDirs[0];
                    string Send = ((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Rom.Name;
                    switch (Program.UserSettings.GoogleSearchMode)
                    {
                        case GoogleSearchMode.RomName_Console: Send += " " +
                            ((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Console.Name;
                            break;
                        case GoogleSearchMode.RomName_Console_Emulator:
                            if (listView_emulators.SelectedItems.Count == 1)
                            {
                                Send += " " + ((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Console.Name
                                    + " " + ((ListViewItem_Emulator)listView_emulators.SelectedItems[0]).Emu.Name;
                            }
                            break;
                    } Send += " " + Program.UserSettings.GoogleSearchPerfix;
                    Frm_ImageSearcher searcher = new Frm_ImageSearcher(Send, selectedPath);
                    searcher.OptionRequest += new EventHandler(searcher_OptionRequest);
                    if (searcher.ShowDialog(this) == System.Windows.Forms.DialogResult.OK)
                    {

                        selectedPath = searcher.FolderSelected;
                        Frm_DownloadFiles down = new Frm_DownloadFiles(searcher.FolderSelected, searcher.SelectedImageLinks.ToArray(), ((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Rom.Name);
                        if (down.ShowDialog(this) == System.Windows.Forms.DialogResult.OK)
                        {
                            if (MessageBox.Show("Clear snapshots collection of this rom first ?", "Clear snapshots", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == System.Windows.Forms.DialogResult.Yes)
                            {
                                ((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Rom.Snapshots.Clear();
                                ((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Rom.Snapshots = down.DownloadedPaths;
                                cntr_ImagesBrowser_snaps.Rom = ((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Rom;
                                cntr_ImagesBrowser_snaps2.Rom = ((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Rom;
                                Save = true;
                            }
                            else
                            {
                                foreach (string file in down.DownloadedPaths)
                                    ((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Rom.Snapshots.Add(file);
                                cntr_ImagesBrowser_snaps.Rom = ((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Rom;
                                cntr_ImagesBrowser_snaps2.Rom = ((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Rom; Save = true;
                            }
                        }
                    }
                }
                if (tabControl3.SelectedIndex == 0)
                {
                    if (treeView_consoles.SelectedNode.GetType() == typeof(TreeNode_Console))
                    {
                        foreach (string dir in ((TreeNode_Console)treeView_consoles.SelectedNode).Console.ImageDirs)
                        {
                            if (dir == selectedPath)
                                return;
                        }
                        ((TreeNode_Console)treeView_consoles.SelectedNode).Console.ImageDirs.Insert(0, selectedPath);
                    }
                    else
                    {
                        foreach (string dir in ((TreeNode_Console)treeView_consoles.SelectedNode.Parent).Console.ImageDirs)
                        {
                            if (dir == selectedPath)
                                return;
                        }
                        ((TreeNode_Console)treeView_consoles.SelectedNode.Parent).Console.ImageDirs.Insert(0, selectedPath);
                    }
                }
                else
                {

                    foreach (string dir in ((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Console.ImageDirs)
                    {
                        if (dir == selectedPath)
                            return;
                    }
                    ((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Console.ImageDirs.Insert(0, selectedPath);
                }
            }
        }
        void searcher_OptionRequest(object sender, EventArgs e)
        {
            Frm_Options op = new Frm_Options();
            op.tabControl1.SelectedIndex = 2;
            if (op.ShowDialog(this) == System.Windows.Forms.DialogResult.OK)
            {
                romStatus1.ShowSnapshot = Program.UserSettings.UseSnapInStatus;
                cntr_ImagesBrowser_snaps.StreatchByDefault = Program.UserSettings.forceStretchSnapshots;
                cntr_ImagesBrowser_covers.StreatchByDefault = Program.UserSettings.forceStretchCovers;
                cntr_ImagesBrowser_snaps2.StreatchByDefault = Program.UserSettings.forceStretchSnapshots;
                cntr_ImagesBrowser_covers2.StreatchByDefault = Program.UserSettings.forceStretchCovers;
            }
        }
        private void searchInternetForCoversToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (listView_roms.SelectedItems.Count == 1)
            {
                string selectedPath = "";
                if (tabControl3.SelectedIndex == 0)
                {
                    if (treeView_consoles.SelectedNode.GetType() == typeof(TreeNode_Console))
                    {
                        if (((TreeNode_Console)treeView_consoles.SelectedNode).Console.CoverDirs == null)
                            ((TreeNode_Console)treeView_consoles.SelectedNode).Console.CoverDirs = new List<string>();
                        if (((TreeNode_Console)treeView_consoles.SelectedNode).Console.CoverDirs.Count > 0)
                            selectedPath = ((TreeNode_Console)treeView_consoles.SelectedNode).Console.CoverDirs[0];
                    }
                    else
                    {
                        if (((TreeNode_Console)treeView_consoles.SelectedNode.Parent).Console.CoverDirs == null)
                            ((TreeNode_Console)treeView_consoles.SelectedNode.Parent).Console.CoverDirs = new List<string>();
                        if (((TreeNode_Console)treeView_consoles.SelectedNode.Parent).Console.CoverDirs.Count > 0)
                            selectedPath = ((TreeNode_Console)treeView_consoles.SelectedNode.Parent).Console.CoverDirs[0];
                    }
                }
                else
                {
                    if (((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Console.CoverDirs == null)
                        ((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Console.CoverDirs = new List<string>();
                    if (((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Console.CoverDirs.Count > 0)
                        selectedPath = ((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Console.CoverDirs[0];
                }
                if (listView_roms.SelectedItems[0].GetType() == typeof(ListViewItem_Rom))
                {
                    string Send = ((ListViewItem_Rom)listView_roms.SelectedItems[0]).ROM.Name;
                    switch (Program.UserSettings.GoogleSearchMode)
                    {
                        case GoogleSearchMode.RomName_Console: Send += " " +
                            ((treeView_consoles.SelectedNode.GetType() == typeof(TreeNode_Console)) ?
                            ((TreeNode_Console)treeView_consoles.SelectedNode).Console.Name :
                            ((TreeNode_Console)treeView_consoles.SelectedNode.Parent).Console.Name);
                            break;
                        case GoogleSearchMode.RomName_Console_Emulator:
                            if (listView_emulators.SelectedItems.Count == 1)
                            {
                                Send += " " +
                    ((treeView_consoles.SelectedNode.GetType() == typeof(TreeNode_Console)) ?
                    ((TreeNode_Console)treeView_consoles.SelectedNode).Console.Name :
                    ((TreeNode_Console)treeView_consoles.SelectedNode.Parent).Console.Name) + " " +
                    ((ListViewItem_Emulator)listView_emulators.SelectedItems[0]).Emu.Name;
                            }
                            break;
                    } Send += " " + Program.UserSettings.GoogleSearchPerfix;
                    Frm_ImageSearcher searcher = new Frm_ImageSearcher(Send, selectedPath);
                    searcher.OptionRequest += new EventHandler(searcher_OptionRequest);
                    if (searcher.ShowDialog(this) == System.Windows.Forms.DialogResult.OK)
                    {
                        selectedPath = searcher.FolderSelected;
                        Frm_DownloadFiles down = new Frm_DownloadFiles(searcher.FolderSelected, searcher.SelectedImageLinks.ToArray(), ((ListViewItem_Rom)listView_roms.SelectedItems[0]).ROM.Name);
                        if (down.ShowDialog(this) == System.Windows.Forms.DialogResult.OK)
                        {
                            if (MessageBox.Show("Clear covers collection of this rom first ?", "Clear covers", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == System.Windows.Forms.DialogResult.Yes)
                            {
                                ((ListViewItem_Rom)listView_roms.SelectedItems[0]).ROM.Covers.Clear();
                                ((ListViewItem_Rom)listView_roms.SelectedItems[0]).ROM.Covers = down.DownloadedPaths;
                                cntr_ImagesBrowser_covers.Rom = ((ListViewItem_Rom)listView_roms.SelectedItems[0]).ROM;
                                cntr_ImagesBrowser_covers2.Rom = ((ListViewItem_Rom)listView_roms.SelectedItems[0]).ROM;
                                Save = true;
                            }
                            else
                            {
                                foreach (string file in down.DownloadedPaths)
                                    ((ListViewItem_Rom)listView_roms.SelectedItems[0]).ROM.Covers.Add(file);
                                cntr_ImagesBrowser_covers.Rom = ((ListViewItem_Rom)listView_roms.SelectedItems[0]).ROM;
                                cntr_ImagesBrowser_covers2.Rom = ((ListViewItem_Rom)listView_roms.SelectedItems[0]).ROM;
                                Save = true;
                            }
                        }
                    }
                }
                else
                {
                    if (((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Console.CoverDirs.Count > 0)
                        selectedPath = ((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Console.CoverDirs[0];
                    string Send = ((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Rom.Name;
                    switch (Program.UserSettings.GoogleSearchMode)
                    {
                        case GoogleSearchMode.RomName_Console: Send += " " +
                            ((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Console.Name;
                            break;
                        case GoogleSearchMode.RomName_Console_Emulator:
                            if (listView_emulators.SelectedItems.Count == 1)
                            {
                                Send += " " + ((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Console.Name
                                    + " " + ((ListViewItem_Emulator)listView_emulators.SelectedItems[0]).Emu.Name;
                            }
                            break;
                    } Send += " " + Program.UserSettings.GoogleSearchPerfix;
                    Frm_ImageSearcher searcher = new Frm_ImageSearcher(Send, selectedPath);
                    searcher.OptionRequest += new EventHandler(searcher_OptionRequest);
                    if (searcher.ShowDialog(this) == System.Windows.Forms.DialogResult.OK)
                    {
                        selectedPath = searcher.FolderSelected;
                        Frm_DownloadFiles down = new Frm_DownloadFiles(searcher.FolderSelected, searcher.SelectedImageLinks.ToArray(), ((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Rom.Name);
                        if (down.ShowDialog(this) == System.Windows.Forms.DialogResult.OK)
                        {
                            if (MessageBox.Show("Clear covers collection of this rom first ?", "Clear covers", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == System.Windows.Forms.DialogResult.Yes)
                            {
                                ((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Rom.Covers.Clear();
                                ((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Rom.Covers = down.DownloadedPaths;
                                cntr_ImagesBrowser_covers.Rom = ((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Rom;
                                cntr_ImagesBrowser_covers.Rom = ((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Rom; Save = true;
                            }
                            else
                            {
                                foreach (string file in down.DownloadedPaths)
                                    ((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Rom.Covers.Add(file);
                                cntr_ImagesBrowser_covers.Rom = ((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Rom;
                                cntr_ImagesBrowser_covers.Rom = ((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Rom;
                                Save = true;
                            }
                        }
                    }
                }
                if (tabControl3.SelectedIndex == 0)
                {
                    if (treeView_consoles.SelectedNode.GetType() == typeof(TreeNode_Console))
                    {
                        foreach (string dir in ((TreeNode_Console)treeView_consoles.SelectedNode).Console.CoverDirs)
                        {
                            if (dir == selectedPath)
                                return;
                        }
                        ((TreeNode_Console)treeView_consoles.SelectedNode).Console.CoverDirs.Insert(0, selectedPath);
                    }
                    else
                    {
                        foreach (string dir in ((TreeNode_Console)treeView_consoles.SelectedNode.Parent).Console.CoverDirs)
                        {
                            if (dir == selectedPath)
                                return;
                        }
                        ((TreeNode_Console)treeView_consoles.SelectedNode.Parent).Console.CoverDirs.Insert(0, selectedPath);
                    }
                }
                else
                {
                    foreach (string dir in ((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Console.CoverDirs)
                    {
                        if (dir == selectedPath)
                            return;
                    }
                    ((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Console.CoverDirs.Insert(0, selectedPath);
                }
            }
        }
        private void infoEditor1_info_Enter(object sender, EventArgs e)
        {
            _ActiveElement = ActiveElement.Info;
        }
        private void infoEditor1_history_Enter(object sender, EventArgs e)
        {
            _ActiveElement = ActiveElement.History;
        }
        private void treeView1_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            if (treeView1.GetNodeAt(e.Location) != null)
            {
                treeView1.SelectedNode = treeView1.GetNodeAt(e.Location);
            }
            try
            {
                Process.Start(treeView1.SelectedNode.Text);
            }
            catch { }
        }
        private void toolStripButton7_EnableCommandlines_CheckedChanged(object sender, EventArgs e)
        {
            useCommandlinesToolStripMenuItem.Checked = toolStripButton7_EnableCommandlines.Checked;
        }
        private void useCommandlinesToolStripMenuItem_CheckedChanged(object sender, EventArgs e)
        {
            toolStripButton7_EnableCommandlines.Checked = useCommandlinesToolStripMenuItem.Checked;
        }
        private void useEmulatorToolStripMenuItem_CheckedChanged(object sender, EventArgs e)
        {
            toolStripButton8_EnableEmulator.Checked = useEmulatorToolStripMenuItem.Checked;
        }
        private void tabControl1_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (tabControl_up_right.SelectedIndex == 3)
            {
                _ActiveElement = ActiveElement.RomOrPlaylistItem;
                countDown = 2;
                timer1_AfterSelectRoms.Start();
            }
        }
        private void locateOnDiskToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (listView_roms.SelectedItems.Count == 1)
            {
                if (listView_roms.SelectedItems[0].GetType() == typeof(ListViewItem_Rom))
                {
                    try
                    {
                        Process.Start("explorer.exe", @"/select, " + Utilities.GetFullPath(((ListViewItem_Rom)listView_roms.SelectedItems[0]).ROM.Path));
                    }
                    catch { }
                }
                else
                {
                    try
                    {
                        Process.Start("explorer.exe", @"/select, " + Utilities.GetFullPath(((ListViewItem_PlayListItem)listView_roms.SelectedItems[0]).PlayListItem.Rom.Path));
                    }
                    catch { }
                }
            }
        }
        private void changeextensionToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (listView_roms.SelectedItems.Count == 0)
            {
                MessageBox.Show("Please select the roms you want to change extension for");
                return;
            }
            if (tabControl3.SelectedIndex != 0)
            {
                MessageBox.Show("you must select the console that these roms belong to");
                return;
            }
            if (treeView_consoles.SelectedNode == null)
            {
                MessageBox.Show("you must select the console that these roms belong to");
                return;
            }
            string NewExtension = "";
            Frm_EnterName nam = new Frm_EnterName("Enter the new extension", ".exe");
            if (nam.ShowDialog(this) == System.Windows.Forms.DialogResult.OK)
            {
                NewExtension = nam.NameEntered;
                if (NewExtension.Substring(0, 1) != ".")
                    NewExtension = "." + NewExtension;
                if (treeView_consoles.SelectedNode.GetType() == typeof(TreeNode_Console))
                {
                    bool found = false;
                    foreach (string ex in ((TreeNode_Console)treeView_consoles.SelectedNode).Console.Extensions)
                    {
                        if (ex.ToLower() == NewExtension.ToLower())
                        {
                            found = true;
                            break;
                        }
                    }
                    if (!found)
                    {
                        if (MessageBox.Show("This extension isn't exist in the console (" + ((TreeNode_Console)treeView_consoles.SelectedNode).Console.Name + "), do you want to add this extension to this console ?", "Extension not found", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == System.Windows.Forms.DialogResult.Yes)
                        {
                            ((TreeNode_Console)treeView_consoles.SelectedNode).Console.Extensions.Add(NewExtension);
                        }
                        else
                            return;
                    }
                }
                else
                {
                    bool found = false;
                    foreach (string ex in ((TreeNode_Console)treeView_consoles.SelectedNode.Parent).Console.Extensions)
                    {
                        if (ex.ToLower() == NewExtension.ToLower())
                        {
                            found = true;
                            break;
                        }
                    }
                    if (!found)
                    {
                        if (MessageBox.Show("This extension isn't exist in the console (" + ((TreeNode_Console)treeView_consoles.SelectedNode.Parent).Console.Name + "), do you want to add this extension to this console ?", "Extension not found", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == System.Windows.Forms.DialogResult.Yes)
                        {
                            ((TreeNode_Console)treeView_consoles.SelectedNode.Parent).Console.Extensions.Add(NewExtension);
                        }
                        else
                            return;
                    }
                }

            }
            else
                return;
            foreach (ListViewItem_Rom item in listView_roms.SelectedItems)
            {
                try
                {
                    string fol = Path.GetDirectoryName(Utilities.GetFullPath(item.ROM.Path));
                    if (fol == "")
                        fol = Path.GetPathRoot(Utilities.GetFullPath(item.ROM.Path));

                    string Orgenal = Utilities.GetFullPath(item.ROM.Path);
                    string Neww = fol + "\\" + Path.GetFileNameWithoutExtension(Utilities.GetFullPath(item.ROM.Path)) + NewExtension;
                    File.Copy(Orgenal, Neww);
                    FileInfo inf = new FileInfo(Orgenal);
                    inf.IsReadOnly = false;
                    File.Delete(Orgenal);

                    item.ROM.Path = Neww;
                    item.RefreshName();
                }
                catch { }
            }
            Save = true;
        }
        private void findToolStripMenuItem_Click_1(object sender, EventArgs e)
        {
            if (listView_roms.Items.Count == 0)
            {
                MessageBox.Show("No rom found to search");
                return;
            }
            List<ListViewItem> items = new List<ListViewItem>();
            foreach (ListViewItem item in listView_roms.Items)
                items.Add(item);
            if (findForm == null)
            {
                findForm = new Frm_Search(ComboBox1_find.Text, items);
                findForm.FindNextRequest += new EventHandler(findForm_FindNextRequest);
                findForm.Show(this); ItemSearched = false; findForm.FindNext();
            }
            else if (!findForm.Visible)
            {
                findForm = new Frm_Search(ComboBox1_find.Text, items);
                findForm.FindNextRequest += new EventHandler(findForm_FindNextRequest);
                findForm.Show(this); ItemSearched = false; findForm.FindNext();
            }
            else
            {
                findForm.items = items;
                findForm.itemSearched = false; findForm.FindNext();
            }
        }
        void findForm_FindNextRequest(object sender, EventArgs e)
        {
            if (findForm.NextResultIndex >= 0)
            {
                for (int j = 0; j < listView_roms.Items.Count; j++)
                { listView_roms.Items[j].Selected = false; }
                listView_roms.Items[findForm.NextResultIndex].Selected = true;
                listView_roms.Items[findForm.NextResultIndex].EnsureVisible();
            }
        }
        private void aboutEmulatorsOrganizerToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Frm_About about = new Frm_About(Program.Version);
            about.ShowDialog(this);
        }
        private void contentToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Help.ShowHelp(this, Application.StartupPath + "//Help.chm", HelpNavigator.TableOfContents);
        }
        private void indexToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Help.ShowHelp(this, Application.StartupPath + "//Help.chm", HelpNavigator.Index);
        }
        private void changeIconToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            if (listView_roms.SelectedItems.Count == 0)
            {
                MessageBox.Show("Please select roms first");
                return;
            }
            OpenFileDialog Op = new OpenFileDialog();
            Op.Title = "Browse for an icon";
            Op.Filter = "Icon (*.jpg;*.png;*.bmp;*.gif;*.jpeg;*.tiff;*.tga;*.ico;*.exe)|*.jpg;*.png;*.bmp;*.gif;*.jpeg;*.tiff;*.tga;*.ico;*.exe";
            Op.Multiselect = false;
            if (Op.ShowDialog(this) == DialogResult.OK)
            {
                Image image;
                if (Path.GetExtension(Op.FileName).ToLower() == ".exe" | Path.GetExtension(Op.FileName).ToLower() == ".ico")
                { image = Icon.ExtractAssociatedIcon(Op.FileName).ToBitmap(); }
                else { image = Image.FromFile(Op.FileName); }
                foreach (ListViewItem_PlayListItem pl in listView_roms.SelectedItems)
                {
                    pl.PlayListItem.Icon = image;
                }
                Save = true;
            }
        }
        private void ResetIconMenuItem1_Click(object sender, EventArgs e)
        {
            if (listView_roms.SelectedItems.Count == 0)
            {
                MessageBox.Show("Please select roms first");
                return;
            }
            foreach (ListViewItem_PlayListItem pl in listView_roms.SelectedItems)
            {
                pl.PlayListItem.Icon = null;
            }
            Save = true;
        }
        private void yearToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Program.UserSettings.Column_year_visible = yearToolStripMenuItem.Checked;
            RefreshColumns();
        }
        private void consoleWizardToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Frm_AddConsoleWizard add = new Frm_AddConsoleWizard(Program.Project);
            if (add.ShowDialog(this) == System.Windows.Forms.DialogResult.OK)
            {
                Program.Project.Consoles.Add(add.Console);
                RefreshConsoles();
                Save = true;
            }
        }
        private void detectVideosToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (tabControl3.SelectedIndex == 0)
            {
                if (treeView_consoles.SelectedNode == null)
                {
                    MessageBox.Show("You must select console to detect videos.");
                    return;
                }
                if (treeView_consoles.SelectedNode.GetType() == typeof(TreeNode_Category))
                {
                    MessageBox.Show("You must select console to detect videos.");
                    return;
                }
                Frm_DetectVideos detect = new Frm_DetectVideos(((TreeNode_Console)treeView_consoles.SelectedNode).Console);
                if (detect.ShowDialog(this) == System.Windows.Forms.DialogResult.OK)
                {
                    Save = true;
                    listView_roms.Refresh();
                }
            }
            else
            {
                MessageBox.Show("You must select console to detect videos.");
            }
        }
        private void videoToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Program.UserSettings.Column_video_visible = videoToolStripMenuItem.Checked;
            RefreshColumns();
        }
        private void soundPlayer1_SaveRequired(object sender, EventArgs e)
        {
            Save = true;
        }
        private void soundsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Program.UserSettings.Column_sounds_visible = soundsToolStripMenuItem.Checked;
            RefreshColumns();
        }
        private void detectSoundToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (tabControl3.SelectedIndex == 0)
            {
                if (treeView_consoles.SelectedNode == null)
                {
                    MessageBox.Show("You must select console to detect sounds.");
                    return;
                }
                if (treeView_consoles.SelectedNode.GetType() == typeof(TreeNode_Category))
                {
                    MessageBox.Show("You must select console to detect sounds.");
                    return;
                }
                Frm_DetectSounds detect = new Frm_DetectSounds(((TreeNode_Console)treeView_consoles.SelectedNode).Console);
                if (detect.ShowDialog(this) == System.Windows.Forms.DialogResult.OK)
                {
                    Save = true;
                    listView_roms.Refresh();
                }
            }
            else
            {
                MessageBox.Show("You must select console to detect sounds.");
            }
        }
        private void videoPlayer1_SaveRequired(object sender, EventArgs e)
        {
            Save = true;
        }
        private void getNamesFromDatabaseToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (tabControl3.SelectedIndex == 0)
            {
                if (treeView_consoles.SelectedNode == null)
                {
                    MessageBox.Show("You must select console");
                    return;
                }
                if (treeView_consoles.SelectedNode.GetType() == typeof(TreeNode_Category))
                {
                    MessageBox.Show("You must select console");
                    return;
                }
                Frm_GetNamesFromDatabase frm = new Frm_GetNamesFromDatabase(((TreeNode_Console)treeView_consoles.SelectedNode).Console);
                if (frm.ShowDialog(this) == System.Windows.Forms.DialogResult.OK)
                {
                    Save = true;
                    Clear();
                    RefreshEmulatorFromConsole();
                    RefreshRomsFromConsole();
                    //enable menu items
                    toolStripSeparator28.Visible = addConsoleToolStripMenuItem.Visible = addCategoryToolStripMenuItem.Visible =
                        toolStripSeparator8.Visible = romToolStripMenuItem2.Visible = addRomsToolStripMenuItem.Visible =
                        toolStripSeparator27.Visible = detectSnapshotsToolStripMenuItem1.Visible = detectCoversToolStripMenuItem1.Visible =
                   detectVideosToolStripMenuItem1.Visible = detectManualsToolStripMenuItem1.Visible = toolStripSeparator9.Visible =
              detectSoundsToolStripMenuItem.Visible = propertiesToolStripMenuItem.Visible = true;
                }
            }
            else
            {
                MessageBox.Show("You must select console");
            }

        }
        private void sortByNameToolStripMenuItem_Click(object sender, EventArgs e)
        {
            string selectedName = "";
            if (treeView_consoles.SelectedNode != null)
                selectedName = treeView_consoles.SelectedNode.Text;
            Program.Project.Consoles.Sort(new ConsoleComparer(consoleATOZ));
            RefreshConsoles();
            foreach (TreeNode_Console con in treeView_consoles.Nodes)
            {
                if (con.Name == selectedName)
                    treeView_consoles.SelectedNode = con;
            }
            consoleATOZ = !consoleATOZ;
            Save = true;
        }
        private void sortByNameToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            string selectedName = "";
            if (treeView_playlists.SelectedNode != null)
                selectedName = treeView_playlists.SelectedNode.Text;
            Program.Project.PlayLists.Sort(new PlaylistComparer(consoleATOZ));
            RefreshPlaylists();
            foreach (TreeNode_PlayList con in treeView_playlists.Nodes)
            {
                if (con.Name == selectedName)
                    treeView_playlists.SelectedNode = con;
            }
            playlistATOZ = !playlistATOZ;
            Save = true;
        }
        private void exportConsoleAsDatabaeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (tabControl3.SelectedIndex == 0)
            {
                if (treeView_consoles.SelectedNode == null)
                {
                    MessageBox.Show("You can only export console !!");
                    return;
                }
                if (treeView_consoles.SelectedNode.GetType() == typeof(TreeNode_Category))
                {
                    MessageBox.Show("You can only export console !!");
                    return;
                }
                Frm_ExportToDatabase frm = new Frm_ExportToDatabase(((TreeNode_Console)treeView_consoles.SelectedNode).Console);
                frm.ShowDialog(this);
            }
            else
            {
                MessageBox.Show("You must select console to detect sounds.");
            }
        }
        private void commandlinesEditor1_Load(object sender, EventArgs e)
        {

        }
        private void listBox_configFiles_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (listBox_configFiles.SelectedIndex < 0)
                return;
            cnt_ConfigurationFileEditor1.ConfigFile = (ConfigurationFile)listBox_configFiles.SelectedItem;
        }
        private void toolStripButton15_Click(object sender, EventArgs e)
        {
            if (listBox_configFiles.SelectedIndex < 0)
                return;
            cnt_ConfigurationFileEditor1.SaveConfigFile();
        }
        private void toolStripButton16_Click(object sender, EventArgs e)
        {
            if (listBox_configFiles.SelectedIndex < 0)
                return;
            cnt_ConfigurationFileEditor1.ReloadConfig();
        }
        private void listBox_configFiles_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            if (listBox_configFiles.SelectedIndex < 0)
                return;
            cnt_ConfigurationFileEditor1.ShowInWindowsEditor();
        }
        private void toolStripButton17_Click(object sender, EventArgs e)
        {
            if (listBox_configFiles.SelectedIndex < 0)
                return;
            cnt_ConfigurationFileEditor1.SaveAs();
        }
    }
}